/*
 *  Copyright (c) Texas Instruments Incorporated 2018
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 *  \file cpsw_appmemutils.h
 *
 *  \brief CPSW DMA memutils header file.
 */

#ifndef CPSW_APPMEMUTILS_H_
#define CPSW_APPMEMUTILS_H_

/* ========================================================================== */
/*                             Include Files                                  */
/* ========================================================================== */

#ifdef __cplusplus
extern "C" {
#endif

/* ========================================================================== */
/*                           Macros & Typedefs                                */
/* ========================================================================== */

/* ========================================================================== */
/*                         Structure Declarations                             */
/* ========================================================================== */

/* ========================================================================== */
/*                          Function Declarations                             */
/* ========================================================================== */

/*! Ring memory allocation function  */
uint8_t *CpswAppMemUtils_allocRingMemFxn(void *appPriv,
                                         uint32_t numRingEle,
                                         uint32_t alignSize);

/*! Ring memory free function  */
void CpswAppMemUtils_freeRingMemFxn(void *appPriv,
                                    void *ringMemPtr,
                                    uint32_t numRingEle);

/*! DMA packet allocation function  */
CpswDma_DmaDesc *CpswAppMemUtils_allocDmaDescFxn(void *appPriv,
                                                 uint32_t alignSize);

/*! DMA packet free function  */
void CpswAppMemUtils_freeDmaDescFxn(void *appPriv,
                                    CpswDma_DmaDesc *dmaDescPtr);

/*! Ethernet packet allocation function  */
CpswDma_PktInfo *CpswAppMemUtils_allocEthPktFxn(void *appPriv,
                                                uint32_t pktSize,
                                                uint32_t alignSize);

/*! Ethernet packet free function  */
void CpswAppMemUtils_freeEthPktFxn(CpswDma_PktInfo *pPktInfo);

/*! Initialize Cpsw memutils module
 * Note - This function should be called after Cpsw is opened as it uses CpswUtils_Q
 * functions */
int32_t CpswAppMemUtils_init(void);

/*! Mem utils deinit  */
void CpswAppMemUtils_deInit(void);

/* ========================================================================== */
/*                       Static Function Definitions                          */
/* ========================================================================== */

/* None */

#ifdef __cplusplus
}
#endif

#endif /* CPSW_APPMEMUTILS_H_ */
