/*
 *  Copyright (c) Texas Instruments Incorporated 2018
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*!
 * \file     cpswapp_soc.c
 *
 * \brief    This file contains the CPSW driver SOC related configuration
 *           functions.
 */

/* ========================================================================== */
/*                             Include Files                                  */
/* ========================================================================== */

#include <ti/drv/cpsw/cpsw.h>
#include <ti/drv/cpsw/examples/cpsw_apputils/inc/cpsw_apputils.h>
#include <ti/drv/cpsw/examples/cpsw_apputils/inc/cpsw_appboardutils.h>
#include <ti/drv/cpsw/examples/cpsw_apputils/inc/cpsw_appsoc.h>
#include <ti/drv/ipc/ipc.h>

/* ========================================================================== */
/*                           Macros & Typedefs                                */
/* ========================================================================== */

/* None */

/* ========================================================================== */
/*                         Structure Declarations                             */
/* ========================================================================== */

/* None */

/* ========================================================================== */
/*                          Function Declarations                             */
/* ========================================================================== */

/* None */

/* ========================================================================== */
/*                            Global Variables                                */
/* ========================================================================== */

/* None */

/* ========================================================================== */
/*                          Function Definitions                              */
/* ========================================================================== */

uint32_t CpswAppSoc_getCoreId(void)
{
    uint32_t coreId;

#if defined(BUILD_MPU1_0)
    coreId = IPC_MPU1_0;
#elif defined(BUILD_MCU2_0)
    coreId = IPC_MCU2_0;
#elif defined(BUILD_MCU2_1)
    coreId = IPC_MCU2_1;
#elif defined(BUILD_MCU1_0)
    coreId = IPC_MCU1_0;
#elif defined(BUILD_MCU1_1)
    coreId = IPC_MCU1_1;
#elif defined(BUILD_MCU3_0) && defined (SOC_J721E)
    coreId = IPC_MCU3_0;
#elif defined(BUILD_MCU3_1) && defined (SOC_J721E)
    coreId = IPC_MCU3_1;
#elif defined(BUILD_C7X_1) && defined (SOC_J721E)
    coreId = IPC_C7X_1;
#elif defined(BUILD_C66X_1) && defined (SOC_J721E)
    coreId = IPC_C66X_1;
#elif defined(BUILD_C66X_2) && defined (SOC_J721E)
    coreId = IPC_C66X_2;
#else
#error "Unsupported core id"
#endif

    return(coreId);
}

int32_t CpswAppSoc_getMacAddrList(Cpsw_Type cpswType,
                                  uint8_t macAddr[][CPSW_MAC_ADDR_LEN],
                                  uint32_t maxMacEntries,
                                  uint32_t *pAvailMacEntries)
{
    int32_t status = CPSW_SOK;

    switch (cpswType)
    {
        case CPSW_2G:
        {
            if (maxMacEntries >= 1)
            {
                CpswSoc_getEFusedMacAddr(macAddr[0]);
                CpswAppUtils_assert(pAvailMacEntries != NULL);
                *pAvailMacEntries = 1;
            }
            else
            {
                status = CPSW_EINVALID_PARAMS;
            }

            break;
        }

        case CPSW_5G:
        case CPSW_9G:
        {
            CpswAppBoardUtils_getMacAddrList(macAddr, maxMacEntries, pAvailMacEntries);
            break;
        }

        default:
            status = CPSW_EINVALID_PARAMS;
    }

    return status;
}
