/*
 *  Copyright (c) Texas Instruments Incorporated 2019
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*!
 * \file cpsw_appboardutils.h
 *
 * \brief Board utilities header file.
 */

#ifndef CPSW_APPBOARDUTILS_H_
#define CPSW_APPBOARDUTILS_H_

/* ========================================================================== */
/*                             Include Files                                  */
/* ========================================================================== */

#ifdef __cplusplus
extern "C" {
#endif

/* ========================================================================== */
/*                                 Macros                                     */
/* ========================================================================== */

/* None */

/* ========================================================================== */
/*                         Structures and Enums                               */
/* ========================================================================== */

/* None */

/* ========================================================================== */
/*                         Global Variables Declarations                      */
/* ========================================================================== */

/* None */

/* ========================================================================== */
/*                          Function Declarations                             */
/* ========================================================================== */

void CpswAppBoardUtils_init(void);

/*!
 * \brief EthFw board init function which does limited configuration.
 */
void CpswAppBoardUtils_initEthFw(void);

void CpswAppBoardUtils_deInit(void);

uint32_t CpswAppBoardUtils_getPhyAddr(Cpsw_Type cpswType,
                                      Cpsw_MacPort portNum);

void CpswAppBoardUtils_setPhyConfig(Cpsw_Type cpswType,
                                    Cpsw_MacPort portNum,
                                    CpswMacPort_Config *macConfig,
                                    CpswMacPort_Interface *interface,
                                    CpswPhy_Config *phyConfig);

void CpswAppBoardUtils_setPhyConfigSgmii(Cpsw_MacPort portNum,
                                         CpswMacPort_Config *macConfig,
                                         CpswMacPort_Interface *interface,
                                         CpswPhy_Config *phyConfig);

void CpswAppBoardUtils_setPhyConfigQsgmii(Cpsw_Type cpswType,
                                          Cpsw_MacPort portNum,
                                          CpswMacPort_Config *macConfig,
                                          CpswMacPort_Interface *interface,
                                          CpswPhy_Config *phyConfig);

void CpswAppBoardUtils_setPhyConfigRmii(Cpsw_MacPort portNum,
                                        CpswMacPort_Config *macConfig,
                                        CpswMacPort_Interface *interface,
                                        CpswPhy_Config *phyConfig);

void CpswAppBoardUtils_setPhyConfigRgmii(Cpsw_Type cpswType,
                                         Cpsw_MacPort portNum,
                                         CpswMacPort_Config *macConfig,
                                         CpswMacPort_Interface *interface,
                                         CpswPhy_Config *phyConfig);

/**
 * \brief Set link, MAC and PHY configuration for loopback.
 *
 * Sets configuration of port link and PHY for PHY loopback.  The port link
 * is set to fixed speed set according to the max speed of the given MAC
 * interface: 100Mbps for MII layer type, 1Gpbs for GMII layer type.
 */
void CpswAppBoardUtils_setLpbkCfg(bool phyLpbk,
                                  Cpsw_MacPort portNum,
                                  const CpswMacPort_Interface *interface,
                                  CpswMacPort_Config *macConfig,
                                  CpswPhy_Config *phyConfig,
                                  CpswMacPort_LinkConfig *linkConfig);

void CpswAppBoardUtils_getMacAddrList(uint8_t macAddr[][CPSW_MAC_ADDR_LEN],
                                      uint32_t maxMacEntries,
                                      uint32_t *pAvailMacEntries);

/*!
 * \brief Enables pinmux for the board interfaces required for EthFw.
 */
int32_t CpswAppBoardUtils_configEthFwPinmux(void);

/**
 *  \brief Utils API to set mode_sel field of ENET_CTRL register
 */
void CpswAppBoardUtils_setEnetControl(Cpsw_Type cpswType,
                                      Cpsw_MacPort portNum,
                                      uint32_t modeSel);

#ifdef __cplusplus
}
#endif

#endif /* CPSW_APPBOARDUTILS_H_ */
