/*
 *  Copyright (c) Texas Instruments Incorporated 2018
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 *  \file cpsw_appmemutils.h
 *
 *  \brief CPSW DMA memutils header file.
 */

#ifndef CPSW_APPMEMUTILS_CFG_H_
#define CPSW_APPMEMUTILS_CFG_H_

/* ========================================================================== */
/*                             Include Files                                  */
/* ========================================================================== */
#include <ti/drv/cpsw/include/cpsw_cfg.h>

#ifdef __cplusplus
extern "C" {
#endif

/* ========================================================================== */
/*                           Macros & Typedefs                                */
/* ========================================================================== */

/*
 * Packet sizes for different pools. LARGE_POOL is largest packet size pool with MEDIUM_POOL
 * second largest and SMALL_POOL smallest
 */
#define CPSW_APPMEMUTILS_LARGE_POOL_PKT_SIZE        CPSW_UTILS_ALIGN(1536U, CPSW_UTILS_CACHELINE_SIZE)
#define CPSW_APPMEMUTILS_MEDIUM_POOL_PKT_SIZE       CPSW_UTILS_ALIGN(1000U, CPSW_UTILS_CACHELINE_SIZE)
#define CPSW_APPMEMUTILS_SMALL_POOL_PKT_SIZE        CPSW_UTILS_ALIGN(500U, CPSW_UTILS_CACHELINE_SIZE)

#define CPSW_APPMEMUTILS_LARGE_POOL_RX_PACKET_NUM   (128U)
#define CPSW_APPMEMUTILS_LARGE_POOL_TX_PACKET_NUM   (128U)
#define CPSW_APPMEMUTILS_LARGE_POOL_PACKET_NUM                             \
    ((CPSW_APPMEMUTILS_LARGE_POOL_RX_PACKET_NUM * CPSW_CFG_RX_FLOWS_NUM) + \
     (CPSW_APPMEMUTILS_LARGE_POOL_TX_PACKET_NUM * CPSW_CFG_TX_CHANNELS_NUM))

#define CPSW_APPMEMUTILS_MEDIUM_POOL_RX_PACKET_NUM  (64U)
#define CPSW_APPMEMUTILS_MEDIUM_POOL_TX_PACKET_NUM  (64U)
#define CPSW_APPMEMUTILS_MEDIUM_POOL_PACKET_NUM                             \
    ((CPSW_APPMEMUTILS_MEDIUM_POOL_RX_PACKET_NUM * CPSW_CFG_RX_FLOWS_NUM) + \
     (CPSW_APPMEMUTILS_MEDIUM_POOL_TX_PACKET_NUM * CPSW_CFG_TX_CHANNELS_NUM))

#define CPSW_APPMEMUTILS_SMALL_POOL_RX_PACKET_NUM   (64U)
#define CPSW_APPMEMUTILS_SMALL_POOL_TX_PACKET_NUM   (64U)
#define CPSW_APPMEMUTILS_SMALL_POOL_PACKET_NUM                             \
    ((CPSW_APPMEMUTILS_SMALL_POOL_RX_PACKET_NUM * CPSW_CFG_RX_FLOWS_NUM) + \
     (CPSW_APPMEMUTILS_SMALL_POOL_TX_PACKET_NUM * CPSW_CFG_TX_CHANNELS_NUM))

#define CPSW_APPMEMUTILS_NUM_TX_PKTS                (128U)
#define CPSW_APPMEMUTILS_NUM_RX_PKTS                (128U)

#define CPSW_APPMEMUTILS_NUM_TX_RING_ENTRIES        (CPSW_APPMEMUTILS_NUM_TX_PKTS)
#define CPSW_APPMEMUTILS_NUM_RX_RING_ENTRIES        (CPSW_APPMEMUTILS_NUM_RX_PKTS)

#define CPSW_APPMEMUTILS_RING_MAX_ELEM_CNT          (256U)

/* ========================================================================== */
/*                         Structure Declarations                             */
/* ========================================================================== */

/* ========================================================================== */
/*                          Function Declarations                             */
/* ========================================================================== */

/* ========================================================================== */
/*                       Static Function Definitions                          */
/* ========================================================================== */

/* None */

#ifdef __cplusplus
}
#endif

#endif /* CPSW_APPMEMUTILS_CFG_H_ */
