/*
 *  Copyright (c) Texas Instruments Incorporated 2018
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*!
 * \file     cpsw_mcm.h
 *
 * \brief    This file contains the structure definitions and function
 *           prototypes of the Multi-client manager.
 */

/*!
 * \addtogroup CPSW_MCM_API
 * @{
 */

#ifndef CPSW_MCM_H_
#define CPSW_MCM_H_

/* ========================================================================== */
/*                             Include Files                                  */
/* ========================================================================== */
#include <xdc/std.h>
#include <ti/sysbios/knl/Mailbox.h>

#ifdef __cplusplus
extern "C" {
#endif

/* ========================================================================== */
/*                                 Macros                                     */
/* ========================================================================== */
typedef void (*CpswMcm_setPortLinkCfg)(Cpsw_OpenPortLinkInArgs *linkArgs,
                                       Cpsw_MacPort macPort);

/* ========================================================================== */
/*                         Structures and Enums                               */
/* ========================================================================== */
typedef struct CpswMcm_InitConfig_s
{
    Cpsw_Type cpswType;

    Cpsw_Config *pCpswCfg;

    CpswMcm_setPortLinkCfg setPortLinkCfg;

    Cpsw_MacPort macPortList[CPSW_MAC_PORT_NUM];

    uint8_t numMacPorts;

    uint32_t periodicTaskPeriod;

    Cpsw_PrintFxnCb printFxn;

    Cpsw_TraceFxnCb traceFxn;
}CpswMcm_InitConfig;

typedef struct CpswMcm_HandleInfo_s
{
    Cpsw_Handle hCpsw;

    Udma_DrvHandle hUdmaDrv;
} CpswMcm_HandleInfo;

typedef struct CpswMcm_CmdIf_s
{
    Mailbox_Handle hMboxCmd;
    Mailbox_Handle hMboxResponse;
}  CpswMcm_CmdIf;

/* ========================================================================== */
/*                         Global Variables Declarations                      */
/* ========================================================================== */

/* ========================================================================== */
/*                          Function Declarations                             */
/* ========================================================================== */

int32_t  CpswMcm_init(const CpswMcm_InitConfig *pCpswMcmInitCfg);

void  CpswMcm_getCmdIf(Cpsw_Type cpswType,
                       CpswMcm_CmdIf *hMcmCmdIf);

void     CpswMcm_acquireHandleInfo(const CpswMcm_CmdIf *hMcmCmdIf,
                                   CpswMcm_HandleInfo *handleInfo);

void     CpswMcm_coreAttach(const CpswMcm_CmdIf *hMcmCmdIf,
                            uint32_t coreId,
                            Cpsw_AttachCoreOutArgs *attachInfo);

void     CpswMcm_coreDetach(const CpswMcm_CmdIf *hMcmCmdIf,
                            uint32_t coreId,
                            uint32_t coreKey);

void            CpswMcm_releaseHandleInfo(const CpswMcm_CmdIf *hMcmCmdIf);

void            CpswMcm_deInit(Cpsw_Type cpswType);

#ifdef __cplusplus
}
#endif

#endif /* CPSW_MCM_H_ */

/* @} */
