/*
 * Copyright (c) 2014-2019, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 *  \file cpsw_networkutils.h
 *
 *  \brief Interface header file containing prototypes and structures for
 *         network APIs.
 */

#ifndef CPSW_NETWORKUTILS_H_
#define CPSW_NETWORKUTILS_H_

/* ========================================================================== */
/*                             Include Files                                  */
/* ========================================================================== */

#include <ti/ndk/inc/socket.h>
#include <ti/ndk/inc/socketndk.h>
#include <ti/drv/cpsw/include/cpsw_types.h>

#ifdef __cplusplus
extern "C" {
#endif

/* ========================================================================== */
/*                           Macros & Typedefs                                */
/* ========================================================================== */
#define CPSW_NETWORKUTILS_ERROR               (CPSW_EFAIL)
#define CPSW_NETWORKUTILS_SUCCESS             (CPSW_SOK)

/* ========================================================================== */
/*                         Structure Declarations                             */
/* ========================================================================== */

typedef struct
{
    SOCKET sockFd;
    /**< Server socket handle */

    SOCKET connectedSockFd;
    /**< socket handle of client that is connected */

    uint32_t port;
    /**< port on which server is listening */

    FDPOLLITEM pollitem[1U];
    /**< Polling structure to use with fdPoll() */

    uint32_t state;
    /**< State of server socket */
} CpswNetworkUtils_SockObj;

/* ========================================================================== */
/*                          Function Declarations                             */
/* ========================================================================== */

int32_t CpswNetworkUtils_createSocket(CpswNetworkUtils_SockObj *pObj);

int32_t CpswNetworkUtils_deleteSocket(CpswNetworkUtils_SockObj *pObj);

int32_t CpswNetworkUtils_writePayload(CpswNetworkUtils_SockObj *pObj,
                                      uint8_t *bufAddr,
                                      uint32_t bufSize);

int32_t CpswNetworkUtils_readPayload(CpswNetworkUtils_SockObj *pObj,
                                     uint8_t *bufAddr,
                                     uint32_t bufSize);

int32_t CpswNetworkUtils_closeSocket(CpswNetworkUtils_SockObj *pObj,
                                     bool closeServerSock);

/* ========================================================================== */
/*                          Extern Function Declarations                      */
/* ========================================================================== */

/* The following declarations are part of socketndk.h, but still there are some
 * build errors. As a temporary fix following declarations are added here.
 */
extern int NDK_send(SOCKET s,
                    void *pbuf,
                    int size,
                    int flags);

extern int NDK_recv(SOCKET s,
                    void *pbuf,
                    int size,
                    int flags);

extern SOCKET NDK_socket(int domain,
                         int type,
                         int protocol);

extern int NDK_setsockopt(SOCKET s,
                          int level,
                          int op,
                          void *pbuf,
                          int bufsize);

extern int NDK_bind(SOCKET s,
                    struct sockaddr *pName,
                    int len);

extern int NDK_listen(SOCKET s,
                      int maxcon);

extern SOCKET NDK_accept(SOCKET s,
                         struct sockaddr *pName,
                         int *plen);

extern int fdClose(void *hFd);

extern int fdOpenSession(void *hOwner);

extern void fdCloseSession(void *hTask);

extern int fdPoll(FDPOLLITEM items[],
                  uint32_t itemcnt,
                  int32_t timeout);

/* ========================================================================== */
/*                       Static Function Definitions                          */
/* ========================================================================== */

/* None */

#ifdef __cplusplus
}
#endif

#endif /* #ifndef CPSW_NETWORKUTILS_H_ */
