/*
 * Copyright (C) 2013 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 *   \file  ndk2cpsw_appif.h
 *
 *   \brief
 *      Header file for App interfaces to the ndk2cpsw.c
 *
 */

#ifndef NDK2CPSW_APPIF_H_
#define NDK2CPSW_APPIF_H_

#ifdef __cplusplus
extern "C" {
#endif

/*---------------------------------------------------------------------------*\
 |                                Header Files                                 |
 \*---------------------------------------------------------------------------*/

/* Standard language headers */
#include <stdint.h>
#include <assert.h>
#include <ti/drv/cpsw/cpsw.h>

/*---------------------------------------------------------------------------*\
 |                           Global Typedefs/Enums                             |
 \*---------------------------------------------------------------------------*/

typedef void (*NimuCpswAppIf_FreePktCbFxn)(void *cbArg,
                                           CpswDma_PktInfoQ *fqPktInfoQ,
                                           CpswDma_PktInfoQ *cqPktInfoQ);

typedef bool (*NimuCpswAppIf_IsPhyLinkedCbFxn)(Cpsw_Handle hCpsw);

typedef struct NimuCpswAppIf_RxConfig_s
{
    void *cbArg;
    CpswDma_PktNotifyCb notifyCb;
    uint32_t numPackets;
} NimuCpswAppIf_RxConfig;

typedef struct NimuCpswAppIf_TxConfig_s
{
    void *cbArg;
    CpswDma_PktNotifyCb notifyCb;
    uint32_t numPackets;
} NimuCpswAppIf_TxConfig;

typedef struct NimuCpswAppIf_GetHandleInArgs_s
{
    NimuCpswAppIf_TxConfig txCfg;
    NimuCpswAppIf_RxConfig rxCfg;
} NimuCpswAppIf_GetHandleInArgs;

typedef struct NimuCpswAppIf_RxHandleInfo_s
{
    /** CPSW DMA receive channel */
    CpswDma_RxFlowHandle hRxFlow;
    /** Flow index for flow used  */
    uint32_t rxFlowStartIdx;
    /** Flow index for flow used  */
    uint32_t rxFlowIdx;
    /** Flag to indicate NIMU is using Default flow or not   */
    bool isDefaultFlow;
    /** Mac Address allocated for the flow */
    uint8_t macAddr[CPSW_MAC_ADDR_LEN];
} NimuCpswAppIf_RxHandleInfo;

typedef struct NimuCpswAppIf_TxHandleInfo_s
{
    /** CPSW DMA transmit channel */
    CpswDma_TxChHandle hTxChannel;
    /** Tx Channel Peer Id */
    uint32_t txChNum;
} NimuCpswAppIf_TxHandleInfo;

typedef struct NimuCpswAppIf_GetHandleOutArgs_s
{
    Cpsw_Handle hCpsw;
    Udma_DrvHandle hUdmaDrv;
    uint32_t coreId;
    uint32_t coreKey;
    Cpsw_PrintFxnCb printFxnCb;
    /** Max Tx packet size per priority */
    uint32_t txMtu[CPSW_PRIORITY_NUM];
    /** Max Rx packet size */
    uint32_t hostPortRxMtu;
    NimuCpswAppIf_IsPhyLinkedCbFxn isPortLinkedFxn;
    NimuCpswAppIf_TxHandleInfo txInfo;
    NimuCpswAppIf_RxHandleInfo rxInfo;
    bool isRingMonUsed;
    bool disableTxEvent;
    /** Timer interval for timer based RX pacing */
    uint32_t timerPeriodUs;
} NimuCpswAppIf_GetHandleOutArgs;

typedef struct NimuCpswAppIf_ReleaseHandleInfo_s
{
    Cpsw_Handle hCpsw;
    Udma_DrvHandle hUdmaDrv;
    uint32_t coreId;
    uint32_t coreKey;
    NimuCpswAppIf_TxHandleInfo txInfo;
    NimuCpswAppIf_RxHandleInfo rxInfo;
    void *freePktCbArg;
    NimuCpswAppIf_FreePktCbFxn txFreePktCb;
    NimuCpswAppIf_FreePktCbFxn rxFreePktCb;
} NimuCpswAppIf_ReleaseHandleInfo;

/*---------------------------------------------------------------------------*\
 |                         Global Function Declarations                        |
 \*---------------------------------------------------------------------------*/

extern void NimuCpswAppCb_getHandle(NimuCpswAppIf_GetHandleInArgs *inArgs,
                                    NimuCpswAppIf_GetHandleOutArgs *outArgs);

extern void NimuCpswAppCb_releaseHandle(NimuCpswAppIf_ReleaseHandleInfo *releaseInfo);

/*---------------------------------------------------------------------------*\
 |                                 End of File                                 |
 \*---------------------------------------------------------------------------*/

#ifdef __cplusplus
}
#endif

#endif /* NDK2CPSW_APPIF_H_ */
