/*
 *  Copyright (c) Texas Instruments Incorporated 2018
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*!
 * \file     cpsw_soc.c
 *
 * \brief    This file contains the CPSW driver SOC related configuration
 *           functions.
 */

/* ========================================================================== */
/*                             Include Files                                  */
/* ========================================================================== */

#include <ti/drv/cpsw/cpsw.h>
#include <ti/drv/cpsw/src/priv/cpsw_priv.h>

/* ========================================================================== */
/*                           Macros & Typedefs                                */
/* ========================================================================== */

/* These defines should come from cslr_ale.h file but have been removed.
 * once seperate RESETVAL header file is provided delete the defines
 * below
 */
#define CSL_ALE_MOD_VER_MODULE_ID_RESETVAL                       (0x00000029U)
#define CSL_ALE_MOD_VER_MAJOR_REVISION_RESETVAL                  (0x00000001U)
#define CSL_ALE_MOD_VER_MINOR_REVISION_RESETVAL                  (0x00000004U)
#define CSL_ALE_2G_MOD_VER_RTL_VERSION_RESETVAL                  (0x00000007U)
#define CSL_ALE_9G_MOD_VER_RTL_VERSION_RESETVAL                  (0x00000008U)

#define CSL_CPSW_SS_MOD_VER_IDENT_RESETVAL                       (0x00006BA0U)
#define CSL_CPSW_SS_MOD_VER_MAJOR_REVISION_RESETVAL              (0x00000001U)
#define CSL_CPSW_SS_MOD_VER_MINOR_REVISION_RESETVAL              (0x00000001U)
#define CSL_CPSW_SS_2G_MOD_VER_RTL_VERSION_RESETVAL              (0x00000000U)
#define CSL_CPSW_SS_9G_MOD_VER_RTL_VERSION_RESETVAL              (0x00000003U)

#define CSL_CPTS_MOD_VER_TX_IDENT_RESETVAL                       (0x00004E8AU)
#define CSL_CPTS_MOD_VER_MAJOR_REVISION_RESETVAL                 (0x00000001U)
#define CSL_CPTS_MOD_VER_MINOR_REVISION_RESETVAL                 (0x0000000AU)
#define CSL_CPTS_2G_MOD_VER_RTL_VERSION_RESETVAL                 (0x00000000U)
#define CSL_CPTS_9G_MOD_VER_RTL_VERSION_RESETVAL                 (0x00000000U)

#define CSL_MDIO_MOD_VER_MODULE_ID_RESETVAL                      (0x00000007U)
#define CSL_MDIO_MOD_VER_MAJOR_REVISION_RESETVAL                 (0x00000001U)
#define CSL_MDIO_MOD_VER_MINOR_REVISION_RESETVAL                 (0x00000007U)
#define CSL_MDIO_2G_MOD_VER_RTL_VERSION_RESETVAL                 (0x00000001U)
#define CSL_MDIO_9G_MOD_VER_RTL_VERSION_RESETVAL                 (0x00000001U)


#define CSL_XGE_CPSW_ID_VER_REG_IDENT_RESETVAL                   (0x00006BA8U)
#define CSL_XGE_CPSW_ID_VER_REG_MAJOR_VER_RESETVAL               (0x00000001U)
#define CSL_XGE_2G_CPSW_ID_VER_REG_MINOR_VER_RESETVAL            (0x00000000U)
#define CSL_XGE_9G_CPSW_ID_VER_REG_MINOR_VER_RESETVAL            (0x00000001U)
#define CSL_XGE_CPSW_ID_VER_REG_RTL_VER_RESETVAL                 (0x0000000EU)

#define CSL_SGMII_MOD_VER_TX_ID_RESETVAL                         (0x00004EC2U)
#define CSL_SGMII_MOD_VER_MAJOR_REVISION_RESETVAL                (0x00000001U)
#define CSL_SGMII_MOD_VER_MINOR_REVISION_RESETVAL                (0x00000002U)
#define CSL_SGMII_MOD_VER_RTL_VERSION_RESETVAL                   (0x00000002U)


/*! \brief SYSFW SRC index values for CPTS signals
 * These macros are not used as Time Sync Router(TSR) is configured directly through CSL APIs*/
#define CPSW_CPTS_COMP0_SRCIDX                       (0U)
#define CPSW_CPTS_GENF0_0_SRCIDX                     (1U)
#define CPSW_CPTS_GENF1_0_SRCIDX                     (2U)
#define CPSW_CPTS_SYNC_SRCIDX                        (3U)

/*! \brief SYSFW SRC index values for CPSW interrupts */
#define CPSW_INTR_EVENT_PEND_SRCIDX                  (4U)
#define CPSW_INTR_MDIO_INTR_SRCIDX                   (5U)
#define CPSW_INTR_STAT_PEND0_SRCIDX                  (6U)

#define CPSW_QSGMII_SUB_PORT_MAX                     (6U)
#define CPSW_QSGMII_SUB_PER_MAIN_MAX                 (3U)

/* ========================================================================== */
/*                         Structure Declarations                             */
/* ========================================================================== */

/* None */

/* ========================================================================== */
/*                          Function Declarations                             */
/* ========================================================================== */

static int32_t CpswSoc_getQsgmiiCfg(Cpsw_Type cpswType,
                             Cpsw_MacPort *qsgmii0Port,
                             Cpsw_MacPort *qsgmii1Port,
                             Cpsw_MacPort qsgmiiSubPorts[CPSW_QSGMII_SUB_PORT_MAX],
                             uint32_t *subPortCnt);

/* ========================================================================== */
/*                            Global Variables                                */
/* ========================================================================== */

CpswSoc_Config cpswSocConfig[CPSW_COUNT] =
{
    [CPSW_2G] =
    {
        /* As per the clocking specification the mcu_sysclk0 is 1000MHz with
         * fixed /3 divider. */
        .cppiClkFreqHz = 333333333LLU,
        .dmscDevIdCpsw = TISCI_DEV_MCU_CPSW0,
        .intrs         =
        {
#if defined(BUILD_MCU1_0)
            .evntPend =
            {
                .coreIntNum = CSLR_MCU_R5FSS0_CORE0_INTR_MCU_CPSW0_EVNT_PEND_0,
                .srcIdx     = CPSW_SOC_DIRECT_INTR_SRCIDX_INVALID,
            },
            .statPend0 =
            {
                .coreIntNum = CSLR_MCU_R5FSS0_CORE0_INTR_MCU_CPSW0_STAT_PEND_0,
                .srcIdx     = CPSW_SOC_DIRECT_INTR_SRCIDX_INVALID,
            },
            .mdioIntr =
            {
                .coreIntNum = CSLR_MCU_R5FSS0_CORE0_INTR_MCU_CPSW0_MDIO_PEND_0,
                .srcIdx     = CPSW_SOC_DIRECT_INTR_SRCIDX_INVALID,
            },
#elif defined(BUILD_MCU2_0) || defined(BUILD_MCU2_1)

            /* Note - As the interrupt router output interrupt numbers are same for core 0 and 1
             *        are same we use macros of core0 for both cores. For ex - we use
             *        CSLR_R5FSS0_CORE0_INTR_R5FSS0_INTROUTER0_OUTL_2 for both core 0 and core 1
             *        (core 1 has CSLR_R5FSS0_CORE0_INTR_R5FSS0_INTROUTER0_OUTL_2) */
            .evntPend =
            {
                .coreIntNum = CSLR_R5FSS0_CORE0_INTR_R5FSS0_INTROUTER0_OUTL_2,   /* Taking IR 2 as default, user can change */
                .srcIdx     = CPSW_INTR_EVENT_PEND_SRCIDX,
            },
            .statPend0 =
            {
                .coreIntNum = CSLR_R5FSS0_CORE0_INTR_R5FSS0_INTROUTER0_OUTL_3,   /* Taking IR 3 as default, user can change */
                .srcIdx     = CPSW_INTR_STAT_PEND0_SRCIDX,
            },
            .mdioIntr =
            {
                .coreIntNum = CSLR_R5FSS0_CORE0_INTR_R5FSS0_INTROUTER0_OUTL_4,   /* Taking IR 4 as default, user can change */
                .srcIdx     = CPSW_INTR_MDIO_INTR_SRCIDX,
            },
#elif defined(BUILD_MCU3_0) || defined(BUILD_MCU3_1)

            /* Note - As the interrupt router output interrupt numbers are same for core 0 and 1
             *        are same we use macros of core0 for both cores. For ex - we use
             *        CSLR_R5FSS1_CORE0_INTR_R5FSS0_INTROUTER0_OUTL_2 for both core 0 and core 1
             *        (core 1 has CSLR_R5FSS1_CORE0_INTR_R5FSS0_INTROUTER0_OUTL_2) */
            .evntPend =
            {
                .coreIntNum = CSLR_R5FSS1_CORE0_INTR_R5FSS0_INTROUTER0_OUTL_2,   /* Taking IR 2 as default, user can change */
                .srcIdx     = CPSW_INTR_EVENT_PEND_SRCIDX,
            },
            .statPend0 =
            {
                .coreIntNum = CSLR_R5FSS1_CORE0_INTR_R5FSS0_INTROUTER0_OUTL_3,   /* Taking IR 3 as default, user can change */
                .srcIdx     = CPSW_INTR_STAT_PEND0_SRCIDX,
            },
            .mdioIntr =
            {
                .coreIntNum = CSLR_R5FSS1_CORE0_INTR_R5FSS0_INTROUTER0_OUTL_4,   /* Taking IR 4 as default, user can change */
                .srcIdx     = CPSW_INTR_MDIO_INTR_SRCIDX,
            },
#elif defined(BUILD_MPU1_0)
            .evntPend =
            {
                .coreIntNum = CSLR_COMPUTE_CLUSTER0_GIC500SS_SPI_MCU_CPSW0_EVNT_PEND_0,
                .srcIdx     = CPSW_SOC_DIRECT_INTR_SRCIDX_INVALID,
            },
            .statPend0 =
            {
                .coreIntNum = CSLR_COMPUTE_CLUSTER0_GIC500SS_SPI_MCU_CPSW0_STAT_PEND_0,
                .srcIdx     = CPSW_SOC_DIRECT_INTR_SRCIDX_INVALID,
            },
            .mdioIntr =
            {
                .coreIntNum = CSLR_COMPUTE_CLUSTER0_GIC500SS_SPI_MCU_CPSW0_MDIO_PEND_0,
                .srcIdx     = CPSW_SOC_DIRECT_INTR_SRCIDX_INVALID,
            },
#else
#error "CPSW SOC V1: Core not supported!!"
#endif
        },
        .clocks =
        {
            .cppiClkId        = TISCI_DEV_MCU_CPSW0_CPPI_CLK_CLK,
            .rgmii250MHzClkId = TISCI_DEV_MCU_CPSW0_RGMII_MHZ_250_CLK,
            .rgmii50MHzClkId  = TISCI_DEV_MCU_CPSW0_RGMII_MHZ_50_CLK,
            .rgmii5MHzClkId   = TISCI_DEV_MCU_CPSW0_RGMII_MHZ_5_CLK,
        },
        .txChPeerThreadId = CSL_PSILCFG_NAVSS_MCU_CPSW0_PSILD_THREAD_OFFSET,
        .rxChPeerThreadId = CSL_PSILCFG_NAVSS_MCU_CPSW0_PSILS_THREAD_OFFSET,
        .txChCount        = CSL_PSILCFG_NAVSS_MCU_CPSW0_PSILD_THREAD_CNT,

        /* Note- Though CPSW supports 64 distinct flow Ids, there are only
         * 8 policer/classifier so can effectively assign only 8 flow ids in CPSW2G */
        .rxFlowCount     = 8U,
        .cptsHwPushCount = 4U,
    },

    [CPSW_9G] =
    {
        .cppiClkFreqHz = 320000000U, /* 320 MHz */
        .dmscDevIdCpsw = TISCI_DEV_CPSW0,
        .intrs         =
        {
#if defined(BUILD_MCU1_0)
            .evntPend =
            {
                .coreIntNum = CSLR_MAIN2MCU_LVL_INTRTR0_IN_CPSW0_EVNT_PEND_0,
                .srcIdx     = CPSW_SOC_DIRECT_INTR_SRCIDX_INVALID,
            },
            .statPend0 =
            {
                .coreIntNum = CSLR_MAIN2MCU_LVL_INTRTR0_IN_CPSW0_STAT_PEND_0,
                .srcIdx     = CPSW_SOC_DIRECT_INTR_SRCIDX_INVALID,
            },
            .mdioIntr =
            {
                .coreIntNum = CSLR_MAIN2MCU_LVL_INTRTR0_IN_CPSW0_MDIO_PEND_0,
                .srcIdx     = CPSW_SOC_DIRECT_INTR_SRCIDX_INVALID,
            },
#elif defined(BUILD_MCU2_0)
            .evntPend =
            {
                .coreIntNum = CSLR_R5FSS0_CORE0_INTR_CPSW0_EVNT_PEND_0,
                .srcIdx     = CPSW_SOC_DIRECT_INTR_SRCIDX_INVALID,
            },
            .statPend0 =
            {
                .coreIntNum = CSLR_R5FSS0_CORE0_INTR_CPSW0_STAT_PEND_0,
                .srcIdx     = CPSW_SOC_DIRECT_INTR_SRCIDX_INVALID,
            },
            .mdioIntr =
            {
                .coreIntNum = CSLR_R5FSS0_CORE0_INTR_CPSW0_MDIO_PEND_0,
                .srcIdx     = CPSW_SOC_DIRECT_INTR_SRCIDX_INVALID,
            },
#elif defined(BUILD_MCU2_1)
            .evntPend =
            {
                .coreIntNum = CSLR_R5FSS0_CORE1_INTR_CPSW0_EVNT_PEND_0,
                .srcIdx     = CPSW_SOC_DIRECT_INTR_SRCIDX_INVALID,
            },
            .statPend0 =
            {
                .coreIntNum = CSLR_R5FSS0_CORE1_INTR_CPSW0_STAT_PEND_0,
                .srcIdx     = CPSW_SOC_DIRECT_INTR_SRCIDX_INVALID,
            },
            .mdioIntr =
            {
                .coreIntNum = CSLR_R5FSS0_CORE1_INTR_CPSW0_MDIO_PEND_0,
                .srcIdx     = CPSW_SOC_DIRECT_INTR_SRCIDX_INVALID,
            },
#elif defined(BUILD_MCU3_0)
            .evntPend =
            {
                .coreIntNum = CSLR_R5FSS1_CORE0_INTR_CPSW0_EVNT_PEND_0,
                .srcIdx     = CPSW_SOC_DIRECT_INTR_SRCIDX_INVALID,
            },
            .statPend0 =
            {
                .coreIntNum = CSLR_R5FSS1_CORE0_INTR_CPSW0_STAT_PEND_0,
                .srcIdx     = CPSW_SOC_DIRECT_INTR_SRCIDX_INVALID,
            },
            .mdioIntr =
            {
                .coreIntNum = CSLR_R5FSS1_CORE0_INTR_CPSW0_MDIO_PEND_0,
                .srcIdx     = CPSW_SOC_DIRECT_INTR_SRCIDX_INVALID,
            },
#elif defined(BUILD_MCU3_1)
            .evntPend =
            {
                .coreIntNum = CSLR_R5FSS1_CORE1_INTR_CPSW0_EVNT_PEND_0,
                .srcIdx     = CPSW_SOC_DIRECT_INTR_SRCIDX_INVALID,
            },
            .statPend0 =
            {
                .coreIntNum = CSLR_R5FSS1_CORE1_INTR_CPSW0_STAT_PEND_0,
                .srcIdx     = CPSW_SOC_DIRECT_INTR_SRCIDX_INVALID,
            },
            .mdioIntr =
            {
                .coreIntNum = CSLR_R5FSS1_CORE1_INTR_CPSW0_MDIO_PEND_0,
                .srcIdx     = CPSW_SOC_DIRECT_INTR_SRCIDX_INVALID,
            },
#elif defined(BUILD_MPU1_0)
            .evntPend =
            {
                .coreIntNum = CSLR_COMPUTE_CLUSTER0_GIC500SS_SPI_CPSW0_EVNT_PEND_0,
                .srcIdx     = CPSW_SOC_DIRECT_INTR_SRCIDX_INVALID,
            },
            .statPend0 =
            {
                .coreIntNum = CSLR_COMPUTE_CLUSTER0_GIC500SS_SPI_CPSW0_STAT_PEND_0,
                .srcIdx     = CPSW_SOC_DIRECT_INTR_SRCIDX_INVALID,
            },
            .mdioIntr =
            {
                .coreIntNum = CSLR_COMPUTE_CLUSTER0_GIC500SS_SPI_CPSW0_MDIO_PEND_0,
                .srcIdx     = CPSW_SOC_DIRECT_INTR_SRCIDX_INVALID,
            },
#else
#error "CPSW SOC V1: Core not supported!!"
#endif
        },
        .clocks =
        {
            .cppiClkId        = TISCI_DEV_CPSW0_CPPI_CLK_CLK,
            .rgmii250MHzClkId = TISCI_DEV_CPSW0_RGMII_MHZ_250_CLK,
            .rgmii50MHzClkId  = TISCI_DEV_CPSW0_RGMII_MHZ_50_CLK,
            .rgmii5MHzClkId   = TISCI_DEV_CPSW0_RGMII_MHZ_5_CLK,
        },
        .txChPeerThreadId = CSL_PSILCFG_NAVSS_MAIN_CPSW9_PSILD_THREAD_OFFSET,
        .rxChPeerThreadId = CSL_PSILCFG_NAVSS_MAIN_CPSW9_PSILS_THREAD_OFFSET,
        .txChCount        = CSL_PSILCFG_NAVSS_MAIN_CPSW9_PSILD_THREAD_CNT,
        .rxFlowCount      = 64U,
        .cptsHwPushCount  = 8U,
    },
};

const CpswSoc_CpswIpInfoTbl cpswIpInfoTbl =
{
    .numCpswIPs = 2,
    .info       =
    {
        [0] =
        {
            .cpswType       = CPSW_2G,
            .errataBitmap   = CPSW_ERRATA_GENFN_RECONFIG,
            .aleInfo        =
                              {
                               .regBaseAddr = (CSL_MCU_CPSW0_NUSS_BASE + CPSW_ALE_OFFSET),
                               .versionInfo = {
                                                  .moduleId = CSL_ALE_MOD_VER_MODULE_ID_RESETVAL,
                                                  .majorVer = CSL_ALE_MOD_VER_MAJOR_REVISION_RESETVAL,
                                                  .minorVer = CSL_ALE_MOD_VER_MINOR_REVISION_RESETVAL,
                                                  .rtlVer   = CSL_ALE_2G_MOD_VER_RTL_VERSION_RESETVAL,
                                              },
                              },
            .mdioInfo        =
                              {
                               .regBaseAddr = (CSL_MCU_CPSW0_NUSS_BASE + CPSW_MDIO_OFFSET),
                               .versionInfo = {
                                                  .moduleId = CSL_MDIO_MOD_VER_MODULE_ID_RESETVAL,
                                                  .majorVer = CSL_MDIO_MOD_VER_MAJOR_REVISION_RESETVAL,
                                                  .minorVer = CSL_MDIO_MOD_VER_MINOR_REVISION_RESETVAL,
                                                  .rtlVer   = CSL_MDIO_2G_MOD_VER_RTL_VERSION_RESETVAL,
                                              },
                              },
            .cptsInfo        =
                              {
                               .regBaseAddr = (CSL_MCU_CPSW0_NUSS_BASE + CPSW_CPTS_OFFSET),
                               .versionInfo = {
                                                  .moduleId = CSL_CPTS_MOD_VER_TX_IDENT_RESETVAL,
                                                  .majorVer = CSL_CPTS_MOD_VER_MAJOR_REVISION_RESETVAL,
                                                  .minorVer = CSL_CPTS_MOD_VER_MINOR_REVISION_RESETVAL,
                                                  .rtlVer   = CSL_CPTS_2G_MOD_VER_RTL_VERSION_RESETVAL,
                                              },
                              },
            .sgmiiInfo      =
                              {
                                  /* SGMII mode not supported for J721E CPSW2G */
                                  .regBaseAddr = 0x00000000UL,
                              },
            .cpswSSInfo     =
                              {
                               .regBaseAddr = (CSL_MCU_CPSW0_NUSS_BASE + CPSW_NUSS_OFFSET),
                               .versionInfo = {
                                                  .moduleId = CSL_CPSW_SS_MOD_VER_IDENT_RESETVAL,
                                                  .majorVer = CSL_CPSW_SS_MOD_VER_MAJOR_REVISION_RESETVAL,
                                                  .minorVer = CSL_CPSW_SS_MOD_VER_MINOR_REVISION_RESETVAL,
                                                  .rtlVer   = CSL_CPSW_SS_2G_MOD_VER_RTL_VERSION_RESETVAL,
                                              },
                              },
            .cpswInfo       =
                              {
                               .regBaseAddr = (CSL_MCU_CPSW0_NUSS_BASE + CPSW_NU_OFFSET),
                               .versionInfo = {
                                                  .moduleId = CSL_XGE_CPSW_ID_VER_REG_IDENT_RESETVAL,
                                                  .majorVer = CSL_XGE_CPSW_ID_VER_REG_MAJOR_VER_RESETVAL,
                                                  .minorVer = CSL_XGE_2G_CPSW_ID_VER_REG_MINOR_VER_RESETVAL,
                                                  .rtlVer   = CSL_XGE_CPSW_ID_VER_REG_RTL_VER_RESETVAL,
                                              },
                              },

            .featureBitmap  = 0,
        },
        [1] =
        {
            .cpswType       = CPSW_9G,
            .errataBitmap   = CPSW_ERRATA_GENFN_RECONFIG,
            .featureBitmap  = (CPSW_FEATURE_INTERVLAN |
                               CPSW_FEATURE_IPHDRWHITELIST |
                               CPSW_FEATURE_FLOWPRIORITY |
                               CPSW_FEATURE_SGMII |
                               CPSW_FEATURE_QSGMII),
            .aleInfo        = {
                               .regBaseAddr = (CSL_CPSW0_NUSS_BASE + CPSW_ALE_OFFSET),
                               .versionInfo = {
                                                  .moduleId = CSL_ALE_MOD_VER_MODULE_ID_RESETVAL,
                                                  .majorVer = CSL_ALE_MOD_VER_MAJOR_REVISION_RESETVAL,
                                                  .minorVer = CSL_ALE_MOD_VER_MINOR_REVISION_RESETVAL,
                                                  .rtlVer   = CSL_ALE_9G_MOD_VER_RTL_VERSION_RESETVAL,
                                              },
                              },
            .mdioInfo        =
                              {
                               .regBaseAddr = (CSL_CPSW0_NUSS_BASE + CPSW_MDIO_OFFSET),
                               .versionInfo = {
                                                  .moduleId = CSL_MDIO_MOD_VER_MODULE_ID_RESETVAL,
                                                  .majorVer = CSL_MDIO_MOD_VER_MAJOR_REVISION_RESETVAL,
                                                  .minorVer = CSL_MDIO_MOD_VER_MINOR_REVISION_RESETVAL,
                                                  .rtlVer   = CSL_MDIO_9G_MOD_VER_RTL_VERSION_RESETVAL,
                                              },
                              },
            .cptsInfo        =
                              {
                               .regBaseAddr = (CSL_CPSW0_NUSS_BASE + CPSW_CPTS_OFFSET),
                               .versionInfo = {
                                                  .moduleId = CSL_CPTS_MOD_VER_TX_IDENT_RESETVAL,
                                                  .majorVer = CSL_CPTS_MOD_VER_MAJOR_REVISION_RESETVAL,
                                                  .minorVer = CSL_CPTS_MOD_VER_MINOR_REVISION_RESETVAL,
                                                  .rtlVer   = CSL_CPTS_9G_MOD_VER_RTL_VERSION_RESETVAL,
                                              },
                              },
            .sgmiiInfo     =
                              {
                               .regBaseAddr = (CSL_CPSW0_NUSS_BASE + CPSW_SGMII0_OFFSET(0U)),
                               .versionInfo = {
                                                  .moduleId = CSL_SGMII_MOD_VER_TX_ID_RESETVAL,
                                                  .majorVer = CSL_SGMII_MOD_VER_MAJOR_REVISION_RESETVAL,
                                                  .minorVer = CSL_SGMII_MOD_VER_MINOR_REVISION_RESETVAL,
                                                  .rtlVer   = CSL_SGMII_MOD_VER_RTL_VERSION_RESETVAL,
                                              },
                              },
            .cpswSSInfo     =
                              {
                               .regBaseAddr = (CSL_CPSW0_NUSS_BASE + CPSW_NUSS_OFFSET),
                               .versionInfo = {
                                                  .moduleId = CSL_CPSW_SS_MOD_VER_IDENT_RESETVAL,
                                                  .majorVer = CSL_CPSW_SS_MOD_VER_MAJOR_REVISION_RESETVAL,
                                                  .minorVer = CSL_CPSW_SS_MOD_VER_MINOR_REVISION_RESETVAL,
                                                  .rtlVer   = CSL_CPSW_SS_9G_MOD_VER_RTL_VERSION_RESETVAL,
                                              },
                              },
            .cpswInfo     =
                              {
                               .regBaseAddr = (CSL_CPSW0_NUSS_BASE + CPSW_NU_OFFSET),
                               .versionInfo = {
                                                  .moduleId = CSL_XGE_CPSW_ID_VER_REG_IDENT_RESETVAL,
                                                  .majorVer = CSL_XGE_CPSW_ID_VER_REG_MAJOR_VER_RESETVAL,
                                                  .minorVer = CSL_XGE_9G_CPSW_ID_VER_REG_MINOR_VER_RESETVAL,
                                                  .rtlVer   = CSL_XGE_CPSW_ID_VER_REG_RTL_VER_RESETVAL,
                                              },
                              },
        },
    },
};

/* ========================================================================== */
/*                          Function Definitions                              */
/* ========================================================================== */

const CpswSoc_Config *CpswSoc_getConfig(Cpsw_Type cpswType)
{
    return &cpswSocConfig[cpswType];
}

void CpswSoc_setConfig(Cpsw_Type cpswType,
                       const CpswSoc_Config *cfg)
{
    cpswSocConfig[cpswType] = *cfg;
}

uint32_t CpswSoc_getCoreDevId(void)
{
    uint32_t coreDevId;

#if defined(BUILD_MPU1_0)
    coreDevId = TISCI_DEV_A72SS0_CORE0;
#elif defined(BUILD_MCU2_0)
    coreDevId = TISCI_DEV_R5FSS0_CORE0;
#elif defined(BUILD_MCU2_1)
    coreDevId = TISCI_DEV_R5FSS0_CORE1;
#elif defined(BUILD_MCU3_0)
    coreDevId = TISCI_DEV_R5FSS1_CORE0;
#elif defined(BUILD_MCU3_1)
    coreDevId = TISCI_DEV_R5FSS1_CORE1;
#elif defined(BUILD_C7X_1)
    coreDevId = TISCI_DEV_C71SS0;
#elif defined(BUILD_C66X_1)
    coreDevId = TISCI_DEV_C66SS0_CORE0;
#elif defined(BUILD_C66X_2)
    coreDevId = TISCI_DEV_C66SS0_CORE1;
#elif defined(BUILD_MCU1_0)
    coreDevId = TISCI_DEV_MCU_R5FSS0_CORE0;
#elif defined(BUILD_MCU1_1)
    coreDevId = TISCI_DEV_MCU_R5FSS0_CORE1;
#else
#error "CPSW SOC V1: Core not supported!!"
#endif

    return(coreDevId);
}

static uint32_t CpswSoc_getMcuEnetControl(Cpsw_MacPort portNum,
                                          uint32_t *modeSel)
{
#ifdef QNX_OS
    CSL_mcu_ctrl_mmr_cfg0Regs *regs = (CSL_mcu_ctrl_mmr_cfg0Regs *) mmap_device_memory(0,
        CSL_MCU_CTRL_MMR0_CFG0_SIZE, PROT_READ|PROT_WRITE|PROT_NOCACHE, 0, CSL_MCU_CTRL_MMR0_CFG0_BASE);
#else
    CSL_mcu_ctrl_mmr_cfg0Regs *regs =
        (CSL_mcu_ctrl_mmr_cfg0Regs *)(uintptr_t)CSL_MCU_CTRL_MMR0_CFG0_BASE;
#endif
    int32_t status = CPSW_SOK;

    switch (portNum)
    {
        case CPSW_MAC_PORT_0:
            *modeSel = CSL_FEXT(regs->MCU_ENET_CTRL,
                                MCU_CTRL_MMR_CFG0_MCU_ENET_CTRL_MODE_SEL);
            break;

        default:
            status = CPSW_EINVALID_PARAMS;
            break;
    }

    if (status == CPSW_SOK)
    {
        switch (*modeSel)
        {
            case CPSW_ENET_CTRL_MODE_RMII:
            case CPSW_ENET_CTRL_MODE_RGMII:
                break;

            default:
                status = CPSW_EINVALID_PARAMS;
                break;
        }
    }

    return status;
}

static uint32_t CpswSoc_getMainEnetControl(Cpsw_MacPort portNum,
                                           uint32_t *modeSel)
{
#ifdef QNX_OS
    CSL_main_ctrl_mmr_cfg0Regs *regs = (CSL_main_ctrl_mmr_cfg0Regs *) mmap_device_memory(0,
        CSL_CTRL_MMR0_CFG0_SIZE, PROT_READ|PROT_WRITE|PROT_NOCACHE, 0, CSL_CTRL_MMR0_CFG0_BASE);
#else
    CSL_main_ctrl_mmr_cfg0Regs *regs =
        (CSL_main_ctrl_mmr_cfg0Regs *)(uintptr_t)CSL_CTRL_MMR0_CFG0_BASE;
#endif
    int32_t status = CPSW_SOK;

    switch (portNum)
    {
        case CPSW_MAC_PORT_0:
            *modeSel = CSL_FEXT(regs->ENET1_CTRL,
                                MAIN_CTRL_MMR_CFG0_ENET1_CTRL_PORT_MODE_SEL);
            break;

        case CPSW_MAC_PORT_1:
            *modeSel = CSL_FEXT(regs->ENET2_CTRL,
                                MAIN_CTRL_MMR_CFG0_ENET2_CTRL_PORT_MODE_SEL);
            break;

        case CPSW_MAC_PORT_2:
            *modeSel = CSL_FEXT(regs->ENET3_CTRL,
                                MAIN_CTRL_MMR_CFG0_ENET3_CTRL_PORT_MODE_SEL);
            break;

        case CPSW_MAC_PORT_3:
            *modeSel = CSL_FEXT(regs->ENET4_CTRL,
                                MAIN_CTRL_MMR_CFG0_ENET4_CTRL_PORT_MODE_SEL);
            break;

        case CPSW_MAC_PORT_4:
            *modeSel = CSL_FEXT(regs->ENET5_CTRL,
                                MAIN_CTRL_MMR_CFG0_ENET5_CTRL_PORT_MODE_SEL);
            break;

        case CPSW_MAC_PORT_5:
            *modeSel = CSL_FEXT(regs->ENET6_CTRL,
                                MAIN_CTRL_MMR_CFG0_ENET6_CTRL_PORT_MODE_SEL);
            break;

        case CPSW_MAC_PORT_6:
            *modeSel = CSL_FEXT(regs->ENET7_CTRL,
                                MAIN_CTRL_MMR_CFG0_ENET7_CTRL_PORT_MODE_SEL);
            break;

        case CPSW_MAC_PORT_7:
            *modeSel = CSL_FEXT(regs->ENET8_CTRL,
                                MAIN_CTRL_MMR_CFG0_ENET8_CTRL_PORT_MODE_SEL);
            break;

        default:
            status = CPSW_EINVALID_PARAMS;
            break;
    }

    if (status == CPSW_SOK)
    {
        switch (*modeSel)
        {
            case CPSW_ENET_CTRL_MODE_RMII:
            case CPSW_ENET_CTRL_MODE_RGMII:
            case CPSW_ENET_CTRL_MODE_SGMII:
            case CPSW_ENET_CTRL_MODE_QSGMII:
            case CPSW_ENET_CTRL_MODE_QSGMII_SUB:
                break;

            default:
                status = CPSW_EINVALID_PARAMS;
                break;
        }
    }

    return status;
}

uint32_t CpswSoc_getEnetControl(Cpsw_Type cpswType,
                                Cpsw_MacPort portNum,
                                CpswMac_LayerType *enetLayer,
                                CpswMac_SublayerType *enetSublayer)
{
    uint32_t modeSel = CPSW_ENET_CTRL_MODE_RGMII;
    int32_t status   = CPSW_SOK;

    switch (cpswType)
    {
        case CPSW_2G:
            status = CpswSoc_getMcuEnetControl(portNum, &modeSel);
            break;

        case CPSW_9G:
            status = CpswSoc_getMainEnetControl(portNum, &modeSel);
            break;

        default:
            Cpsw_assert(false);
            break;
    }

    if (status == CPSW_SOK)
    {
        switch (modeSel)
        {
            /* RMII */
            case CPSW_ENET_CTRL_MODE_RMII:
                *enetLayer    = CPSW_MAC_LAYER_MII;
                *enetSublayer = CPSW_MAC_SUBLAYER_REDUCED;
                break;

            /* RGMII */
            case CPSW_ENET_CTRL_MODE_RGMII:
                *enetLayer    = CPSW_MAC_LAYER_GMII;
                *enetSublayer = CPSW_MAC_SUBLAYER_REDUCED;
                break;

            /* SGMII */
            case CPSW_ENET_CTRL_MODE_SGMII:
                *enetLayer    = CPSW_MAC_LAYER_GMII;
                *enetSublayer = CPSW_MAC_SUBLAYER_SERIAL;
                break;

            /* QSGMII */
            case CPSW_ENET_CTRL_MODE_QSGMII:
                *enetLayer    = CPSW_MAC_LAYER_GMII;
                *enetSublayer = CPSW_MAC_SUBLAYER_QUAD_SERIAL_MAIN;
                break;

            /* QSGMII_SUB */
            case CPSW_ENET_CTRL_MODE_QSGMII_SUB:
                *enetLayer    = CPSW_MAC_LAYER_GMII;
                *enetSublayer = CPSW_MAC_SUBLAYER_QUAD_SERIAL_SUB;
                break;

            default:
                status = CPSW_EINVALID_PARAMS;
                break;
        }
    }

    return status;
}

void CpswSoc_getEFusedMacAddr(uint8_t *macAddr)
{
    /* Update default config with the correct MAC addresses */
#ifdef QNX_OS
    CSL_mcu_ctrl_mmr_cfg0Regs *mmrRegs = (CSL_mcu_ctrl_mmr_cfg0Regs *) mmap_device_memory(0,
        CSL_MCU_CTRL_MMR0_CFG0_SIZE, PROT_READ|PROT_WRITE|PROT_NOCACHE, 0, CSL_MCU_CTRL_MMR0_CFG0_BASE);
#else
    CSL_mcu_ctrl_mmr_cfg0Regs *mmrRegs =
        (CSL_mcu_ctrl_mmr_cfg0Regs *)(uintptr_t)CSL_MCU_CTRL_MMR0_CFG0_BASE;
#endif

    uint32_t val = CSL_REG32_RD(&mmrRegs->MAC_ID0);

    macAddr[5] = (uint8_t)((val & 0x000000FFU) >> 0U);
    macAddr[4] = (uint8_t)((val & 0x0000FF00U) >> 8U);
    macAddr[3] = (uint8_t)((val & 0x00FF0000U) >> 16U);
    macAddr[2] = (uint8_t)((val & 0xFF000000U) >> 24U);

    val = CSL_REG32_RD(&mmrRegs->MAC_ID1);

    macAddr[1] = (uint8_t)((val & 0x000000FFU) >> 0U);
    macAddr[0] = (uint8_t)((val & 0x0000FF00U) >> 8U);

    return;
}

uint32_t CpswSoc_getMacCaps(Cpsw_Type cpswType,
                            Cpsw_MacPort portNum)
{
    uint32_t linkCaps = 0U;

    if (((cpswType == CPSW_2G) && (portNum == CPSW_MAC_PORT_0)) ||
        (cpswType == CPSW_9G))
    {
        linkCaps = CPSW_LINK_CAP_HD10 | CPSW_LINK_CAP_FD10 |
                   CPSW_LINK_CAP_HD100 | CPSW_LINK_CAP_FD100 |
                   CPSW_LINK_CAP_FD1000;
    }

    return linkCaps;
}

const CpswSoc_CpswIpInfoTbl *CpswSoc_getIPInfoTbl(void)
{
    return &cpswIpInfoTbl;
}


int32_t CpswSoc_validateQsgmiiCfg(Cpsw_Type cpswType)
{
    int32_t status = CPSW_SOK;
    Cpsw_MacPort qsgmii0Port, qsgmii1Port;
    uint32_t subPortCnt;
    Cpsw_MacPort qsgmiiSubPorts[CPSW_QSGMII_SUB_PORT_MAX];

    if (CPSW_9G == cpswType)
    {
        status = CpswSoc_getQsgmiiCfg(cpswType,
                                      &qsgmii0Port,
                                      &qsgmii1Port,
                                      &qsgmiiSubPorts[0U],
                                      &subPortCnt);
        if (CPSW_SOK == status)
        {
            /* error checks */
            if ( (subPortCnt != 0U) &&
                 (CPSW_MAC_PORT_INV == qsgmii0Port) &&
                 (CPSW_MAC_PORT_INV == qsgmii1Port) )
            {
                /* QSGMII_SUB ports without a QSGMII port is a configuration error */
                status += CPSW_EINVALID_PARAMS;
            }

            if ( (subPortCnt > CPSW_QSGMII_SUB_PER_MAIN_MAX) &&
                 (CPSW_MAC_PORT_INV == qsgmii1Port) )
            {
               /* Four or more QSGMII_SUB ports without two QSGMII ports is a configuration error */
                status += CPSW_EINVALID_PARAMS;
            }
        }
    }

    return status;
}

int32_t CpswSoc_mapPort2QsgmiiId(Cpsw_Type cpswType,
                                 Cpsw_MacPort portNum,
                                 uint32_t *qsgmiiId)
{
    int32_t status;
    uint32_t i, subPortCnt;
    Cpsw_MacPort qsgmii0Port, qsgmii1Port;
    Cpsw_MacPort qsgmiiSubPorts[CPSW_QSGMII_SUB_PORT_MAX];

    /* Translate portNum to QSGMII Id.
     * In QSGMII mode, first QSGMII main port used is QSGMII ID 0 and next  QSGMII ID 1.
     * For example – if MAC port 0 and port 3 are configured in QSGMII mode. Port 0 is QSMII 0
     * and 3 QSGMII1. Software is transparent to this selection */

    status = CpswSoc_getQsgmiiCfg(cpswType,
                                  &qsgmii0Port,
                                  &qsgmii1Port,
                                  &qsgmiiSubPorts[0U],
                                  &subPortCnt);
    if (CPSW_SOK == status)
    {
        if (portNum == qsgmii0Port)
        {
            *qsgmiiId = 0U;
        }
        else if (portNum == qsgmii1Port)
        {
            *qsgmiiId = 1U;
        }
        else
        {
            if (CPSW_MAC_PORT_INV != qsgmii1Port)
            {

                /* It is mandatory to have 3 sub ports to QSGMII0 when 2 QSGMIIs are used */
                for (i = 0U; i < CPSW_QSGMII_SUB_PER_MAIN_MAX; i++)
                {
                    if (qsgmiiSubPorts[i] == portNum)
                    {
                        *qsgmiiId = 0U;
                        break;
                    }
                }

                if (CPSW_MAC_PORT_INV == *qsgmiiId)
                {
                    *qsgmiiId = 1U;
                }
            }
            else
            {
                *qsgmiiId = 0U;
            }
        }
    }

    return status;
}

static int32_t CpswSoc_getQsgmiiCfg(Cpsw_Type cpswType,
                                    Cpsw_MacPort *qsgmii0Port,
                                    Cpsw_MacPort *qsgmii1Port,
                                    Cpsw_MacPort qsgmiiSubPorts[CPSW_QSGMII_SUB_PORT_MAX],
                                    uint32_t *subPortCnt)
{
    int32_t status;
    uint32_t i, qsgmiiCnt = 0U;
    CpswMac_LayerType enetLayer;
    CpswMac_SublayerType enetSublayer;

    *qsgmii0Port = CPSW_MAC_PORT_INV;
    *qsgmii1Port = CPSW_MAC_PORT_INV;

    for (i = 0U; i < CPSW_QSGMII_SUB_PORT_MAX; i++)
    {
        qsgmiiSubPorts[i] = CPSW_MAC_PORT_INV;
    }

    for (i = 0U; i < 8U; i++)
    {
        status = CpswSoc_getEnetControl(cpswType,
                                        (Cpsw_MacPort)i,
                                        &enetLayer,
                                        &enetSublayer);
        if (CPSW_SOK == status)
        {
            if ((enetLayer == CPSW_MAC_LAYER_GMII) &&
                (enetSublayer == CPSW_MAC_SUBLAYER_QUAD_SERIAL_MAIN))
            {
                if (CPSW_MAC_PORT_INV == *qsgmii0Port)
                {
                    *qsgmii0Port = (Cpsw_MacPort)i;
                }
                else if (CPSW_MAC_PORT_INV == *qsgmii1Port)
                {
                    *qsgmii1Port = (Cpsw_MacPort)i;
                }
                else
                {
                    /* More than two QSGMII main ports is a configuration error */
                    status = CPSW_EINVALID_PARAMS;
                    break;
                }
            }
            else if ((enetLayer == CPSW_MAC_LAYER_GMII) &&
                     (enetSublayer == CPSW_MAC_SUBLAYER_QUAD_SERIAL_SUB))
            {
                if (qsgmiiCnt >= CPSW_QSGMII_SUB_PORT_MAX)
                {
                    /* We can have maximum of 6U sub ports in the J721E */
                    status = CPSW_EINVALID_PARAMS;
                    break;
                }
                else
                {
                    qsgmiiSubPorts[qsgmiiCnt++] = (Cpsw_MacPort)i;
                    *subPortCnt  = qsgmiiCnt;
                }

            }
            else
            {
                /* Dont care */
            }
        }
    }

    return status;
}


