/*
 *  Copyright (c) Texas Instruments Incorporated 2018
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*!
 * \file     cpsw_port_priv.h
 *
 * \brief    This file contains the private definitions of the host port
 *           submodule.
 */

/*!
 * \addtogroup CPSW_HOSTPORT_API
 * @{
 */

#ifndef CPSW_HOSTPORT_PRIV_H_
#define CPSW_HOSTPORT_PRIV_H_

/* ========================================================================== */
/*                             Include Files                                  */
/* ========================================================================== */

#include "cpsw_priv.h"

#ifdef __cplusplus
extern "C" {
#endif

/* ========================================================================== */
/*                                 Macros                                     */
/* ========================================================================== */

#define CPSW_IOCTL_HOSTPORT_PRIV_BASE         (CPSW_IOCTL_HOSTPORT_BASE | \
                                               CPSW_IOCTL_TYPE_PRIVATE)

/* ========================================================================== */
/*                         Structures and Enums                               */
/* ========================================================================== */

/*!
 * \brief Host port private IOCTL commands
 */
typedef enum CpswHostPort_PrivateIoctls_e
{
    /*!
     * \brief Set flow ID offset
     *
     * Offset value which is added to the transmit (egress) Flow Id
     *
     * IOCTL parameters:
     *   inArgs: uint32_t
     *  outArgs: None
     */
    CPSW_HOSTPORT_SET_FLOW_ID_OFFSET = CPSW_IOCTL_HOSTPORT_PRIV_BASE | CPSW_IOCTL_MIN(0U),
} CpswHostPort_PrivateIoctls;

/*!
 * \brief Host port submodule object
 */
typedef struct CpswHostPort_Obj_s
{
    /*! Magic value to identify whether object handle is valid or not */
    Cpsw_Magic magic;

    /*! CPSW instance type */
    Cpsw_Type cpswType;

    /*! CPSW register overlay */
    CSL_Xge_cpswRegs *cpswRegs;
}
CpswHostPort_Obj;

/* ========================================================================== */
/*                         Global Variables Declarations                      */
/* ========================================================================== */

/* None */

/* ========================================================================== */
/*                          Function Declarations                             */
/* ========================================================================== */

void CpswHostPort_initParams(CpswHostPort_Config *cfg);

CpswHostPort_Handle CpswHostPort_open(Cpsw_Type cpswType,
                                      const CpswHostPort_Config *cfg);

CpswHostPort_Handle CpswHostPort_reopen(Cpsw_Type cpswType);

void CpswHostPort_close(CpswHostPort_Handle hPort);

int32_t CpswHostPort_ioctl(CpswHostPort_Handle hPort,
                           uint32_t cmd,
                           Cpsw_IoctlPrms *prms);

/* ========================================================================== */
/*                        Deprecated Function Declarations                    */
/* ========================================================================== */

/* None */

#ifdef __cplusplus
}
#endif

#endif /* CPSW_HOSTPORT_PRIV_H_ */

/* @} */
