/*
 *  Copyright (c) Texas Instruments Incorporated 2018
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 *  \file cpsw_osal.h
 *
 *  \brief CPSW OSAL header file.
 */

#ifndef CPSW_OSAL_H_
#define CPSW_OSAL_H_

/* ========================================================================== */
/*                             Include Files                                  */
/* ========================================================================== */

#include <ti/drv/cpsw/cpsw.h>
#include <ti/drv/cpsw/include/cpsw_types.h>

#ifdef __cplusplus
extern "C" {
#endif

/* ========================================================================== */
/*                           Macros & Typedefs                                */
/* ========================================================================== */

/* ========================================================================== */
/*                         Structure Declarations                             */
/* ========================================================================== */

/* ========================================================================== */
/*                          Function Declarations                             */
/* ========================================================================== */

void CpswOsal_open(const CpswOsal_Prms *pOsalPrms);

void CpswOsal_close(void);

void *CpswOsal_registerIntr(CpswOsal_IsrFxn isrFxn,
                            uint32_t coreIntrNum,
                            uint32_t intrPriority,
                            void *arg);

void CpswOsal_cacheInv(const void *addr,
                       int32_t size);

void CpswOsal_cacheWb(const void *addr,
                      int32_t size);

void CpswOsal_cacheWbInv(const void *addr,
                         int32_t size);

void CpswOsal_unRegisterIntr(void *hwiHandle);

void CpswOsal_disableIntr(uint32_t coreIntrNum);

void *CpswOsal_mutexCreate(void);

void CpswOsal_mutexDelete(void *mutexHandle);

void CpswOsal_mutexLock(void *mutexHandle);

void CpswOsal_mutexUnlock(void *mutexHandle);

void CpswOsal_restoreAllIntr(uintptr_t cookie);

uintptr_t CpswOsal_disableAllIntr(void);

uint32_t CpswOsal_timerRead(void);

uint32_t CpswOsal_timerGetDiff(uint32_t startTime);

/* ========================================================================== */
/*                       Static Function Definitions                          */
/* ========================================================================== */

/* None */

#ifdef __cplusplus
}
#endif

#endif /* #ifndef CPSW_OSAL_H_ */
