/*
 *  Copyright (c) Texas Instruments Incorporated 2018
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*!
 * \file     cpsw_stats_priv.h
 *
 * \brief    This file contains the private definitions of the statistics
 *           submodule.
 */

/*!
 * \addtogroup CPSW_STATS_API
 * @{
 */

#ifndef CPSW_STATS_PRIV_H_
#define CPSW_STATS_PRIV_H_

/* ========================================================================== */
/*                             Include Files                                  */
/* ========================================================================== */

#include "cpsw_types.h"
#include "cpsw_priv.h"

#ifdef __cplusplus
extern "C" {
#endif

/* ========================================================================== */
/*                                 Macros                                     */
/* ========================================================================== */

#define CPSW_STATS_PRIVATE_IOCTL(x)           (CPSW_IOCTL_STATS_BASE |   \
                                               CPSW_IOCTL_TYPE_PRIVATE | \
                                               CPSW_IOCTL_MIN(x))

/* ========================================================================== */
/*                         Structures and Enums                               */
/* ========================================================================== */

/*!
 * \brief Statistics private IOCTL commands
 */
typedef enum CpswStats_PrivateIoctls_e
{
    /*!
     * \brief Synchronize hardware and software statistics
     *
     * IOCTL parameters:
     *   inArgs: None
     *  outArgs: None
     */
    CPSW_STATS_IOCTL_SYNC = CPSW_STATS_PRIVATE_IOCTL(0U),
} CpswStats_PrivateIoctls;

/*!
 * \brief Statistic submodule object
 */
typedef struct CpswStats_Obj_s
{
    /*! Magic value to identify whether object handle is valid or not */
    Cpsw_Magic magic;

    /*! CPSW instance type */
    Cpsw_Type cpswType;

    /*! Host port statistics */
    CpswStats_PortStats hostPortStats;

    /*! Ethernet ports statistics */
    CpswStats_PortStats macPortStats[CPSW_MAC_PORT_NUM];

    /*! CPSW register overlay */
    CSL_Xge_cpswRegs *cpswRegs;
} CpswStats_Obj;

/* ========================================================================== */
/*                         Global Variables Declarations                      */
/* ========================================================================== */

/* None */

/* ========================================================================== */
/*                          Function Declarations                             */
/* ========================================================================== */

CpswStats_Handle CpswStats_open(Cpsw_Type cpswType);

CpswStats_Handle CpswStats_reopen(Cpsw_Type cpswType);

void CpswStats_close(CpswStats_Handle hStats);

int32_t CpswStats_ioctl(CpswStats_Handle hStats,
                        uint32_t cmd,
                        Cpsw_IoctlPrms *prms);

/* ========================================================================== */
/*                        Deprecated Function Declarations                    */
/* ========================================================================== */

/* None */

#ifdef __cplusplus
}
#endif

#endif /* CPSW_STATS_PRIV_H_ */

/* @} */
