/*
 *  Copyright (c) Texas Instruments Incorporated 2018
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*!
 * \file     cpsw_switching_logic.c
 *
 * \brief    This file contains the cpsw_switching_logic test implementation.
 */

/* ========================================================================== */
/*                             Include Files                                  */
/* ========================================================================== */

#include <string.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <xdc/std.h>

#include <ti/drv/uart/UART_stdio.h>
#include <ti/csl/csl_cpswitch.h>

#include <ti/drv/udma/udma.h>
#include <ti/drv/cpsw/cpsw.h>
#include <ti/drv/cpsw/examples/cpsw_apputils/inc/cpsw_appmemutils_cfg.h>
#include <ti/drv/cpsw/examples/cpsw_apputils/inc/cpsw_appmemutils.h>
#include <ti/drv/cpsw/examples/cpsw_apputils/inc/cpsw_apputils.h>
#include <ti/drv/cpsw/examples/cpsw_apputils/inc/cpswapp_ethutils.h>

#include <ti/osal/osal.h>

#include <ti/board/board.h>

#include "cpsw_test_base.h"
#include "cpsw_test_autolearn.h"

/* ========================================================================== */
/*                           Macros & Typedefs                                */
/* ========================================================================== */

/* ========================================================================== */
/*                         Structures and Enums                               */
/* ========================================================================== */

/* ========================================================================== */
/*                 Internal Function Declarations                             */
/* ========================================================================== */
static int32_t CpswApp_lookupAleEntry(CpswTestTaskObj *taskObj,
                                      uint8_t macAddr[]);

/* ========================================================================== */
/*                            Global Variables                                */
/* ========================================================================== */

/* Test ALE Entry Address */
static uint8_t testAddr[CPSW_MAC_ADDR_LEN] =
{
    0x40U, 0xcdU, 0xefU, 0xfeU, 0xdcU, 0xbaU
};

/* ========================================================================== */
/*                          Function Definitions                              */
/* ========================================================================== */

int32_t CpswTestAutoLearn_Run(CpswTestTaskObj *taskObj)
{
    int32_t status;

    status = CpswTestCommon_Run(taskObj);
    if (status == CPSW_SOK)
    {
        status = CpswApp_lookupAleEntry(taskObj, &testAddr[0U]);
    }

    return status;
}

static int32_t CpswApp_lookupAleEntry(CpswTestTaskObj *taskObj,
                                      uint8_t macAddr[])
{
    int32_t status = CPSW_SOK;
    Cpsw_IoctlPrms prms;
    CpswAle_MacAddrInfo macAddrInfo;
    CpswAle_GetUcastEntryOutArgs getUcastInArgs;
    CpswTestStateObj *stateObj = &taskObj->stateObj;

    memcpy(&macAddrInfo.addr[0], macAddr, sizeof(macAddrInfo.addr));
    macAddrInfo.vlanId = 0U;

    CPSW_IOCTL_SET_INOUT_ARGS(&prms, &macAddrInfo, &getUcastInArgs);

    status = Cpsw_ioctl(stateObj->hCpsw, stateObj->coreId, CPSW_ALE_IOCTL_LOOKUP_UNICAST,
                        &prms);
    if (status != CPSW_SOK)
    {
        // testPass = false;
        CpswAppUtils_print("ALE auto learn test pass \n");
        status = CPSW_SOK;
    }

    return status;
}

void CpswTestAutoLearn_updatePortLinkCfg(Cpsw_OpenPortLinkInArgs *pLinkArgs,
                                         Cpsw_MacPort portNum)
{
    pLinkArgs->macConfig.enableLoopback = FALSE;
}

void CpswTestAutoLearn_setOpenPrms(CpswTestTaskObj *taskObj,
                                   Cpsw_Config *pCpswCfg,
                                   CpswOsal_Prms *pOsalPrms,
                                   CpswUtils_Prms *pUtilsPrms)
{
    pCpswCfg->aleConfig.portCfg[0].learningCfg.noLearn = true;
    pCpswCfg->aleConfig.portCfg[1].learningCfg.noLearn = true;
}
