/*
 *  Copyright (c) Texas Instruments Incorporated 2018
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 *  \file cpsw_test_base.h
 *
 *  \brief cpsw_test_base test private header file.
 */

#ifndef CPSW_TEST_BASE_H_
#define CPSW_TEST_BASE_H_

/* ========================================================================== */
/*                             Include Files                                  */
/* ========================================================================== */
#include <ti/drv/udma/udma.h>
#include "cpsw_test.h"

#ifdef __cplusplus
extern "C" {
#endif

#define CPSW_TEST_COMMON_PKT_LEN               (1500U)

#define CPSW_TEST_COMMON_NUM_LOOP               (10U)
#define CPSW_TEST_COMMON_PKT_SENDCOUNT          (1000U)
#define CPSW_TEST_COMMON_PKT_RECVCOUNT          (1000U)

typedef struct CpswTestMacPortList_s
{
    uint32_t numMacPorts;

    Cpsw_MacPort macPortList[CPSW_MAC_PORT_NUM];
} CpswTestMacPortList_t;

/* ========================================================================== */
/*                       Static Function Definitions                          */
/* ========================================================================== */

int32_t CpswTestCommon_Init(CpswTestTaskObj *taskObj);

int32_t CpswTestCommon_Run(CpswTestTaskObj *taskObj);

int32_t CpswTestCommon_DeInit(CpswTestTaskObj *taskObj);

int32_t CpswTestCommon_processRxPkt(CpswTestTaskObj *taskObj,
                                    uint32_t rxFlowId,
                                    uint32_t pktNum,
                                    CpswDma_PktInfo *pktInfo,
                                    bool *testComplete);

Cpsw_Handle CpswTestCommon_getCpswHandle(CpswTestTaskObj *taskObj);

uint32_t CpswTestCommon_retrieveFreeTxPkts(CpswTestTaskObj *taskObj,
                                           uint32_t txCfgIndex);

uint32_t CpswTestCommon_receivePkts(CpswTestTaskObj *taskObj,
                                    uint32_t rxCfgIndex);

extern Udma_DrvHandle gUdmaDrvHandle[CPSW_COUNT];
void CpswTestCommon_disableHostAleEntrySecure(CpswTestTaskObj *taskObj);

int32_t CpswTestCommon_getAleMulticastEntry(CpswTestTaskObj *taskObj,
                                            uint8_t macAddr[],
                                            uint32_t vlanId,
                                            uint32_t numIgnBits,
                                            uint32_t *pPortMask);

int32_t CpswTestCommon_setAleMulticastEntry(CpswTestTaskObj *taskObj,
                                            uint8_t macAddr[],
                                            uint32_t vlanId,
                                            uint32_t numIgnBits,
                                            uint32_t portMask);

void  CpswTestCommon_createRxSem(CpswTestTaskObj *taskObj,
                                 uint32_t rxCfgIndex);

void CpswTestCommon_rxIsrFxn(void *appData);

void CpswTestCommon_discardRxPkts(CpswTestTaskObj *taskObj,
                                  uint32_t rxCfgIndex);

void  CpswTestCommon_getMacPortList(uint32_t macPortBitMask,
                                    CpswTestMacPortList_t *macPortList);

void    CpswTestCommon_waitForPortLink(CpswTestTaskObj *taskObj);

uint32_t CpswTestCommon_getMacPortAleMask(CpswTestTaskObj *taskObj);

int32_t CpswTestCommon_setTxPktInfo(CpswTestTaskObj *taskObj,
                                    uint32_t txChIndex,
                                    uint32_t pktNum,
                                    CpswDma_PktInfo *pktInfo,
                                    bool *testComplete);

int32_t CpswTestCommon_pktTxRx(CpswTestTaskObj *taskObj);

void CpswTestCommon_getDefaultHostMacAddr(uint8_t *macAddr);


#ifdef __cplusplus
}
#endif

#endif /* #ifndef CPSW_TEST_BASE_H_ */
