/*
 *  Copyright (c) Texas Instruments Incorporated 2018
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*!
 * \file     cpsw_test_pvid.c
 *
 * \brief    This file contains the cpsw_test_pvid test implementation.
 */

/* ========================================================================== */
/*                             Include Files                                  */
/* ========================================================================== */

#include <string.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <xdc/std.h>

#include <ti/drv/uart/UART_stdio.h>
#include <ti/csl/csl_cpswitch.h>

#include <ti/drv/udma/udma.h>
#include <ti/drv/cpsw/cpsw.h>
#include <ti/drv/cpsw/examples/cpsw_apputils/inc/cpsw_appmemutils_cfg.h>
#include <ti/drv/cpsw/examples/cpsw_apputils/inc/cpsw_appmemutils.h>
#include <ti/drv/cpsw/examples/cpsw_apputils/inc/cpsw_apputils.h>
#include <ti/drv/cpsw/examples/cpsw_apputils/inc/cpswapp_ethutils.h>

#include <ti/osal/osal.h>

#include <ti/board/board.h>

#include "cpsw_test_base.h"
#include "cpsw_test_entry.h"
#include "cpsw_test_pvid.h"

/* ========================================================================== */
/*                           Macros & Typedefs                                */
/* ========================================================================== */
#define ETH_TEST_MAC_PORT_VLAN_ID_BASE            (100U)
#define ETH_TEST_HOST_PORT_VLAN_ID                (200U)

/* ========================================================================== */
/*                         Structures and Enums                               */
/* ========================================================================== */

/* ========================================================================== */
/*                 Internal Function Declarations                             */
/* ========================================================================== */

/* ========================================================================== */
/*                            Global Variables                                */
/* ========================================================================== */

/* ========================================================================== */
/*                          Function Definitions                              */
/* ========================================================================== */

int32_t CpswTestPvid_Run(CpswTestTaskObj *taskObj)
{
    int32_t status = CPSW_SOK;

    status = CpswTestCommon_Run(taskObj);

    return status;
}

void CpswTestPvid_setOpenPrms(CpswTestTaskObj *taskObj,
                              Cpsw_Config *pCpswCfg,
                              CpswOsal_Prms *pOsalPrms,
                              CpswUtils_Prms *pUtilsPrms)
{
    Cpsw_MacPort i;

    pCpswCfg->aleConfig.modeFlags =
        (CPSW_ALE_CONFIG_MASK_ALE_MODULE_ENABLE | CPSW_ALE_CONFIG_MASK_UNKNOWN_UNICAST_FLOOD2HOST);
    pCpswCfg->aleConfig.vlanConfig.unknownUnregMcastFloodMask = 0x0;
    pCpswCfg->aleConfig.vlanConfig.unknownRegMcastFloodMask   = 0x0;
    pCpswCfg->aleConfig.vlanConfig.unknownVlanMemberListMask  = CPSW_ALE_ALL_PORTS_MASK;
    pCpswCfg->aleConfig.vlanConfig.aleVlanAwareMode           = TRUE;
    pCpswCfg->aleConfig.vlanConfig.autoLearnWithVLAN          = TRUE;
    pCpswCfg->aleConfig.vlanConfig.cpswVlanAwareMode          = TRUE;
    pCpswCfg->aleConfig.nwSecCfg.enableVid0Mode               = TRUE;

    pCpswCfg->aleConfig.portCfg[CPSW_ALE_HOST_PORT_NUM].learningCfg.noLearn              = FALSE;
    pCpswCfg->aleConfig.portCfg[CPSW_ALE_HOST_PORT_NUM].vlanCfg.dropUntagged             = FALSE;
    pCpswCfg->aleConfig.portCfg[CPSW_ALE_HOST_PORT_NUM].pvidCfg.unregMcastFloodMask      = 0x0;
    pCpswCfg->aleConfig.portCfg[CPSW_ALE_HOST_PORT_NUM].pvidCfg.regMcastFloodMask        = CPSW_ALE_ALL_PORTS_MASK;
    pCpswCfg->aleConfig.portCfg[CPSW_ALE_HOST_PORT_NUM].pvidCfg.forceUntaggedEgressMask  = 0;
    pCpswCfg->aleConfig.portCfg[CPSW_ALE_HOST_PORT_NUM].pvidCfg.noLearnMask              = 0x0;
    pCpswCfg->aleConfig.portCfg[CPSW_ALE_HOST_PORT_NUM].pvidCfg.vidIngressCheck          = 0x0;
    pCpswCfg->aleConfig.portCfg[CPSW_ALE_HOST_PORT_NUM].pvidCfg.limitIPNxtHdr            = false;
    pCpswCfg->aleConfig.portCfg[CPSW_ALE_HOST_PORT_NUM].pvidCfg.disallowIPFragmentation  = false;
    pCpswCfg->aleConfig.portCfg[CPSW_ALE_HOST_PORT_NUM].pvidCfg.vlanIdInfo.outerVlanFlag = false;
    pCpswCfg->aleConfig.portCfg[CPSW_ALE_HOST_PORT_NUM].pvidCfg.vlanIdInfo.vlanId        = ETH_TEST_HOST_PORT_VLAN_ID;
    pCpswCfg->aleConfig.portCfg[CPSW_ALE_HOST_PORT_NUM].pvidCfg.vlanMemberList           = CPSW_ALE_ALL_PORTS_MASK;

    for (i = CPSW_MAC_PORT_FIRST; i <= CPSW_MAC_PORT_LAST; i++)
    {
        pCpswCfg->aleConfig.portCfg[CPSW_ALE_MACPORT_TO_ALEPORT(i)].learningCfg.noLearn              = FALSE;
        pCpswCfg->aleConfig.portCfg[CPSW_ALE_MACPORT_TO_ALEPORT(i)].vlanCfg.dropUntagged             = FALSE;
        pCpswCfg->aleConfig.portCfg[CPSW_ALE_MACPORT_TO_ALEPORT(i)].pvidCfg.unregMcastFloodMask      = 0x0;
        pCpswCfg->aleConfig.portCfg[CPSW_ALE_MACPORT_TO_ALEPORT(i)].pvidCfg.regMcastFloodMask        = CPSW_ALE_ALL_PORTS_MASK;
        pCpswCfg->aleConfig.portCfg[CPSW_ALE_MACPORT_TO_ALEPORT(i)].pvidCfg.forceUntaggedEgressMask  = 0;
        pCpswCfg->aleConfig.portCfg[CPSW_ALE_MACPORT_TO_ALEPORT(i)].pvidCfg.noLearnMask              = 0x0;
        pCpswCfg->aleConfig.portCfg[CPSW_ALE_MACPORT_TO_ALEPORT(i)].pvidCfg.vidIngressCheck          = 0x0;
        pCpswCfg->aleConfig.portCfg[CPSW_ALE_MACPORT_TO_ALEPORT(i)].pvidCfg.limitIPNxtHdr            = false;
        pCpswCfg->aleConfig.portCfg[CPSW_ALE_MACPORT_TO_ALEPORT(i)].pvidCfg.disallowIPFragmentation  = false;
        pCpswCfg->aleConfig.portCfg[CPSW_ALE_MACPORT_TO_ALEPORT(i)].pvidCfg.vlanIdInfo.outerVlanFlag = false;
        pCpswCfg->aleConfig.portCfg[CPSW_ALE_MACPORT_TO_ALEPORT(i)].pvidCfg.vlanIdInfo.vlanId        = ETH_TEST_MAC_PORT_VLAN_ID_BASE + CPSW_NORMALIZE_MACPORT(i);
        pCpswCfg->aleConfig.portCfg[CPSW_ALE_MACPORT_TO_ALEPORT(i)].pvidCfg.vlanMemberList           = CPSW_ALE_ALL_PORTS_MASK;
    }

    pCpswCfg->hostPortConfig.vlanCfg.portPri = 7;
    pCpswCfg->hostPortConfig.vlanCfg.portCfi = 0;
    pCpswCfg->hostPortConfig.vlanCfg.portVID = ETH_TEST_HOST_PORT_VLAN_ID;
    pCpswCfg->vlanConfig.vlanAware           = TRUE;
}

void CpswTestPvid_updatePortLinkCfg(Cpsw_OpenPortLinkInArgs *pLinkArgs,
                                    Cpsw_MacPort portNum)
{
    pLinkArgs->macConfig.enableLoopback = FALSE;

    pLinkArgs->macConfig.vlanConfig.portPri = CPSW_NORMALIZE_MACPORT(portNum);
    pLinkArgs->macConfig.vlanConfig.portCfi = 0;
    pLinkArgs->macConfig.vlanConfig.portVID = ETH_TEST_MAC_PORT_VLAN_ID_BASE + CPSW_NORMALIZE_MACPORT(portNum);
}

int32_t CpswTestPvid_processRxPkt(CpswTestTaskObj *taskObj,
                                  uint32_t rxFlowId,
                                  uint32_t pktNum,
                                  CpswDma_PktInfo *pktInfo,
                                  bool *testComplete)
{
    int32_t status = CPSW_SOK;
    EthVlanFrame *frame;
    uint32_t vid;
    uint32_t pcp;

    /* Consume the packet by just printing its content */
    frame = (EthVlanFrame *)pktInfo->bufPtr;

    vid = (ntohs(frame->hdr.tci) & 0xFFFU);
    pcp = (((htons(frame->hdr.tci)) >> 13) & 0x7);

    if ((vid == ETH_TEST_MAC_PORT_VLAN_ID_BASE + CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_3)) && (pcp == CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_3)))
    {
        status = CPSW_SOK;
    }
    else
    {
        Cpsw_IoctlPrms prms;
        CpswTestStateObj *stateObj = &taskObj->stateObj;

        CPSW_IOCTL_SET_NO_ARGS(&prms);
        status = Cpsw_ioctl(stateObj->hCpsw, stateObj->coreId, CPSW_ALE_IOCTL_DUMP_TABLE,
                            &prms);
        CpswAppUtils_assert(status == CPSW_SOK);

        CPSW_IOCTL_SET_NO_ARGS(&prms);
        status = Cpsw_ioctl(stateObj->hCpsw, stateObj->coreId, CPSW_ALE_IOCTL_DUMP_POLICER_ENTRIES,
                            &prms);

        CpswAppUtils_assert(status == CPSW_SOK);
        status = CPSW_EFAIL;
    }

    *testComplete = FALSE;
    return status;
}
