/*
 *  Copyright (c) Texas Instruments Incorporated 2018
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*!
 * \file     cpsw_test_stats.c
 *
 * \brief    This file contains the cpsw stats test implementation.
 */

/* ========================================================================== */
/*                             Include Files                                  */
/* ========================================================================== */

#include <string.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>

#include <xdc/std.h>

#include <ti/drv/uart/UART_stdio.h>
#include <ti/csl/csl_cpswitch.h>

#include <ti/drv/udma/udma.h>
#include <ti/drv/cpsw/cpsw.h>
#include <ti/drv/cpsw/examples/cpsw_apputils/inc/cpsw_appmemutils_cfg.h>
#include <ti/drv/cpsw/examples/cpsw_apputils/inc/cpsw_appmemutils.h>
#include <ti/drv/cpsw/examples/cpsw_apputils/inc/cpsw_apputils.h>
#include <ti/drv/cpsw/examples/cpsw_apputils/inc/cpswapp_ethutils.h>

#include <ti/osal/osal.h>

#include <ti/board/board.h>

#include "cpsw_test_entry.h"
#include "cpsw_test_base.h"
#include "cpsw_test_stats.h"

/* ========================================================================== */
/*                           Macros & Typedefs                                */
/* ========================================================================== */

/* ========================================================================== */
/*                         Structures and Enums                               */
/* ========================================================================== */

/* ========================================================================== */
/*                 Internal Function Declarations                             */
/* ========================================================================== */

static void CpswTestStats_testStatsShowStats(CpswTestTaskObj *taskObj);

/* ========================================================================== */
/*                            Global Variables                                */
/* ========================================================================== */

/* ========================================================================== */
/*                          Function Definitions                              */
/* ========================================================================== */

int32_t CpswTestStats_PostProcessRxPkt(CpswTestTaskObj *taskObj,
                                       uint32_t rxCfgIndex)
{
    /* Print CPSW statistics of all ports */
    CpswTestStats_testStatsShowStats(taskObj);
    return CPSW_SOK;
}

static void CpswTestStats_testStatsShowStats(CpswTestTaskObj *taskObj)
{
    Cpsw_IoctlPrms prms;
    CpswStats_GenericMacPortInArgs inArgs;
    CpswStats_PortStats portStats;
    CpswStats_HostPort_2g *hostPortStats =
        (CpswStats_HostPort_2g *)&portStats;
    CpswStats_MacPort_2g *macPortStats =
        (CpswStats_MacPort_2g *)&portStats;
    int32_t status             = CPSW_SOK;
    CpswTestStateObj *stateObj = &taskObj->stateObj;

    CPSW_IOCTL_SET_OUT_ARGS(&prms, &portStats);
    status =
        Cpsw_ioctl(stateObj->hCpsw, stateObj->coreId, CPSW_STATS_IOCTL_GET_HOSTPORT_STATS,
                   &prms);
    if (status == CPSW_SOK)
    {
        CpswAppUtils_print("\n Port 0 Statistics\n");
        CpswAppUtils_print("-----------------------------------------\n");
        CpswAppUtils_printHostPortStats2G(hostPortStats);
        CpswAppUtils_print("\n");

        CpswAppUtils_print("netOctets check: %s (got %llu)\n",
                           (hostPortStats->netOctets <
                            0xFFFFFFFFLLU) ? "FAILED" : "PASSED",
                           hostPortStats->netOctets);
    }
    else
    {
        CpswAppUtils_print("CpswUtApp_showStats() failed to get host stats: %d\n",
                           status);
    }

    if (status == CPSW_SOK)
    {
        inArgs.portNum = CPSW_MAC_PORT_0;
        CPSW_IOCTL_SET_INOUT_ARGS(&prms, &inArgs, macPortStats);
        status =
            Cpsw_ioctl(stateObj->hCpsw, stateObj->coreId, CPSW_STATS_IOCTL_GET_MACPORT_STATS,
                       &prms);
        if (status == CPSW_SOK)
        {
            CpswAppUtils_print("\n Port 1 Statistics\n");
            CpswAppUtils_print("-----------------------------------------\n");
            CpswAppUtils_printMacPortStats2G(macPortStats);
            CpswAppUtils_print("\n");

            CpswAppUtils_print("netOctets check: %s (got %llu)\n",
                               (macPortStats->netOctets <
                                0xFFFFFFFFLLU) ? "FAILED" : "PASSED",
                               macPortStats->netOctets);
        }
        else
        {
            CpswAppUtils_print("CpswApp_showStats() failed to get MAC stats: %d\n",
                               status);
        }
    }
}

void CpswTestStats_updatePortLinkCfg(Cpsw_OpenPortLinkInArgs *pLinkArgs,
                                     Cpsw_MacPort portNum)
{
    pLinkArgs->macConfig.enableLoopback = TRUE;
    pLinkArgs->interface.variantType    = CPSW_MAC_VARIANT_FORCED;
}
