/*
 *  Copyright (c) Texas Instruments Incorporated 2018
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*!
 * \file     cpsw_test_traffic_shaping.c
 *
 * \brief    This file contains the cpsw_test_traffic_shaping test implementation.
 */

/* ========================================================================== */
/*                             Include Files                                  */
/* ========================================================================== */

#include <string.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <xdc/std.h>

#include <ti/drv/uart/UART_stdio.h>
#include <ti/csl/csl_cpswitch.h>

#include <ti/drv/udma/udma.h>
#include <ti/drv/cpsw/cpsw.h>
#include <ti/drv/cpsw/examples/cpsw_apputils/inc/cpsw_appmemutils_cfg.h>
#include <ti/drv/cpsw/examples/cpsw_apputils/inc/cpsw_appmemutils.h>
#include <ti/drv/cpsw/examples/cpsw_apputils/inc/cpsw_apputils.h>
#include <ti/drv/cpsw/examples/cpsw_apputils/inc/cpswapp_ethutils.h>

#include <ti/osal/osal.h>

#include <ti/board/board.h>

#include "cpsw_test_base.h"
#include "cpsw_test_entry.h"
#include "cpsw_test_traffic_shaping.h"

/* ========================================================================== */
/*                           Macros & Typedefs                                */
/* ========================================================================== */
#define ETH_TEST_MAC_PORT_VLAN_ID_BASE            (100U)
#define ETH_TEST_HOST_PORT_VLAN_ID                (200U)

#define CPSW_TEST_TRAFFIC_SHAPING_MBPS(x)        ((x) * 1000000U)
#define CPSW_TEST_TRAFFIC_SHAPING_KBPS(x)        ((x) * 1000U)

#define CPSW_TEST_TRAFFIC_SHAPING_EIR_PRI0       CPSW_TEST_TRAFFIC_SHAPING_KBPS(50)
#define CPSW_TEST_TRAFFIC_SHAPING_CIR_PRI0       CPSW_TEST_TRAFFIC_SHAPING_MBPS(12.5)

#define CPSW_TEST_TRAFFIC_SHAPING_EIR_PRI1       CPSW_TEST_TRAFFIC_SHAPING_KBPS(50)
#define CPSW_TEST_TRAFFIC_SHAPING_CIR_PRI1       CPSW_TEST_TRAFFIC_SHAPING_MBPS(12.5)

#define CPSW_TEST_TRAFFIC_SHAPING_EIR_PRI2       CPSW_TEST_TRAFFIC_SHAPING_KBPS(50)
#define CPSW_TEST_TRAFFIC_SHAPING_CIR_PRI2       CPSW_TEST_TRAFFIC_SHAPING_MBPS(12.5)

#define CPSW_TEST_TRAFFIC_SHAPING_EIR_PRI3       CPSW_TEST_TRAFFIC_SHAPING_KBPS(50)
#define CPSW_TEST_TRAFFIC_SHAPING_CIR_PRI3       CPSW_TEST_TRAFFIC_SHAPING_MBPS(12.5)

#define CPSW_TEST_TRAFFIC_SHAPING_EIR_PRI4       CPSW_TEST_TRAFFIC_SHAPING_KBPS(50)
#define CPSW_TEST_TRAFFIC_SHAPING_CIR_PRI4       CPSW_TEST_TRAFFIC_SHAPING_MBPS(12.5)

#define CPSW_TEST_TRAFFIC_SHAPING_EIR_PRI5       CPSW_TEST_TRAFFIC_SHAPING_KBPS(50)
#define CPSW_TEST_TRAFFIC_SHAPING_CIR_PRI5       CPSW_TEST_TRAFFIC_SHAPING_MBPS(12.5)

#define CPSW_TEST_TRAFFIC_SHAPING_EIR_PRI6       CPSW_TEST_TRAFFIC_SHAPING_KBPS(50)
#define CPSW_TEST_TRAFFIC_SHAPING_CIR_PRI6       CPSW_TEST_TRAFFIC_SHAPING_MBPS(12.5)

#define CPSW_TEST_TRAFFIC_SHAPING_EIR_PRI7       CPSW_TEST_TRAFFIC_SHAPING_KBPS(50)
#define CPSW_TEST_TRAFFIC_SHAPING_CIR_PRI7       CPSW_TEST_TRAFFIC_SHAPING_MBPS(12.5)

#define CPSW_TEST_TRAFFIC_SHAPING_ALE_PRIORITY_PIR_PRI0       CPSW_TEST_TRAFFIC_SHAPING_MBPS(6.25)
#define CPSW_TEST_TRAFFIC_SHAPING_ALE_PRIORITY_CIR_PRI0       CPSW_TEST_TRAFFIC_SHAPING_MBPS(3)

#define CPSW_TEST_TRAFFIC_SHAPING_ALE_PRIORITY_PIR_PRI1       CPSW_TEST_TRAFFIC_SHAPING_MBPS(6.25)
#define CPSW_TEST_TRAFFIC_SHAPING_ALE_PRIORITY_CIR_PRI1       CPSW_TEST_TRAFFIC_SHAPING_MBPS(3)

#define CPSW_TEST_TRAFFIC_SHAPING_ALE_PRIORITY_PIR_PRI2       CPSW_TEST_TRAFFIC_SHAPING_MBPS(6.25)
#define CPSW_TEST_TRAFFIC_SHAPING_ALE_PRIORITY_CIR_PRI2       CPSW_TEST_TRAFFIC_SHAPING_MBPS(3)

#define CPSW_TEST_TRAFFIC_SHAPING_ALE_PRIORITY_PIR_PRI3       CPSW_TEST_TRAFFIC_SHAPING_MBPS(6.25)
#define CPSW_TEST_TRAFFIC_SHAPING_ALE_PRIORITY_CIR_PRI3       CPSW_TEST_TRAFFIC_SHAPING_MBPS(3)

#define CPSW_TEST_TRAFFIC_SHAPING_ALE_PRIORITY_PIR_PRI4       CPSW_TEST_TRAFFIC_SHAPING_MBPS(6.25)
#define CPSW_TEST_TRAFFIC_SHAPING_ALE_PRIORITY_CIR_PRI4       CPSW_TEST_TRAFFIC_SHAPING_MBPS(3)

#define CPSW_TEST_TRAFFIC_SHAPING_ALE_PRIORITY_PIR_PRI5       CPSW_TEST_TRAFFIC_SHAPING_MBPS(6.25)
#define CPSW_TEST_TRAFFIC_SHAPING_ALE_PRIORITY_CIR_PRI5       CPSW_TEST_TRAFFIC_SHAPING_MBPS(3)

#define CPSW_TEST_TRAFFIC_SHAPING_ALE_PRIORITY_PIR_PRI6       CPSW_TEST_TRAFFIC_SHAPING_MBPS(6.25)
#define CPSW_TEST_TRAFFIC_SHAPING_ALE_PRIORITY_CIR_PRI6       CPSW_TEST_TRAFFIC_SHAPING_MBPS(3)

#define CPSW_TEST_TRAFFIC_SHAPING_ALE_PRIORITY_PIR_PRI7       CPSW_TEST_TRAFFIC_SHAPING_MBPS(6.25)
#define CPSW_TEST_TRAFFIC_SHAPING_ALE_PRIORITY_CIR_PRI7       CPSW_TEST_TRAFFIC_SHAPING_MBPS(3)

#define CPSW_TEST_TRAFFIC_SHAPING_ALE_IPDST_PIR_PRI0          CPSW_TEST_TRAFFIC_SHAPING_MBPS(12.5)
#define CPSW_TEST_TRAFFIC_SHAPING_ALE_IPDST_CIR_PRI0          CPSW_TEST_TRAFFIC_SHAPING_MBPS(6.25)

#define CPSW_TEST_TRAFFIC_SHAPING_ALE_IPDST_PIR_PRI1          CPSW_TEST_TRAFFIC_SHAPING_MBPS(12.5)
#define CPSW_TEST_TRAFFIC_SHAPING_ALE_IPDST_CIR_PRI1          CPSW_TEST_TRAFFIC_SHAPING_MBPS(6.25)

#define CPSW_TEST_TRAFFIC_SHAPING_ALE_IPDST_PIR_PRI2          CPSW_TEST_TRAFFIC_SHAPING_MBPS(12.5)
#define CPSW_TEST_TRAFFIC_SHAPING_ALE_IPDST_CIR_PRI2          CPSW_TEST_TRAFFIC_SHAPING_MBPS(6.25)

#define CPSW_TEST_TRAFFIC_SHAPING_ALE_IPDST_PIR_PRI3          CPSW_TEST_TRAFFIC_SHAPING_MBPS(12.5)
#define CPSW_TEST_TRAFFIC_SHAPING_ALE_IPDST_CIR_PRI3          CPSW_TEST_TRAFFIC_SHAPING_MBPS(6.25)

#define CPSW_TEST_TRAFFIC_SHAPING_ALE_IPDST_PIR_PRI4          CPSW_TEST_TRAFFIC_SHAPING_MBPS(12.5)
#define CPSW_TEST_TRAFFIC_SHAPING_ALE_IPDST_CIR_PRI4          CPSW_TEST_TRAFFIC_SHAPING_MBPS(6.25)

#define CPSW_TEST_TRAFFIC_SHAPING_ALE_IPDST_PIR_PRI5          CPSW_TEST_TRAFFIC_SHAPING_MBPS(12.5)
#define CPSW_TEST_TRAFFIC_SHAPING_ALE_IPDST_CIR_PRI5          CPSW_TEST_TRAFFIC_SHAPING_MBPS(6.25)

#define CPSW_TEST_TRAFFIC_SHAPING_ALE_IPDST_PIR_PRI6          CPSW_TEST_TRAFFIC_SHAPING_MBPS(12.5)
#define CPSW_TEST_TRAFFIC_SHAPING_ALE_IPDST_CIR_PRI6          CPSW_TEST_TRAFFIC_SHAPING_MBPS(6.25)

#define CPSW_TEST_TRAFFIC_SHAPING_ALE_IPDST_PIR_PRI7          CPSW_TEST_TRAFFIC_SHAPING_MBPS(12.5)
#define CPSW_TEST_TRAFFIC_SHAPING_ALE_IPDST_CIR_PRI7          CPSW_TEST_TRAFFIC_SHAPING_MBPS(6.25)

#define CPSW_TEST_TRAFFIC_SHAPING_INGRESS_PORT    (CPSW_MAC_PORT_3)
#define CPSW_TEST_TRAFFIC_SHAPING_EGRESS_PORT     (CPSW_MAC_PORT_2)

#define CPSW_TEST_TRAFFIC_SHAPING_CPSW_RATE_LIMIT_DIVFACTOR (32768)

/* ========================================================================== */
/*                         Structures and Enums                               */
/* ========================================================================== */

/* ========================================================================== */
/*                 Internal Function Declarations                             */
/* ========================================================================== */
static int32_t CpswTestTrafficShaping_Config(CpswTestTaskObj *taskObj);

static int32_t CpswTestTrafficShaping_setRxPriority(CpswTestTaskObj *taskObj);

static int32_t CpswTestTrafficShaping_setRxDscpPriority(CpswTestTaskObj *taskObj);

static int32_t CpswTestTrafficShaping_setPolicerPriority(CpswTestTaskObj *taskObj);

static int32_t CpswTestTrafficShaping_setPolicerPriorityDstIpv4(CpswTestTaskObj *taskObj);

static int32_t CpswTestTrafficShaping_AddAleEntry(CpswTestTaskObj *taskObj);

/* ========================================================================== */
/*                            Global Variables                                */
/* ========================================================================== */
static CpswPort_TrafficShapingRates gCpswTestTrafficShaping_priorityRate[CPSW_PRIORITY_NUM] =
{
    [0] = {.committedRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_CIR_PRI0, .excessRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_EIR_PRI0},
    [1] = {.committedRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_CIR_PRI1, .excessRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_EIR_PRI1},
    [2] = {.committedRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_CIR_PRI2, .excessRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_EIR_PRI2},
    [3] = {.committedRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_CIR_PRI3, .excessRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_EIR_PRI3},
    [4] = {.committedRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_CIR_PRI4, .excessRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_EIR_PRI4},
    [5] = {.committedRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_CIR_PRI5, .excessRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_EIR_PRI5},
    [6] = {.committedRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_CIR_PRI6, .excessRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_EIR_PRI6},
    [7] = {.committedRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_CIR_PRI7, .excessRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_EIR_PRI7},
};

typedef struct CpswTestTrafficShapingAleRate_s
{
    uint32_t committedInfoRateBitsPerSec;
    uint32_t peakInfoRateBitsPerSec;
} CpswTestTrafficShapingAleRate_t;

static CpswTestTrafficShapingAleRate_t gCpswTestTrafficShaping_priorityAleRate[CPSW_PRIORITY_NUM] =
{
    [0] = {.committedInfoRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_ALE_PRIORITY_CIR_PRI0, .peakInfoRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_ALE_PRIORITY_PIR_PRI0},
    [1] = {.committedInfoRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_ALE_PRIORITY_CIR_PRI1, .peakInfoRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_ALE_PRIORITY_PIR_PRI1},
    [2] = {.committedInfoRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_ALE_PRIORITY_CIR_PRI2, .peakInfoRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_ALE_PRIORITY_PIR_PRI2},
    [3] = {.committedInfoRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_ALE_PRIORITY_CIR_PRI3, .peakInfoRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_ALE_PRIORITY_PIR_PRI3},
    [4] = {.committedInfoRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_ALE_PRIORITY_CIR_PRI4, .peakInfoRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_ALE_PRIORITY_PIR_PRI4},
    [5] = {.committedInfoRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_ALE_PRIORITY_CIR_PRI5, .peakInfoRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_ALE_PRIORITY_PIR_PRI5},
    [6] = {.committedInfoRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_ALE_PRIORITY_CIR_PRI6, .peakInfoRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_ALE_PRIORITY_PIR_PRI6},
    [7] = {.committedInfoRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_ALE_PRIORITY_CIR_PRI7, .peakInfoRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_ALE_PRIORITY_PIR_PRI7},
};

static CpswTestTrafficShapingAleRate_t gCpswTestTrafficShaping_ipDstAleRate[CPSW_PRIORITY_NUM] =
{
    [0] = {.committedInfoRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_ALE_IPDST_CIR_PRI0, .peakInfoRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_ALE_IPDST_PIR_PRI0},
    [1] = {.committedInfoRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_ALE_IPDST_CIR_PRI1, .peakInfoRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_ALE_IPDST_PIR_PRI1},
    [2] = {.committedInfoRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_ALE_IPDST_CIR_PRI2, .peakInfoRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_ALE_IPDST_PIR_PRI2},
    [3] = {.committedInfoRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_ALE_IPDST_CIR_PRI3, .peakInfoRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_ALE_IPDST_PIR_PRI3},
    [4] = {.committedInfoRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_ALE_IPDST_CIR_PRI4, .peakInfoRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_ALE_IPDST_PIR_PRI4},
    [5] = {.committedInfoRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_ALE_IPDST_CIR_PRI5, .peakInfoRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_ALE_IPDST_PIR_PRI5},
    [6] = {.committedInfoRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_ALE_IPDST_CIR_PRI6, .peakInfoRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_ALE_IPDST_PIR_PRI6},
    [7] = {.committedInfoRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_ALE_IPDST_CIR_PRI7, .peakInfoRateBitsPerSec = CPSW_TEST_TRAFFIC_SHAPING_ALE_IPDST_PIR_PRI7},
};

static uint8_t testEgressPortDestAddr[CPSW_MAC_ADDR_LEN] =
{
    0x08, 0x00, 0x00, 0x00, 0x00, 0x02
};

static uint8_t testDestIpv4Addr[CPSW_ALE_IPV4ADDR_NUM_OCTETS] =
{
    128, 24, 190, 64
};

static uint8_t testSrcAddr[CPSW_MAC_ADDR_LEN] =
{
    0x02, 0x00, 0x00, 0x00, 0x00, 0x08
};

/* ========================================================================== */
/*                          Function Definitions                              */
/* ========================================================================== */

int32_t CpswTestTrafficShaping_Run(CpswTestTaskObj *taskObj)
{
    int32_t status             = CPSW_SOK;
    CpswTestStateObj *stateObj = &taskObj->stateObj;

    status = CpswTestTrafficShaping_Config(taskObj);
    if (status == CPSW_SOK)
    {
        status = CpswTestTrafficShaping_AddAleEntry(taskObj);
    }

    if (status == CPSW_SOK)
    {
        status = CpswTestTrafficShaping_setRxPriority(taskObj);
    }

    if (status == CPSW_SOK)
    {
        status = CpswTestTrafficShaping_setRxDscpPriority(taskObj);
    }

    if (status == CPSW_SOK)
    {
        status = CpswTestTrafficShaping_setPolicerPriority(taskObj);
    }

    if (status == CPSW_SOK)
    {
        status = CpswTestTrafficShaping_setPolicerPriorityDstIpv4(taskObj);
    }

    if (CPSW_SOK == status)
    {
        Cpsw_IoctlPrms prms;

        CPSW_IOCTL_SET_NO_ARGS(&prms);
        status = Cpsw_ioctl(stateObj->hCpsw, stateObj->coreId, CPSW_ALE_IOCTL_DUMP_TABLE,
                            &prms);
        CpswAppUtils_assert(status == CPSW_SOK);

        CPSW_IOCTL_SET_NO_ARGS(&prms);
        status = Cpsw_ioctl(stateObj->hCpsw, stateObj->coreId, CPSW_ALE_IOCTL_DUMP_POLICER_ENTRIES,
                            &prms);

        CpswAppUtils_assert(status == CPSW_SOK);

        status = CpswTestCommon_Run(taskObj);
    }

    return status;
}

static int32_t CpswTestTrafficShaping_Config(CpswTestTaskObj *taskObj)
{
    CpswTestStateObj *stateObj = &taskObj->stateObj;
    const CpswSoc_Config *socCfg;
    Cpsw_IoctlPrms prms;
    CpswMacPort_SetTrafficShapingInArgs setInArgs;
    CpswMacPort_GenericInArgs getInArgs;
    CpswPort_TrafficShapingConfig getOutArgs;
    uint32_t cir1;
    uint32_t cir2;
    uint32_t eir1;
    uint32_t eir2;
    uint32_t i;
    uint32_t roundingError;
    int32_t status;

    socCfg        = CpswSoc_getConfig(taskObj->taskCfg->cpswType);
    roundingError = socCfg->cppiClkFreqHz / CPSW_TEST_TRAFFIC_SHAPING_CPSW_RATE_LIMIT_DIVFACTOR;

    /* Set traffic shaping CIR/EIR rates for all priorities */
    setInArgs.portNum = CPSW_TEST_TRAFFIC_SHAPING_EGRESS_PORT;
    memcpy(&setInArgs.trafficShapingConfig.rates[0U],
           &gCpswTestTrafficShaping_priorityRate[0U],
           sizeof(setInArgs.trafficShapingConfig.rates));
    CPSW_IOCTL_SET_IN_ARGS(&prms, &setInArgs);

    status = Cpsw_ioctl(stateObj->hCpsw, stateObj->coreId,
                        CPSW_MACPORT_IOCTL_SET_TRAFFIC_SHAPING,
                        &prms);
    if (status != CPSW_SOK)
    {
        CpswAppUtils_print("CpswTestTrafficShaping_Config() failed CPSW_MACPORT_IOCTL_SET_TRAFFIC_SHAPING: %d\n",
                           status);
    }

    /* Get traffic shaping rates from the driver */
    if (status == CPSW_SOK)
    {
        getInArgs.portNum = CPSW_TEST_TRAFFIC_SHAPING_EGRESS_PORT;
        CPSW_IOCTL_SET_INOUT_ARGS(&prms, &getInArgs, &getOutArgs);

        status = Cpsw_ioctl(stateObj->hCpsw, stateObj->coreId,
                            CPSW_MACPORT_IOCTL_GET_TRAFFIC_SHAPING,
                            &prms);
        if (status != CPSW_SOK)
        {
            CpswAppUtils_print("CpswTestTrafficShaping_Config() failed CPSW_MACPORT_IOCTL_GET_TRAFFIC_SHAPING: %d\n",
                               status);
        }
    }

    /* Compare set and retrieved traffic shaping configuration */
    if (status == CPSW_SOK)
    {
        CpswAppUtils_assert(setInArgs.portNum == getInArgs.portNum);

        for (i = 0U; i < CPSW_PRIORITY_NUM; i++)
        {
            cir1 = setInArgs.trafficShapingConfig.rates[i].committedRateBitsPerSec;
            eir1 = setInArgs.trafficShapingConfig.rates[i].excessRateBitsPerSec;
            cir2 = getOutArgs.rates[i].committedRateBitsPerSec;
            eir2 = getOutArgs.rates[i].excessRateBitsPerSec;

            CpswAppUtils_assert(abs(cir1 - cir2) <= roundingError);
            CpswAppUtils_assert(abs(eir1 - eir2) <= roundingError);

            if (cir1 != cir2)
            {
                CpswAppUtils_print("Traffic Shaping CIR configured to %u due to rounding error\n",
                                   cir2);
            }

            if (eir1 != eir2)
            {
                CpswAppUtils_print("Traffic Shaping EIR configured to %u due to rounding error\n",
                                   eir2);
            }
        }
    }

    return status;
}

static int32_t CpswTestTrafficShaping_AddAleEntry(CpswTestTaskObj *taskObj)
{
    int32_t status;
    Cpsw_IoctlPrms prms;
    CpswAle_AddEntryOutArgs setUcastOutArgs;
    CpswAle_SetUcastEntryInArgs setUcastInArgs;
    CpswTestStateObj *stateObj = &taskObj->stateObj;

    memset(&setUcastInArgs, 0, sizeof(setUcastInArgs));
    memcpy(&setUcastInArgs.addr.addr[0U], testEgressPortDestAddr,
           sizeof(setUcastInArgs.addr.addr));
    setUcastInArgs.addr.vlanId  = 0;
    setUcastInArgs.info.portNum = CPSW_ALE_MACPORT_TO_ALEPORT(CPSW_TEST_TRAFFIC_SHAPING_EGRESS_PORT);
    setUcastInArgs.info.blocked = false;
    setUcastInArgs.info.secure  = false;
    setUcastInArgs.info.super   = 0U;
    setUcastInArgs.info.ageable = false;
    setUcastInArgs.info.trunk   = false;

    CPSW_IOCTL_SET_INOUT_ARGS(&prms, &setUcastInArgs, &setUcastOutArgs);

    status = Cpsw_ioctl(stateObj->hCpsw, stateObj->coreId, CPSW_ALE_IOCTL_ADD_UNICAST,
                        &prms);
    if (status != CPSW_SOK)
    {
        CpswAppUtils_print("CpswTestTrafficShaping_AddAleEntry() failed CPSW_ALE_IOCTL_ADD_UNICAST: %d\n",
                           status);
    }

    return status;
}

void CpswTestTrafficShaping_setOpenPrms(CpswTestTaskObj *taskObj,
                                        Cpsw_Config *pCpswCfg,
                                        CpswOsal_Prms *pOsalPrms,
                                        CpswUtils_Prms *pUtilsPrms)
{
    Cpsw_MacPort i;

    pCpswCfg->aleConfig.policerGlobalConfig.policingEnable = TRUE;
    pCpswCfg->hostPortConfig.passPriorityTaggedUnchanged   = TRUE;

    pCpswCfg->aleConfig.modeFlags                             = CPSW_ALE_CONFIG_MASK_ALE_MODULE_ENABLE;
    pCpswCfg->aleConfig.vlanConfig.unknownUnregMcastFloodMask = 0x0;
    pCpswCfg->aleConfig.vlanConfig.unknownRegMcastFloodMask   = 0x0;
    pCpswCfg->aleConfig.vlanConfig.unknownVlanMemberListMask  = CPSW_ALE_ALL_PORTS_MASK;
    pCpswCfg->aleConfig.vlanConfig.aleVlanAwareMode           = TRUE;
    pCpswCfg->aleConfig.vlanConfig.autoLearnWithVLAN          = FALSE;
    pCpswCfg->aleConfig.vlanConfig.cpswVlanAwareMode          = TRUE;
    pCpswCfg->aleConfig.nwSecCfg.enableVid0Mode               = TRUE;

    pCpswCfg->aleConfig.portCfg[CPSW_ALE_HOST_PORT_NUM].learningCfg.noLearn              = FALSE;
    pCpswCfg->aleConfig.portCfg[CPSW_ALE_HOST_PORT_NUM].vlanCfg.dropUntagged             = FALSE;
    pCpswCfg->aleConfig.portCfg[CPSW_ALE_HOST_PORT_NUM].pvidCfg.unregMcastFloodMask      = 0x0;
    pCpswCfg->aleConfig.portCfg[CPSW_ALE_HOST_PORT_NUM].pvidCfg.regMcastFloodMask        = CPSW_ALE_ALL_PORTS_MASK;
    pCpswCfg->aleConfig.portCfg[CPSW_ALE_HOST_PORT_NUM].pvidCfg.forceUntaggedEgressMask  = 0;
    pCpswCfg->aleConfig.portCfg[CPSW_ALE_HOST_PORT_NUM].pvidCfg.noLearnMask              = 0x0;
    pCpswCfg->aleConfig.portCfg[CPSW_ALE_HOST_PORT_NUM].pvidCfg.vidIngressCheck          = 0x0;
    pCpswCfg->aleConfig.portCfg[CPSW_ALE_HOST_PORT_NUM].pvidCfg.limitIPNxtHdr            = false;
    pCpswCfg->aleConfig.portCfg[CPSW_ALE_HOST_PORT_NUM].pvidCfg.disallowIPFragmentation  = false;
    pCpswCfg->aleConfig.portCfg[CPSW_ALE_HOST_PORT_NUM].pvidCfg.vlanIdInfo.outerVlanFlag = false;
    pCpswCfg->aleConfig.portCfg[CPSW_ALE_HOST_PORT_NUM].pvidCfg.vlanIdInfo.vlanId        = ETH_TEST_HOST_PORT_VLAN_ID;
    pCpswCfg->aleConfig.portCfg[CPSW_ALE_HOST_PORT_NUM].pvidCfg.vlanMemberList           = CPSW_ALE_ALL_PORTS_MASK;

    for (i = CPSW_MAC_PORT_FIRST; i <= CPSW_MAC_PORT_LAST; i++)
    {
        pCpswCfg->aleConfig.portCfg[CPSW_ALE_MACPORT_TO_ALEPORT(i)].learningCfg.noLearn              = FALSE;
        pCpswCfg->aleConfig.portCfg[CPSW_ALE_MACPORT_TO_ALEPORT(i)].vlanCfg.dropUntagged             = FALSE;
        pCpswCfg->aleConfig.portCfg[CPSW_ALE_MACPORT_TO_ALEPORT(i)].pvidCfg.unregMcastFloodMask      = 0x0;
        pCpswCfg->aleConfig.portCfg[CPSW_ALE_MACPORT_TO_ALEPORT(i)].pvidCfg.regMcastFloodMask        = CPSW_ALE_ALL_PORTS_MASK;
        pCpswCfg->aleConfig.portCfg[CPSW_ALE_MACPORT_TO_ALEPORT(i)].pvidCfg.forceUntaggedEgressMask  = 0;
        pCpswCfg->aleConfig.portCfg[CPSW_ALE_MACPORT_TO_ALEPORT(i)].pvidCfg.noLearnMask              = 0x0;
        pCpswCfg->aleConfig.portCfg[CPSW_ALE_MACPORT_TO_ALEPORT(i)].pvidCfg.vidIngressCheck          = 0x0;
        pCpswCfg->aleConfig.portCfg[CPSW_ALE_MACPORT_TO_ALEPORT(i)].pvidCfg.limitIPNxtHdr            = false;
        pCpswCfg->aleConfig.portCfg[CPSW_ALE_MACPORT_TO_ALEPORT(i)].pvidCfg.disallowIPFragmentation  = false;
        pCpswCfg->aleConfig.portCfg[CPSW_ALE_MACPORT_TO_ALEPORT(i)].pvidCfg.vlanIdInfo.outerVlanFlag = false;
        pCpswCfg->aleConfig.portCfg[CPSW_ALE_MACPORT_TO_ALEPORT(i)].pvidCfg.vlanIdInfo.vlanId        = ETH_TEST_MAC_PORT_VLAN_ID_BASE + CPSW_NORMALIZE_MACPORT(i);
        pCpswCfg->aleConfig.portCfg[CPSW_ALE_MACPORT_TO_ALEPORT(i)].pvidCfg.vlanMemberList           = CPSW_ALE_ALL_PORTS_MASK;
    }

    pCpswCfg->hostPortConfig.vlanCfg.portPri = 7;
    pCpswCfg->hostPortConfig.vlanCfg.portCfi = 0;
    pCpswCfg->hostPortConfig.vlanCfg.portVID = ETH_TEST_HOST_PORT_VLAN_ID;
    pCpswCfg->vlanConfig.vlanAware           = TRUE;

    pCpswCfg->aleConfig.policerGlobalConfig.policingEnable     = TRUE;
    pCpswCfg->aleConfig.policerGlobalConfig.yellowDropEnable   = FALSE;
    pCpswCfg->aleConfig.policerGlobalConfig.redDropEnable      = TRUE;
    pCpswCfg->aleConfig.policerGlobalConfig.policerNoMatchMode = CPSW_ALE_POLICER_NOMATCH_MODE_GREEN;
}

void CpswTestTrafficShaping_updatePortLinkCfg(Cpsw_OpenPortLinkInArgs *pLinkArgs,
                                              Cpsw_MacPort portNum)
{
    pLinkArgs->macConfig.enableLoopback              = FALSE;
    pLinkArgs->macConfig.passPriorityTaggedUnchanged = TRUE;

    pLinkArgs->macConfig.vlanConfig.portPri = CPSW_NORMALIZE_MACPORT(portNum);
    pLinkArgs->macConfig.vlanConfig.portCfi = 0;
    pLinkArgs->macConfig.vlanConfig.portVID = ETH_TEST_MAC_PORT_VLAN_ID_BASE + CPSW_NORMALIZE_MACPORT(portNum);
}

static int32_t CpswTestTrafficShaping_setRxDscpPriority(CpswTestTaskObj *taskObj)
{
    CpswPort_DscpPriority setHostInArgs;
    CpswPort_DscpPriority getHostOutArgs;
    Cpsw_IoctlPrms hostPrms;
    CpswMacPort_SetRxDscpPriorityInArgs setMacInArgs;
    CpswMacPort_GenericInArgs getMacInArgs;
    CpswPort_DscpPriority getMacOutArgs;
    Cpsw_IoctlPrms macPrms;
    uint32_t i;
    int32_t status;
    CpswTestStateObj *stateObj = &taskObj->stateObj;
    uint32_t portIter;
    CpswTestMacPortList_t enabledPorts;

    setHostInArgs.enableDscpIPv4 = TRUE;
    setHostInArgs.enableDscpIPv6 = TRUE;
    for (i = 0; i < CPSW_UTILS_ARRAYSIZE(setHostInArgs.tosMap); i++)
    {
        setHostInArgs.tosMap[i] = (i / 8);
    }

    CPSW_IOCTL_SET_IN_ARGS(&hostPrms, &setHostInArgs);

    status = Cpsw_ioctl(stateObj->hCpsw, stateObj->coreId, CPSW_HOSTPORT_IOCTL_SET_RX_DSCP_PRIORITY,
                        &hostPrms);
    if (status != CPSW_SOK)
    {
        CpswAppUtils_print("CpswTestTrafficShaping_setRxDscpPriority() failed CPSW_HOSTPORT_IOCTL_SET_RX_DSCP_PRIORITY: %d\n",
                           status);
    }

    CPSW_IOCTL_SET_OUT_ARGS(&hostPrms, &getHostOutArgs);

    status = Cpsw_ioctl(stateObj->hCpsw, stateObj->coreId, CPSW_HOSTPORT_IOCTL_GET_RX_DSCP_PRIORITY,
                        &hostPrms);
    if (status != CPSW_SOK)
    {
        CpswAppUtils_print("CpswTestTrafficShaping_setRxDscpPriority() failed CPSW_HOSTPORT_IOCTL_GET_RX_DSCP_PRIORITY: %d\n",
                           status);
    }
    else
    {
        CpswAppUtils_assert(setHostInArgs.enableDscpIPv4 ==
                            getHostOutArgs.enableDscpIPv4);
        CpswAppUtils_assert(setHostInArgs.enableDscpIPv6 ==
                            getHostOutArgs.enableDscpIPv6);
        for (i = 0; i < CPSW_UTILS_ARRAYSIZE(setHostInArgs.tosMap); i++)
        {
            CpswAppUtils_assert(setHostInArgs.tosMap[i] ==
                                getHostOutArgs.tosMap[i]);
        }
    }

    CpswTestCommon_getMacPortList(taskObj->taskCfg->macPortMask, &enabledPorts);
    /* Set Mac Port DSCP priority mapping for all mac ports */
    for (portIter = 0; portIter < enabledPorts.numMacPorts; portIter++)
    {
        setMacInArgs.portNum                     = enabledPorts.macPortList[portIter];
        setMacInArgs.dscpPriority.enableDscpIPv4 = TRUE;
        setMacInArgs.dscpPriority.enableDscpIPv6 = TRUE;
        for (i = 0; i < CPSW_UTILS_ARRAYSIZE(setMacInArgs.dscpPriority.tosMap); i++)
        {
            setMacInArgs.dscpPriority.tosMap[i] = (i / 8);
        }

        CPSW_IOCTL_SET_IN_ARGS(&macPrms, &setMacInArgs);

        status = Cpsw_ioctl(stateObj->hCpsw, stateObj->coreId, CPSW_MACPORT_IOCTL_SET_RX_DSCP_PRIORITY,
                            &macPrms);
        if (status != CPSW_SOK)
        {
            CpswAppUtils_print("CpswTestTrafficShaping_setRxDscpPriority() failed CPSW_MACPORT_IOCTL_SET_RX_DSCP_PRIORITY: %d\n",
                               status);
        }

        getMacInArgs.portNum = enabledPorts.macPortList[portIter];
        CPSW_IOCTL_SET_INOUT_ARGS(&macPrms, &getMacInArgs, &getMacOutArgs);

        status = Cpsw_ioctl(stateObj->hCpsw, stateObj->coreId, CPSW_MACPORT_IOCTL_GET_RX_DSCP_PRIORITY,
                            &macPrms);
        if (status != CPSW_SOK)
        {
            CpswAppUtils_print("CpswTestTrafficShaping_setRxDscpPriority() failed CPSW_MACPORT_IOCTL_GET_RX_DSCP_PRIORITY: %d\n",
                               status);
        }
        else
        {
            CpswAppUtils_assert(setMacInArgs.dscpPriority.enableDscpIPv4 ==
                                getMacOutArgs.enableDscpIPv4);
            CpswAppUtils_assert(setMacInArgs.dscpPriority.enableDscpIPv6 ==
                                getMacOutArgs.enableDscpIPv6);
            for (i = 0; i < CPSW_UTILS_ARRAYSIZE(setMacInArgs.dscpPriority.tosMap); i++)
            {
                CpswAppUtils_assert(setMacInArgs.dscpPriority.tosMap[i] ==
                                    getMacOutArgs.tosMap[i]);
            }
        }
    }

    return status;
}

static int32_t CpswTestTrafficShaping_setRxPriority(CpswTestTaskObj *taskObj)
{
    CpswPort_RxPriority setHostInArgs;
    Cpsw_IoctlPrms hostPrms;
    CpswMacPort_SetRxPriorityInArgs setMacInArgs;
    Cpsw_IoctlPrms macPrms;
    uint32_t i;
    int32_t status;
    CpswTestStateObj *stateObj = &taskObj->stateObj;
    uint32_t portIter;
    CpswTestMacPortList_t enabledPorts;

    for (i = 0; i < CPSW_UTILS_ARRAYSIZE(setHostInArgs.priorityMap); i++)
    {
        setHostInArgs.priorityMap[i] = i;
    }

    CPSW_IOCTL_SET_IN_ARGS(&hostPrms, &setHostInArgs);

    status = Cpsw_ioctl(stateObj->hCpsw, stateObj->coreId, CPSW_HOSTPORT_IOCTL_SET_RX_PRIORITY,
                        &hostPrms);
    if (status != CPSW_SOK)
    {
        CpswAppUtils_print("CpswTestTrafficShaping_setRxPriority() failed CPSW_HOSTPORT_IOCTL_SET_RX_PRIORITY: %d\n",
                           status);
    }

    CpswTestCommon_getMacPortList(taskObj->taskCfg->macPortMask, &enabledPorts);
    /* Set Mac Port DSCP priority mapping for all mac ports */
    for (portIter = 0; portIter < enabledPorts.numMacPorts; portIter++)
    {
        setMacInArgs.portNum = enabledPorts.macPortList[portIter];
        for (i = 0; i < CPSW_UTILS_ARRAYSIZE(setMacInArgs.rxPriority.priorityMap); i++)
        {
            setMacInArgs.rxPriority.priorityMap[i] = i;
        }

        CPSW_IOCTL_SET_IN_ARGS(&macPrms, &setMacInArgs);

        status = Cpsw_ioctl(stateObj->hCpsw, stateObj->coreId, CPSW_MACPORT_IOCTL_SET_RX_PRIORITY,
                            &macPrms);
        if (status != CPSW_SOK)
        {
            CpswAppUtils_print("CpswTestTrafficShaping_setRxPriority() failed CPSW_MACPORT_IOCTL_SET_RX_PRIORITY: %d\n",
                               status);
        }
    }

    return status;
}

static int32_t CpswTestTrafficShaping_setPolicerPriority(CpswTestTaskObj *taskObj)
{
    int32_t status;
    Cpsw_IoctlPrms prms;
    CpswAle_SetPolicerEntryOutArgs setPolicerOutArgs;
    CpswAle_SetPolicerEntryInArgs setPolicerInArgs;
    CpswTestStateObj *stateObj = &taskObj->stateObj;
    uint32_t i;

    for (i = 0; i < CPSW_PRIORITY_NUM; i++)
    {
        setPolicerInArgs.policerMatch.policerMatchEnableMask = CPSW_ALE_POLICER_MATCH_PRIORITY;
        setPolicerInArgs.policerMatch.priority               = i;

        setPolicerInArgs.policerMatch.policerMatchEnableMask |= CPSW_ALE_POLICER_MATCH_MACSRC;
        memcpy(&setPolicerInArgs.policerMatch.srcMacAddr.addr.addr[0U], testSrcAddr,
               sizeof(setPolicerInArgs.policerMatch.srcMacAddr.addr.addr));
        setPolicerInArgs.policerMatch.srcMacAddr.addr.vlanId    = 0;
        setPolicerInArgs.policerMatch.srcMacAddr.ingressPortNum = CPSW_ALE_MACPORT_TO_ALEPORT(CPSW_TEST_TRAFFIC_SHAPING_INGRESS_PORT);

        setPolicerInArgs.threadIdEnable         = FALSE;
        setPolicerInArgs.threadId               = 0;
        setPolicerInArgs.peakRateInBitsPerSec   = gCpswTestTrafficShaping_priorityAleRate[i].peakInfoRateBitsPerSec;
        setPolicerInArgs.commitRateInBitsPerSec = gCpswTestTrafficShaping_priorityAleRate[i].committedInfoRateBitsPerSec;

        CPSW_IOCTL_SET_INOUT_ARGS(&prms, &setPolicerInArgs, &setPolicerOutArgs);

        status = Cpsw_ioctl(stateObj->hCpsw, stateObj->coreId, CPSW_ALE_IOCTL_SET_POLICER,
                            &prms);
        if (status != CPSW_SOK)
        {
            CpswAppUtils_print("CpswTestTrafficShaping_setPolicerPriority() failed CPSW_ALE_IOCTL_SET_POLICER: %d\n",
                               status);
            break;
        }
    }

    return status;
}

static int32_t CpswTestTrafficShaping_setPolicerPriorityDstIpv4(CpswTestTaskObj *taskObj)
{
    int32_t status;
    Cpsw_IoctlPrms prms;
    CpswAle_SetPolicerEntryOutArgs setPolicerOutArgs;
    CpswAle_SetPolicerEntryInArgs setPolicerInArgs;
    CpswTestStateObj *stateObj = &taskObj->stateObj;
    uint32_t i;

    for (i = 0; i < CPSW_PRIORITY_NUM; i++)
    {
        /* Set Policer entry */
        memset(&setPolicerInArgs, 0, sizeof(setPolicerInArgs));
        setPolicerInArgs.policerMatch.policerMatchEnableMask = (CPSW_ALE_POLICER_MATCH_IPDST);
        memcpy(&setPolicerInArgs.policerMatch.dstIp.ipv4.ipv4Addr[0U], testDestIpv4Addr,
               sizeof(setPolicerInArgs.policerMatch.dstIp.ipv4.ipv4Addr));
        setPolicerInArgs.policerMatch.dstIp.ipv4.ipv4Addr[0U]    += i;
        setPolicerInArgs.policerMatch.dstIp.ipv4.numLSBIgnoreBits = 0;
        setPolicerInArgs.policerMatch.dstIp.ipAddrtype            = CPSW_ALE_IPADDR_CLASSIFIER_IPV4;

        setPolicerInArgs.threadIdEnable         = FALSE;
        setPolicerInArgs.peakRateInBitsPerSec   = gCpswTestTrafficShaping_ipDstAleRate[i].peakInfoRateBitsPerSec;
        setPolicerInArgs.commitRateInBitsPerSec = gCpswTestTrafficShaping_ipDstAleRate[i].committedInfoRateBitsPerSec;

        CPSW_IOCTL_SET_INOUT_ARGS(&prms, &setPolicerInArgs, &setPolicerOutArgs);

        status = Cpsw_ioctl(stateObj->hCpsw, stateObj->coreId, CPSW_ALE_IOCTL_SET_POLICER,
                            &prms);
        if (status != CPSW_SOK)
        {
            CpswAppUtils_print("CpswTestTrafficShaping_setPolicerDstIp() failed CPSW_ALE_IOCTL_SET_POLICER: %d\n",
                               status);
            break;
        }
    }

    return status;
}
