/*
 *  Copyright (c) Texas Instruments Incorporated 2019
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 *  \file cpsw_testconfig.h
 *
 *  \brief This file defines the common configurations like driver config etc...
 */

#ifndef CPSW_TEST_CONFIG_H_
#define CPSW_TEST_CONFIG_H_

/* ========================================================================== */
/*                             Include Files                                  */
/* ========================================================================== */
#include "cpsw_test.h"
#include "cpsw_test_base.h"
#include "cpsw_test_basicswitching.h"
#include "cpsw_test_autolearn.h"
#include "cpsw_test_stats.h"
#include "cpsw_test_vlan.h"
#include "cpsw_test_autolearn_vlan.h"
#include "cpsw_test_aging_maxlength.h"
#include "cpsw_test_crcstrip.h"
#include "cpsw_test_fifostats.h"
#include "cpsw_test_ale_srcaddr_update.h"
#include "cpsw_test_ale_tablefull.h"
#include "cpsw_test_vlan_dropuntagged.h"
#include "cpsw_test_multicast.h"
#include "cpsw_test_policer.h"
#include "cpsw_test_network_security.h"
#include "cpsw_test_hostport_rxfilter.h"
#include "cpsw_test_policer_nomatch.h"
#include "cpsw_test_intervlan.h"
#include "cpsw_test_pvid.h"
#include "cpsw_test_sanity.h"
#include "cpsw_test_outer_vlan.h"
#include "cpsw_test_traffic_shaping.h"
#include "cpsw_test_nway.h"
#include "cpsw_test_rxflow_mtu.h"
#include "cpsw_test_phystrapping.h"
#include "cpsw_test_priority_regeneration.h"
#include "cpsw_test_bcastmcast_limit.h"
#include "cpsw_test_directed_pkt.h"

/** ToDo: Below tests use J7 specific functions/macros causing build errors for AM65xx.
 * Have to fix them post 6.02 release.
 */
#if defined(SOC_J721E) || defined(SOC_J7200)
#include "cpsw_test_default_priority.h"
#include "cpsw_test_sgmii_lpbk.h"
#include "cpsw_test_cpts_event.h"
#endif

#ifdef __cplusplus
extern "C" {
#endif

/* ========================================================================== */
/*                           Macros & Typedefs                                */
/* ========================================================================== */
/* Numbe of task configuration available */
#define CPSW_TEST_NUM_TASK_CFG_PARAMS         ((sizeof(gTestTaskCfg)) / \
                                               (sizeof(CpswTestTaskCfg)))

#define CPSW_TEST_NUM_TX_CH_CFG_PARAMS         ((sizeof(gTestTxChCfg)) / \
                                                (sizeof(CpswTestTxChCfgInfo)))

#define CPSW_TEST_NUM_RX_FLOW_CFG_PARAMS         ((sizeof(gTestRxFlowCfg)) / \
                                                  (sizeof(                   \
                                                          CpswTestRxFlowCfgInfo)))

/* IDs for test task configuration */
#define TASK_CFG_ID_SWITCH_TEST                     ((uint32_t)0U)
#define TASK_CFG_ID_ALE_AUTO_LEARN_TEST             ((uint32_t)1U)
#define TASK_CFG_ID_ALE_AUTO_LEARN_VLAN_TEST        ((uint32_t)2U)
#define TASK_CFG_ID_MAC_SPEED_TEST                  ((uint32_t)3U)
#define TASK_CFG_ID_STATS_TEST                      ((uint32_t)4U)
#define TASK_CFG_ID_VLAN_TEST                       ((uint32_t)5U)
#define TASK_CFG_ID_AGING_MAXLEN_TEST               ((uint32_t)6U)
#define TASK_CFG_ID_CRCSTRIP_TEST                   ((uint32_t)7U)
#define TASK_CFG_ID_FIFOSTATS_TEST                  ((uint32_t)8U)
#define TASK_CFG_ID_SRCADDRUPDATE_TEST              ((uint32_t)9U)
#define TASK_CFG_ID_TABLEFULL_TEST                  ((uint32_t)10U)
#define TASK_CFG_ID_VLAN_DROP_UNTAGGED_TEST         ((uint32_t)11U)
#define TASK_CFG_ID_MULTICAST_TEST                  ((uint32_t)12U)
#define TASK_CFG_ID_POLICER_TEST                    ((uint32_t)13U)
#define TASK_CFG_ID_SECURITY_TEST                   ((uint32_t)14U)
#define TASK_CFG_ID_RXFILTER_TEST                   ((uint32_t)15U)
#define TASK_CFG_ID_POLICER_NOMATCH_TEST            ((uint32_t)16U)
#define TASK_CFG_ID_INTERVLAN_TEST                  ((uint32_t)17U)
#define TASK_CFG_ID_DEFAULT_PRIORITY_TEST           ((uint32_t)18U)
#define TASK_CFG_ID_PVID_TEST                       ((uint32_t)19U)
#define TASK_CFG_ID_SANITY_TEST                     ((uint32_t)20U)
#define TASK_CFG_ID_OUTERVLAN_TEST                  ((uint32_t)21U)
#define TASK_CFG_ID_TRAFFICSHAPING_TEST             ((uint32_t)22U)
#define TASK_CFG_ID_AUTONEGOTIATION_1G_FD_TEST      ((uint32_t)23U)
#define TASK_CFG_ID_AUTONEGOTIATION_100M_FD_TEST    ((uint32_t)24U)
#define TASK_CFG_ID_AUTONEGOTIATION_100M_HD_TEST    ((uint32_t)25U)
#define TASK_CFG_ID_RXFLOW_MTU_TEST                 ((uint32_t)26U)
#define TASK_CFG_ID_PHY_STRAPPING_TEST              ((uint32_t)27U)
#define TASK_CFG_ID_PRIORITY_REGENERATION_TEST      ((uint32_t)28U)
#define TASK_CFG_ID_SGMII_LPBK_TEST                 ((uint32_t)29U)
#define TASK_CFG_ID_QSGMII_SWITCHING_TEST           ((uint32_t)30U)
#define TASK_CFG_ID_ALL_PORTS_SWITCH_TEST           ((uint32_t)31U)
#define TASK_CFG_ID_CPTS_EVENT_TEST                 ((uint32_t)32U)
#define TASK_CFG_ID_BCASTMCAST_LIMIT_TEST           ((uint32_t)33U)
#define TASK_CFG_ID_DIRECTED_PKT_TEST               ((uint32_t)34U)

/* IDs for Packet Tx  configuration */
#define TXCH_CFG_ID_COMMON_TEST                      ((uint32_t)0U)
#define TXCH_CFG_ID_STATS_TEST                       ((uint32_t)1U)
#define TXCH_CFG_ID_CRCSTRIP_TEST                    ((uint32_t)2U)
#define TXCH_CFG_ID_SRCADDRUPDATE_TEST               ((uint32_t)3U)
#define TXCH_CFG_ID_TABLEFULL_TEST                   ((uint32_t)4U)
#define TXCH_CFG_ID_FIFOSTATS_TEST                   ((uint32_t)5U)
#define TXCH_CFG_ID_POLICER_TEST                     ((uint32_t)6U)
#define TXCH_CFG_ID_INTERVLAN_TEST                   ((uint32_t)7U)
#define TXCH_CFG_ID_DEFAULT_PRIORITY_TEST            ((uint32_t)8U)
#define TXCH_CFG_ID_RXFLOW_MTU_TEST                  ((uint32_t)9U)
#define TXCH_CFG_ID_CPTS_EVENT_TEST                  ((uint32_t)10U)
#define TXCH_CFG_ID_DIRECTED_PKT_TEST                ((uint32_t)11U)

/* IDs for Packet Rx  configuration */
#define RXFLOW_CFG_ID_COMMON_TEST                       ((uint32_t)0U)
#define RXFLOW_CFG_ID_SWITCHING_TEST                    ((uint32_t)1U)
#define RXFLOW_CFG_ID_CRCSTRIP_TEST                     ((uint32_t)2U)
#define RXFLOW_CFG_ID_FIFOSTATS_TEST                    ((uint32_t)3U)
#define RXFLOW_CFG_ID_MULTICAST_TEST                    ((uint32_t)4U)
#define RXFLOW_CFG_ID_STATS_TEST                        ((uint32_t)5U)
#define RXFLOW_CFG_ID_SECURITY_TEST                     ((uint32_t)6U)
#define RXFLOW_CFG_ID_RXFILTER_TEST                     ((uint32_t)7U)
#define RXFLOW_CFG_ID_POLICER1_TEST                     ((uint32_t)8U)
#define RXFLOW_CFG_ID_POLICER2_TEST                     ((uint32_t)9U)
#define RXFLOW_CFG_ID_POLICER_NOMATCH_TEST              ((uint32_t)10U)
#define RXFLOW_CFG_ID_INTERVLAN_TEST                    ((uint32_t)11U)
#define RXFLOW_CFG_ID_DEFAULT_PRIORITY_TEST             ((uint32_t)12U)
#define RXFLOW_CFG_ID_NONDEFAULT_POLICER_NOMATCH_TEST   ((uint32_t)13U)
#define RXFLOW_CFG_ID_PVID_TEST                         ((uint32_t)14U)
#define RXFLOW_CFG_ID_OUTERVLAN_TEST                    ((uint32_t)15U)
#define RXFLOW_CFG_ID_RXFLOW_MTU_TEST                   ((uint32_t)16U)
#define RXFLOW_CFG_ID_CPTS_EVENT_TEST                   ((uint32_t)17U)
#define RXFLOW_CFG_ID_DIRECTED_PKT_TEST                 ((uint32_t)18U)

#define CPSW_TEST_ALL_MACPORT_MASK (CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_0)) | \
                                    CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_1)) | \
                                    CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_2)) | \
                                    CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_3)) | \
                                    CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_4)) | \
                                    CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_5)) | \
                                    CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_6)) | \
                                    CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_7)))

#define CPSW_TEST_EVM_QSGMII_MACPORT_MASK (CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_4)) | \
                                       CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_5)) | \
                                       CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_6)))

#define CPSW_TEST_EVM_RGMII_MACPORT_MASK (CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_0)) | \
                                      CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_2)) | \
                                      CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_3)) | \
                                      CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_7)))

/* ========================================================================== */
/*                          Function Declarations                             */
/* ========================================================================== */
static inline CpswTestTaskCfg *Test_getTaskCfgParams(uint32_t taskCfgId);

static inline CpswTestTxChCfgInfo *Test_getTxChCfgParams(uint32_t txChCfgId);

static inline CpswTestRxFlowCfgInfo *Test_getRxFlowCfgParams(uint32_t rxFlowCfgId);

/* ========================================================================== */
/*                            Global Variables                                */
/* ========================================================================== */

CpswTestTaskCfg gTestTaskCfg[] =
{
    /* Switch logic test */
    {
        .taskCfgId          = TASK_CFG_ID_SWITCH_TEST,
        .txChCfgId          = {TXCH_CFG_ID_COMMON_TEST},
        .rxFlowCfgId        = {RXFLOW_CFG_ID_SWITCHING_TEST},
        .openPrmsUpdate     = NULL,
        .portLinkPrmsUpdate = NULL,
        .runTest            = &CpswTestSwitching_RunWithIPG,
        .deInitTest         = NULL,
        .taskId             = 0U,
        .cpswType           = CPSW_9G,
        .macPortMask        = (CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_3)) |
                               CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_2))),
        .numTxCh   = 1,
        .numRxFlow = 1,
    },
    /* ALE auto learn */
    {
        .taskCfgId          = TASK_CFG_ID_ALE_AUTO_LEARN_TEST,
        .txChCfgId          = {TXCH_CFG_ID_COMMON_TEST},
        .rxFlowCfgId        = {RXFLOW_CFG_ID_COMMON_TEST},
        .taskId             = 1U,
        .cpswType           = CPSW_9G,
        .openPrmsUpdate     = &CpswTestAutoLearn_setOpenPrms,
        .portLinkPrmsUpdate = &CpswTestAutoLearn_updatePortLinkCfg,
        .runTest            = &CpswTestAutoLearn_Run,
        .deInitTest         = NULL,
        .macPortMask        = CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_0)),
        .numTxCh            = 1,
        .numRxFlow          = 1,
    },
    /* ALE auto learn VLAN*/
    {
        .taskCfgId          = TASK_CFG_ID_ALE_AUTO_LEARN_VLAN_TEST,
        .txChCfgId          = {TXCH_CFG_ID_COMMON_TEST},
        .rxFlowCfgId        = {RXFLOW_CFG_ID_COMMON_TEST},
        .taskId             = 2U,
        .cpswType           = CPSW_9G,
        .openPrmsUpdate     = &CpswTestAutoLearnVlan_setOpenPrms,
        .portLinkPrmsUpdate = &CpswTestAutoLearnVlan_updatePortLinkCfg,
        .runTest            = &CpswTestAutoLearnVlan_Run,
        .deInitTest         = NULL,
        .macPortMask        = CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_0)),
        .numTxCh            = 1,
        .numRxFlow          = 1,
    },
    /* Mac Speed test */
    {
        .taskCfgId          = TASK_CFG_ID_MAC_SPEED_TEST,
        .txChCfgId          = {TXCH_CFG_ID_COMMON_TEST},
        .rxFlowCfgId        = {RXFLOW_CFG_ID_COMMON_TEST},
        .taskId             = 3U,
        .cpswType           = CPSW_9G,
        .openPrmsUpdate     = NULL,
        .portLinkPrmsUpdate = &CpswTestSanity_updatePortLinkCfg100Mbps,
        .runTest            = &CpswTestMacSpeed_Run,
        .deInitTest         = NULL,
        .macPortMask        = CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_0)),
        .numTxCh            = 1,
        .numRxFlow          = 1,
    },
    /* Statistics test */
    {
        .taskCfgId          = TASK_CFG_ID_STATS_TEST,
        .txChCfgId          = {TXCH_CFG_ID_STATS_TEST},
        .rxFlowCfgId        = {RXFLOW_CFG_ID_STATS_TEST},
        .taskId             = 4U,
        .cpswType           = CPSW_9G,
        .openPrmsUpdate     = NULL,
        .portLinkPrmsUpdate = &CpswTestStats_updatePortLinkCfg,
        .runTest            = &CpswTestCommon_Run,
        .deInitTest         = NULL,
        .macPortMask        = CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_0)),
        .numTxCh            = 1,
        .numRxFlow          = 1,
    },
    /* Vlan test */
    {
        .taskCfgId          = TASK_CFG_ID_VLAN_TEST,
        .txChCfgId          = {TXCH_CFG_ID_COMMON_TEST},
        .rxFlowCfgId        = {RXFLOW_CFG_ID_COMMON_TEST},
        .taskId             = 5U,
        .cpswType           = CPSW_9G,
        .openPrmsUpdate     = &CpswTestVlan_setOpenPrms,
        .portLinkPrmsUpdate = &CpswTestVlan_updatePortLinkCfg,
        .runTest            = &CpswTestVlan_Run,
        .deInitTest         = NULL,
        .macPortMask        = CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_0)),
        .numTxCh            = 1,
        .numRxFlow          = 1,
    },
    /* Aging and Max Packet length */
    {
        .taskCfgId          = TASK_CFG_ID_AGING_MAXLEN_TEST,
        .txChCfgId          = {TXCH_CFG_ID_COMMON_TEST},
        .rxFlowCfgId        = {RXFLOW_CFG_ID_COMMON_TEST},
        .taskId             = 6U,
        .cpswType           = CPSW_9G,
        .openPrmsUpdate     = &CpswTestAgingMaxPktLength_setOpenPrms,
        .portLinkPrmsUpdate = &CpswTestAgingMaxPktLength_updatePortLinkCfg,
        .runTest            = &CpswTestAgingMaxPktLength_Run,
        .deInitTest         = NULL,
        .macPortMask        = CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_1)),
        .numTxCh            = 1,
        .numRxFlow          = 1,
    },
    /* CRC Strip */
    {
        .taskCfgId          = TASK_CFG_ID_CRCSTRIP_TEST,
        .txChCfgId          = {TXCH_CFG_ID_CRCSTRIP_TEST},
        .rxFlowCfgId        = {RXFLOW_CFG_ID_CRCSTRIP_TEST},
        .taskId             = 7U,
        .cpswType           = CPSW_9G,
        .openPrmsUpdate     = &CpswTestCrcStrip_setOpenPrms,
        .portLinkPrmsUpdate = &CpswTestCrcStrip_updatePortLinkCfg,
        .runTest            = &CpswTestCommon_Run,
        .deInitTest         = NULL,
        .macPortMask        = CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_0)),
        .numTxCh            = 1,
        .numRxFlow          = 1,
    },
    /* FIFO Stats */
    {
        .taskCfgId          = TASK_CFG_ID_FIFOSTATS_TEST,
        .txChCfgId          = {TXCH_CFG_ID_FIFOSTATS_TEST},
        .rxFlowCfgId        = {RXFLOW_CFG_ID_FIFOSTATS_TEST},
        .taskId             = 8U,
        .cpswType           = CPSW_9G,
        .openPrmsUpdate     = NULL,
        .portLinkPrmsUpdate = &CpswTestFifoStats_updatePortLinkCfg,
        .runTest            = &CpswTestFifoStats_Run,
        .deInitTest         = NULL,
        .macPortMask        = CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_0)),
        .numTxCh            = 1,
        .numRxFlow          = 1,
    },
    /* Source Address update */
    {
        .taskCfgId          = TASK_CFG_ID_SRCADDRUPDATE_TEST,
        .txChCfgId          = {TXCH_CFG_ID_SRCADDRUPDATE_TEST},
        .rxFlowCfgId        = {RXFLOW_CFG_ID_COMMON_TEST},
        .taskId             = 9U,
        .cpswType           = CPSW_9G,
        .openPrmsUpdate     = &CpswTestAleSrcAddrUpdate_setOpenPrms,
        .portLinkPrmsUpdate = &CpswTestAleSrcAddrUpdate_updatePortLinkCfg,
        .runTest            = &CpswTestAleSrcAddrUpdate_Run,
        .deInitTest         = NULL,
        .macPortMask        = CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_0)),
        .numTxCh            = 1,
        .numRxFlow          = 1,
    },
    /* ALE Table full */
    {
        .taskCfgId          = TASK_CFG_ID_TABLEFULL_TEST,
        .txChCfgId          = {TXCH_CFG_ID_TABLEFULL_TEST},
        .rxFlowCfgId        = {RXFLOW_CFG_ID_COMMON_TEST},
        .taskId             = 10U,
        .cpswType           = CPSW_9G,
        .openPrmsUpdate     = &CpswTestAleTableFull_setOpenPrms,
        .portLinkPrmsUpdate = &CpswTestAleTableFull_updatePortLinkCfg,
        .runTest            = &CpswTestAleTableFull_Run,
        .deInitTest         = NULL,
        .macPortMask        = CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_0)),
        .numTxCh            = 1,
        .numRxFlow          = 1,
    },
    /* VALN Drop untagged */
    {
        .taskCfgId          = TASK_CFG_ID_VLAN_DROP_UNTAGGED_TEST,
        .txChCfgId          = {TXCH_CFG_ID_COMMON_TEST},
        .rxFlowCfgId        = {RXFLOW_CFG_ID_COMMON_TEST},
        .taskId             = 11U,
        .cpswType           = CPSW_9G,
        .openPrmsUpdate     = &CpswTestVlanDropUntagged_setOpenPrms,
        .portLinkPrmsUpdate = NULL,
        .runTest            = &CpswTestCommon_Run,
        .deInitTest         = NULL,
        .macPortMask        = CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_0)),
        .numTxCh            = 1,
        .numRxFlow          = 1,
    },
    /* Multicast support */
    {
        .taskCfgId          = TASK_CFG_ID_MULTICAST_TEST,
        .txChCfgId          = {TXCH_CFG_ID_COMMON_TEST},
        .rxFlowCfgId        = {RXFLOW_CFG_ID_MULTICAST_TEST},
        .taskId             = 12U,
        .cpswType           = CPSW_9G,
        .openPrmsUpdate     = &CpswTestMulticast_setOpenPrms,
        .portLinkPrmsUpdate = &CpswTestMulticast_updatePortLinkCfg,
        .runTest            = &CpswTestMulticast_Run,
        .deInitTest         = NULL,
        .macPortMask        = CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_0)),
        .numTxCh            = 1,
        .numRxFlow          = 1,
    },
    /* Policer support */
    {
        .taskCfgId = TASK_CFG_ID_POLICER_TEST,
        .txChCfgId =
        {
            [CPSWTESTPOLICER_DEFAULT_TXCHCFGID] = TXCH_CFG_ID_COMMON_TEST,
            [CPSWTESTPOLICER_TYPE1_TXCHCFGID]   = TXCH_CFG_ID_POLICER_TEST,
            [CPSWTESTPOLICER_TYPE2_TXCHCFGID]   = TXCH_CFG_ID_COMMON_TEST
        },
        .rxFlowCfgId =
        {
            [CPSWTESTPOLICER_DEFAULT_RXFLOWCFGID] = RXFLOW_CFG_ID_POLICER1_TEST,
            [CPSWTESTPOLICER_TYPE1_RXFLOWCFGID]   = RXFLOW_CFG_ID_POLICER2_TEST,
            [CPSWTESTPOLICER_TYPE2_RXFLOWCFGID]   = RXFLOW_CFG_ID_POLICER2_TEST,
            [CPSWTESTPOLICER_TYPE3_RXFLOWCFGID]   = RXFLOW_CFG_ID_POLICER2_TEST,
        },
        .taskId             = 13U,
        .cpswType           = CPSW_9G,
        .openPrmsUpdate     = &CpswTestPolicer_setOpenPrms,
        .portLinkPrmsUpdate = &CpswTestPolicer_updatePortLinkCfg,
        .runTest            = &CpswTestPolicer_Run,
        .deInitTest         = NULL,
        .macPortMask        = (CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_3)) |
                               CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_2))),
        .numTxCh   = 3,
        .numRxFlow = 4,
    },
    /* Network security */
    {
        .taskCfgId          = TASK_CFG_ID_SECURITY_TEST,
        .txChCfgId          = {TXCH_CFG_ID_COMMON_TEST},
        .rxFlowCfgId        = {RXFLOW_CFG_ID_SECURITY_TEST},
        .taskId             = 14U,
        .cpswType           = CPSW_9G,
        .openPrmsUpdate     = &CpswTestSecurity_setOpenPrms,
        .portLinkPrmsUpdate = &CpswTestSecurity_updatePortLinkCfg,
        .runTest            = &CpswTestSecurity_Run,
        .deInitTest         = NULL,
        .macPortMask        = (CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_3)) |
                               CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_2))),
        .numTxCh   = 1,
        .numRxFlow = 1,
    },
    /* Host port Rx Filter */
    {
        .taskCfgId          = TASK_CFG_ID_RXFILTER_TEST,
        .txChCfgId          = {TXCH_CFG_ID_COMMON_TEST},
        .rxFlowCfgId        = {RXFLOW_CFG_ID_RXFILTER_TEST},
        .taskId             = 15U,
        .cpswType           = CPSW_9G,
        .openPrmsUpdate     = &CpswTestRxFilter_setOpenPrms,
        .portLinkPrmsUpdate = &CpswTestRxFilter_updatePortLinkCfg,
        .runTest            = &CpswTestRxFilter_Run,
        .deInitTest         = NULL,
        .macPortMask        = CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_3)),
        .numTxCh            = 1,
        .numRxFlow          = 1,
    },
    /* Policer nomatch */
    {
        .taskCfgId   = TASK_CFG_ID_POLICER_NOMATCH_TEST,
        .txChCfgId   = {TXCH_CFG_ID_COMMON_TEST},
        .rxFlowCfgId =
        {
            [CPSWTESTPOLICER_NOMATCH_DEFAULT_RXFLOWCFGID] = RXFLOW_CFG_ID_POLICER_NOMATCH_TEST,
            [CPSWTESTPOLICER_NOMATCH_TYPE1_RXFLOWCFGID]   = RXFLOW_CFG_ID_NONDEFAULT_POLICER_NOMATCH_TEST
        },
        .taskId             = 16U,
        .cpswType           = CPSW_9G,
        .openPrmsUpdate     = &CpswTestPolicerNomatch_setOpenPrms,
        .portLinkPrmsUpdate = &CpswTestPolicerNomatch_updatePortLinkCfg,
        .runTest            = &CpswTestPolicerNomatch_Run,
        .deInitTest         = NULL,
        .macPortMask        = (CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_3)) |
                               CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_2))),
        .numTxCh   = 1,
        .numRxFlow = 2,
    },
    /* Intervlan  */
    {
        .taskCfgId          = TASK_CFG_ID_INTERVLAN_TEST,
        .txChCfgId          = {TXCH_CFG_ID_INTERVLAN_TEST},
        .rxFlowCfgId        = {RXFLOW_CFG_ID_INTERVLAN_TEST},
        .taskId             = 17U,
        .cpswType           = CPSW_9G,
        .openPrmsUpdate     = &CpswTestInterVlan_setOpenPrms,
        .portLinkPrmsUpdate = &CpswTestInterVlan_updatePortLinkCfg,
        .runTest            = &CpswTestInterVlan_Run,
        .deInitTest         = NULL,
        .macPortMask        = (CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_3)) |
                               CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_2))),
        .numTxCh   = 1,
        .numRxFlow = 1,
    },
#if defined(SOC_J721E)
    /* Default Priority test  */
    {
        .taskCfgId = TASK_CFG_ID_DEFAULT_PRIORITY_TEST,
        .txChCfgId =
        {
            [CPSWTESTDEFAULTPRIORITY_DEFAULTCHID] = TXCH_CFG_ID_COMMON_TEST,
            [CPSWTESTDEFAULTPRIORITY1_TXCHID]     = TXCH_CFG_ID_DEFAULT_PRIORITY_TEST,
            [CPSWTESTDEFAULTPRIORITY2_TXCHID]     = TXCH_CFG_ID_DEFAULT_PRIORITY_TEST,
            [CPSWTESTDEFAULTPRIORITY3_TXCHID]     = TXCH_CFG_ID_DEFAULT_PRIORITY_TEST,
        },
        .rxFlowCfgId =
        {
            [CPSWTESTDEFAULTPRIORITY_DEFAULTFLOWID] = RXFLOW_CFG_ID_COMMON_TEST,
            [CPSWTESTDEFAULTPRIORITY1_RXFLOWID]     = RXFLOW_CFG_ID_DEFAULT_PRIORITY_TEST,
            [CPSWTESTDEFAULTPRIORITY2_RXFLOWID]     = RXFLOW_CFG_ID_DEFAULT_PRIORITY_TEST,
            [CPSWTESTDEFAULTPRIORITY3_RXFLOWID]     = RXFLOW_CFG_ID_DEFAULT_PRIORITY_TEST,
        },
        .taskId             = 18U,
        .cpswType           = CPSW_9G,
        .openPrmsUpdate     = &CpswTestDefaultPriority_setOpenPrms,
        .portLinkPrmsUpdate = &CpswTestDefaultPriority_updatePortLinkCfg,
        .runTest            = &CpswTestDefaultPriority_Run,
        .deInitTest         = NULL,
        .macPortMask        = (CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_3)) |
                               CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_2))),
        .numTxCh   = 4,
        .numRxFlow = 4,
    },
#endif
    /* Port Vlan Id  */
    {
        .taskCfgId          = TASK_CFG_ID_PVID_TEST,
        .txChCfgId          = {TXCH_CFG_ID_COMMON_TEST},
        .rxFlowCfgId        = {RXFLOW_CFG_ID_PVID_TEST},
        .taskId             = 19U,
        .cpswType           = CPSW_9G,
        .openPrmsUpdate     = &CpswTestPvid_setOpenPrms,
        .portLinkPrmsUpdate = &CpswTestPvid_updatePortLinkCfg,
        .runTest            = &CpswTestPvid_Run,
        .deInitTest         = NULL,
        .macPortMask        = CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_3)),
        .numTxCh            = 1,
        .numRxFlow          = 1,
    },
    /* Sanity Test */
    {
        .taskCfgId          = TASK_CFG_ID_SANITY_TEST,
        .txChCfgId          = {TXCH_CFG_ID_COMMON_TEST},
        .rxFlowCfgId        = {RXFLOW_CFG_ID_COMMON_TEST},
        .taskId             = 20U,
        .cpswType           = CPSW_9G,
        .openPrmsUpdate     = NULL,
        .portLinkPrmsUpdate = &CpswTestSanity_updatePortLinkCfgAutoNeg,
        .runTest            = &CpswTestSanity_Run,
        .deInitTest         = NULL,
        .macPortMask        = CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_3)),
        .numTxCh            = 1,
        .numRxFlow          = 1,
    },
    /* Outer VLAN */
    {
        .taskCfgId          = TASK_CFG_ID_OUTERVLAN_TEST,
        .txChCfgId          = {TXCH_CFG_ID_COMMON_TEST},
        .rxFlowCfgId        = {RXFLOW_CFG_ID_OUTERVLAN_TEST},
        .taskId             = 21U,
        .cpswType           = CPSW_9G,
        .openPrmsUpdate     = &CpswTestOuterVlan_setOpenPrms,
        .portLinkPrmsUpdate = &CpswTestOuterVlan_updatePortLinkCfg,
        .runTest            = &CpswTestOuterVlan_Run,
        .deInitTest         = NULL,
        .macPortMask        = (CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_3)) |
                               CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_2))),
        .numTxCh   = 1,
        .numRxFlow = 1,
    },
    /* Traffic shaping */
    {
        .taskCfgId          = TASK_CFG_ID_TRAFFICSHAPING_TEST,
        .txChCfgId          = {TXCH_CFG_ID_COMMON_TEST},
        .rxFlowCfgId        = {RXFLOW_CFG_ID_COMMON_TEST},
        .taskId             = 22U,
        .cpswType           = CPSW_9G,
        .openPrmsUpdate     = &CpswTestTrafficShaping_setOpenPrms,
        .portLinkPrmsUpdate = &CpswTestTrafficShaping_updatePortLinkCfg,
        .runTest            = &CpswTestTrafficShaping_Run,
        .deInitTest         = NULL,
        .macPortMask        = (CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_3)) |
                               CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_2))),
        .numTxCh   = 1,
        .numRxFlow = 1,
    },
    /* 1-Gbps full-duplex auto-negotiation test */
    {
        .taskCfgId          = TASK_CFG_ID_AUTONEGOTIATION_1G_FD_TEST,
        .txChCfgId          = {TXCH_CFG_ID_COMMON_TEST},
        .rxFlowCfgId        = {RXFLOW_CFG_ID_COMMON_TEST},
        .taskId             = 23U,
        .cpswType           = CPSW_9G,
        .openPrmsUpdate     = NULL,
        .portLinkPrmsUpdate = &CpswTestNway_updatePortLinkCfg1GFd,
        .runTest            = &CpswTestNway_Run,
        .deInitTest         = NULL,
        .macPortMask        = CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_0)),
        .numTxCh            = 1,
        .numRxFlow          = 1,
    },
    /* 100-Mbps full-duplex auto-negotiation test */
    {
        .taskCfgId          = TASK_CFG_ID_AUTONEGOTIATION_100M_FD_TEST,
        .txChCfgId          = {TXCH_CFG_ID_COMMON_TEST},
        .rxFlowCfgId        = {RXFLOW_CFG_ID_COMMON_TEST},
        .taskId             = 24U,
        .cpswType           = CPSW_9G,
        .openPrmsUpdate     = NULL,
        .portLinkPrmsUpdate = &CpswTestNway_updatePortLinkCfg100MFd,
        .runTest            = &CpswTestNway_Run,
        .deInitTest         = NULL,
        .macPortMask        = CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_0)),
        .numTxCh            = 1,
        .numRxFlow          = 1,
    },
    /* 100-Mbps half-duplex auto-negotiation test */
    {
        .taskCfgId          = TASK_CFG_ID_AUTONEGOTIATION_100M_HD_TEST,
        .txChCfgId          = {TXCH_CFG_ID_COMMON_TEST},
        .rxFlowCfgId        = {RXFLOW_CFG_ID_COMMON_TEST},
        .taskId             = 25U,
        .cpswType           = CPSW_9G,
        .openPrmsUpdate     = NULL,
        .portLinkPrmsUpdate = &CpswTestNway_updatePortLinkCfg100MHd,
        .runTest            = &CpswTestNway_Run,
        .deInitTest         = NULL,
        .macPortMask        = CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_0)),
        .numTxCh            = 1,
        .numRxFlow          = 1,
    },
    /* PDK-3723 - DMA Rx flow Max. length test - RX Flow to drop packet if exceed rxFlowMtu */

    /* In this test, we configure port in loopback mode and from host port send alternate packets
     * with correct RX flow MTU & packets greater than RX flow MTU. Since packets > RX flow MTU gets
     * dropped in drop ring using "RX flow size threshold" configuration, we expect to receive only
     * half packets. We mark test passed if we don't receive all packets transmitted */
    {
        .taskCfgId          = TASK_CFG_ID_RXFLOW_MTU_TEST,
        .txChCfgId          = {TXCH_CFG_ID_RXFLOW_MTU_TEST},
        .rxFlowCfgId        = {RXFLOW_CFG_ID_RXFLOW_MTU_TEST},
        .taskId             = 26U,
        .cpswType           = CPSW_9G,
        .openPrmsUpdate     = NULL,
        .portLinkPrmsUpdate = CpswTestRxFlowMtu_updatePortLinkCfg,
        .runTest            = CpswTestCommon_Run, // &CpswTestRxFlowMtu_Run,
        .deInitTest         = NULL,
        .macPortMask        = CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_0)),
        .numTxCh            = 1,
        .numRxFlow          = 1,
    },
    /* PDK-3722 - PHY strapping mode - PHY LLD to bypass the some states in FSM for faster link up */
    {
        .taskCfgId          = TASK_CFG_ID_PHY_STRAPPING_TEST,
        .txChCfgId          = {TXCH_CFG_ID_COMMON_TEST},
        .rxFlowCfgId        = {RXFLOW_CFG_ID_COMMON_TEST},
        .taskId             = 27U,
        .cpswType           = CPSW_9G,
        .openPrmsUpdate     = &CpswTestPhyStrap_openPrmsUpdate,
        .portLinkPrmsUpdate = &CpswTestPhyStrap_updatePortLinkCfg,
        .runTest            = &CpswTestCommon_Run,
        .deInitTest         = NULL,

        /* We use MAC PORT 7 with RMII PHY for strap test as this PHY doesn't need any additional
         * configuration via extendedConfig. We can't use any of RGMII PHY's as they need this extendedConfig
         * for delay, drive strength settings */
        .macPortMask = CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_7)),
        .numTxCh     = 1,
        .numRxFlow   = 1,
    },
    /* Priority Regeneration test  */
    {
        .taskCfgId = TASK_CFG_ID_PRIORITY_REGENERATION_TEST,
        .txChCfgId = {[CPSWTESTPRIORITYREGENERATION_DEFAULTCHID] = TXCH_CFG_ID_COMMON_TEST, },
        .rxFlowCfgId = {[CPSWTESTPRIORITYREGENERATION_DEFAULTFLOWID] = RXFLOW_CFG_ID_COMMON_TEST, },
        .taskId             = 28U,
        .cpswType           = CPSW_9G,
        .openPrmsUpdate     = &CpswTestPriorityRegeneration_setOpenPrms,
        .portLinkPrmsUpdate = &CpswTestPriorityRegeneration_setMacConfig,
        .runTest            = &CpswTestPriorityRegeneration_run,
        .deInitTest         = NULL,
        .macPortMask        = (CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_3)) |
                               CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_2))),
        .numTxCh   = 1,
        .numRxFlow = 1,
    },
#if defined(SOC_J721E)
    /* PDK- - SGMII loopback test - SGMII internal/digital loopback(before the SERDES) from the CPSGMII
     * transmit to the CPSGMII receive */
    {
        .taskCfgId          = TASK_CFG_ID_SGMII_LPBK_TEST,
        .txChCfgId          = {TXCH_CFG_ID_COMMON_TEST},
        .rxFlowCfgId        = {RXFLOW_CFG_ID_COMMON_TEST},
        .taskId             = 29U,
        .cpswType           = CPSW_9G,
        .openPrmsUpdate     = NULL,
        .portLinkPrmsUpdate = CpswTestSgmiiLpbk_updatePortLinkCfg,
        .runTest            = &CpswTestCommon_Run,
        .deInitTest         = NULL,
        .macPortMask = CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_1)),
        .numTxCh     = 1,
        .numRxFlow   = 1,
    },
#endif
    /* Switch logic test for QSGMII DB (ENET_EXP) ports*/
    {
        .taskCfgId          = TASK_CFG_ID_QSGMII_SWITCHING_TEST,
        .txChCfgId          = {TXCH_CFG_ID_COMMON_TEST},
        .rxFlowCfgId        = {RXFLOW_CFG_ID_SWITCHING_TEST},
        .openPrmsUpdate     = NULL,
        .portLinkPrmsUpdate = NULL,
        .runTest            = &CpswTestSwitching_Run,
        .deInitTest         = NULL,
        .taskId             = 30U,
        .cpswType           = CPSW_9G,
        .macPortMask        = CPSW_TEST_EVM_QSGMII_MACPORT_MASK,
        .numTxCh   = 1,
        .numRxFlow = 1,
    },
    /* Switch logic test with all 8 ports*/
    {
        .taskCfgId          = TASK_CFG_ID_ALL_PORTS_SWITCH_TEST,
        .txChCfgId          = {TXCH_CFG_ID_COMMON_TEST},
        .rxFlowCfgId        = {RXFLOW_CFG_ID_SWITCHING_TEST},
        .openPrmsUpdate     = NULL,
        .portLinkPrmsUpdate = NULL,
        .runTest            = &CpswTestSwitching_Run,
        .deInitTest         = NULL,
        .taskId             = 31U,
        .cpswType           = CPSW_9G,
        .macPortMask        = CPSW_TEST_ALL_MACPORT_MASK,
        .numTxCh   = 1,
        .numRxFlow = 1,
    },
#if defined(SOC_J721E)
    /* CPTS Event test */
    {
        .taskCfgId          = TASK_CFG_ID_CPTS_EVENT_TEST,
        .txChCfgId          = {TXCH_CFG_ID_CPTS_EVENT_TEST},
        .rxFlowCfgId        = {RXFLOW_CFG_ID_CPTS_EVENT_TEST},
        .taskId             = 32U,
        .cpswType           = CPSW_9G,
        .openPrmsUpdate     = &CpswTestCptsEvent_setOpenPrms,
        .portLinkPrmsUpdate = &CpswTestCptsEvent_updatePortLinkCfg,
        .runTest            = &CpswTestCptsEvent_Run,
        .deInitTest         = NULL,
        .macPortMask        = CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_0)),
        .numTxCh            = 1,
        .numRxFlow          = 1,
    },
#endif
    /* BCast/MCast Rate Limit test  */
    {
        .taskCfgId          = TASK_CFG_ID_BCASTMCAST_LIMIT_TEST,
        .txChCfgId          = {TXCH_CFG_ID_COMMON_TEST},
        .rxFlowCfgId        = {RXFLOW_CFG_ID_COMMON_TEST},
        .taskId             = 33U,
        .cpswType           = CPSW_9G,
        .openPrmsUpdate     = &CpswTestBcastMcastLimit_setOpenPrms,
        .portLinkPrmsUpdate = NULL,
        .runTest            = &CpswTestBcastMcastLimit_Run,
        .deInitTest         = NULL,
        .macPortMask        = (CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_3)) |
                               CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_2))),
        .numTxCh            = 1,
        .numRxFlow          = 1,
    },
#if defined(SOC_J721E)
    /* Directed packet test  */
    {
        .taskCfgId          = TASK_CFG_ID_DIRECTED_PKT_TEST,
        .txChCfgId          = {TXCH_CFG_ID_DIRECTED_PKT_TEST},
        .rxFlowCfgId        = {RXFLOW_CFG_ID_DIRECTED_PKT_TEST},
        .taskId             = 34U,
        .cpswType           = CPSW_9G,
        .openPrmsUpdate     = NULL,
        .portLinkPrmsUpdate = &CpswTestDirectedPkt_updatePortLinkCfg,
        .runTest            = &CpswTestDirectedPkt_Run,
        .deInitTest         = NULL,
        .macPortMask        = CPSW_SET_BIT(CPSW_NORMALIZE_MACPORT(CPSW_MAC_PORT_3)),
        .numTxCh            = 1,
        .numRxFlow          = 1,
    },
#endif
};

CpswTestTxChCfgInfo gTestTxChCfg[] =
{
    {
        .txChCfgId        = TXCH_CFG_ID_COMMON_TEST,
        .updateTxChPrms   = NULL,
        .setTxPktInfo     = NULL,
        .maxTxPktLen      = CPSW_TEST_COMMON_PKT_LEN,
        .pktSendCount     = CPSW_TEST_COMMON_PKT_SENDCOUNT,
        .pktSendLoopCount = CPSW_TEST_COMMON_NUM_LOOP,
    },
    {
        .txChCfgId        = TXCH_CFG_ID_STATS_TEST,
        .updateTxChPrms   = NULL,
        .setTxPktInfo     = NULL,
        .maxTxPktLen      = CPSW_TEST_COMMON_PKT_LEN,
        .pktSendCount     = CPSW_TEST_STATS_PKT_COUNT,
        .pktSendLoopCount = CPSW_TEST_COMMON_NUM_LOOP,
    },
    {
        .txChCfgId        = TXCH_CFG_ID_CRCSTRIP_TEST,
        .updateTxChPrms   = NULL,
        .setTxPktInfo     = &CpswTestCrcStrip_setTxPktInfo,
        .maxTxPktLen      = CPSW_TEST_COMMON_PKT_LEN,
        .pktSendCount     = CPSW_TEST_COMMON_PKT_SENDCOUNT,
        .pktSendLoopCount = CPSW_TEST_COMMON_NUM_LOOP,
    },
    {
        .txChCfgId        = TXCH_CFG_ID_SRCADDRUPDATE_TEST,
        .updateTxChPrms   = NULL,
        .setTxPktInfo     = &CpswTestAleSrcAddrUpdate_setTxPktInfo,
        .maxTxPktLen      = CPSW_TEST_COMMON_PKT_LEN,
        .pktSendCount     = CPSW_TEST_SRCADDRUPDATE_PKT_SENDCOUNT,
        .pktSendLoopCount = CPSW_TEST_COMMON_NUM_LOOP,
        .postTxSend       = &CpswTestAleSrcAddrUpdate_postTxSend,
    },
    {
        .txChCfgId        = TXCH_CFG_ID_TABLEFULL_TEST,
        .updateTxChPrms   = NULL,
        .setTxPktInfo     = NULL,
        .maxTxPktLen      = CPSW_TEST_COMMON_PKT_LEN,
        .pktSendCount     = CPSW_TEST_TABLEFULL_PKT_SENDCOUNT,
        .pktSendLoopCount = CPSW_TEST_COMMON_NUM_LOOP,
    },
    {
        .txChCfgId        = TXCH_CFG_ID_FIFOSTATS_TEST,
        .updateTxChPrms   = NULL,
        .setTxPktInfo     = NULL,
        .maxTxPktLen      = CPSW_TEST_COMMON_PKT_LEN,
        .pktSendCount     = CPSW_TEST_FIFISTATS_PKT_COUNT,
        .pktSendLoopCount = CPSW_TEST_COMMON_NUM_LOOP,
    },
    {
        .txChCfgId        = TXCH_CFG_ID_POLICER_TEST,
        .updateTxChPrms   = &CpswTestPolicer_setTxOpenParams,
        .setTxPktInfo     = NULL,
        .maxTxPktLen      = CPSW_TEST_COMMON_PKT_LEN,
        .pktSendCount     = CPSW_TEST_COMMON_PKT_SENDCOUNT,
        .pktSendLoopCount = CPSW_TEST_COMMON_NUM_LOOP,
    },
    {
        .txChCfgId        = TXCH_CFG_ID_INTERVLAN_TEST,
        .updateTxChPrms   = NULL,
        .setTxPktInfo     = NULL,
        .maxTxPktLen      = CPSW_TEST_COMMON_PKT_LEN,
        .pktSendCount     = CPSW_TEST_INTERVLAN_PKT_SENDCOUNT,
        .pktSendLoopCount = CPSW_TEST_COMMON_NUM_LOOP,
    },
    {
        .txChCfgId        = TXCH_CFG_ID_RXFLOW_MTU_TEST,
        .updateTxChPrms   = NULL,
        .setTxPktInfo     = &CpswTestRxFlowMtu_setTxPktInfo,
        .maxTxPktLen      = CPSW_TEST_COMMON_PKT_LEN,
        .pktSendCount     = CPSW_TEST_RXFLOW_MTU_PKT_SENDCOUNT,
        .pktSendLoopCount = CPSW_TEST_COMMON_NUM_LOOP,
    },
#if defined(SOC_J721E)
    {
        .txChCfgId        = TXCH_CFG_ID_DEFAULT_PRIORITY_TEST,
        .updateTxChPrms   = NULL,
        .setTxPktInfo     = &CpswTestDefaultPriority_setTxPktInfo,
        .maxTxPktLen      = CPSW_TEST_COMMON_PKT_LEN,
        .pktSendCount     = CPSW_TEST_COMMON_PKT_SENDCOUNT,
        .pktSendLoopCount = CPSW_TEST_COMMON_NUM_LOOP,
    },
    {
        .txChCfgId        = TXCH_CFG_ID_CPTS_EVENT_TEST,
        .updateTxChPrms   = NULL,
        .setTxPktInfo     = &CpswTestCptsEvent_setTxPktInfo,
        .maxTxPktLen      = CPSW_TEST_COMMON_PKT_LEN,
        .pktSendCount     = CPSW_TEST_CPTS_EVENT_SENDCOUNT,
        .pktSendLoopCount = CPSW_TEST_CPTS_EVENT_NUM_LOOP,
    },
    {
        .txChCfgId        = TXCH_CFG_ID_DIRECTED_PKT_TEST,
        .updateTxChPrms   = NULL,
        .setTxPktInfo     = &CpswTestDirectedPkt_setTxPktInfo,
        .maxTxPktLen      = CPSW_TEST_COMMON_PKT_LEN,
        .pktSendCount     = CPSW_TEST_DIRECTED_PKT_SENDCOUNT,
        .pktSendLoopCount = CPSW_TEST_DIRECTED_PKT_NUM_LOOP,
    },
#endif
};

CpswTestRxFlowCfgInfo gTestRxFlowCfg[] =
{
    {
        .rxFlowCfgId      = RXFLOW_CFG_ID_COMMON_TEST,
        .updateRxFlowPrms = NULL,
        .rxFlowMtu        = CPSW_TEST_COMMON_PKT_LEN,
        .processRxPkt     = NULL,
        .pktRecvCount     = CPSW_TEST_COMMON_PKT_RECVCOUNT,
        .useDefaultFlow   = true,
    },
    {
        .rxFlowCfgId      = RXFLOW_CFG_ID_SWITCHING_TEST,
        .updateRxFlowPrms = NULL,
        .rxFlowMtu        = CPSW_TEST_COMMON_PKT_LEN,
        .processRxPkt     = NULL,
        .pktRecvCount     = CPSW_TEST_SWITCHING_PKT_RECVCOUNT,
        .useDefaultFlow   = true,
    },
    {
        .rxFlowCfgId      = RXFLOW_CFG_ID_CRCSTRIP_TEST,
        .updateRxFlowPrms = NULL,
        .rxFlowMtu        = CPSW_TEST_COMMON_PKT_LEN,
        .processRxPkt     = &CpswTestCrcStrip_processRxPkt,
        .pktRecvCount     = CPSW_TEST_COMMON_PKT_RECVCOUNT,
        .useDefaultFlow   = true,
    },
    {
        .rxFlowCfgId      = RXFLOW_CFG_ID_FIFOSTATS_TEST,
        .updateRxFlowPrms = NULL,
        .rxFlowMtu        = CPSW_TEST_COMMON_PKT_LEN,
        .processRxPkt     = &CpswTestFifoStats_processRxPkt,
        .pktRecvCount     = CPSW_TEST_FIFISTATS_PKT_COUNT,
        .useDefaultFlow   = true,
    },
    {
        .rxFlowCfgId      = RXFLOW_CFG_ID_MULTICAST_TEST,
        .updateRxFlowPrms = NULL,
        .rxFlowMtu        = CPSW_TEST_COMMON_PKT_LEN,
        .processRxPkt     = NULL,
        .pktRecvCount     = CPSW_TEST_MULTICAST_PKT_RECVCOUNT,
        .useDefaultFlow   = true,
    },
    {
        .rxFlowCfgId      = RXFLOW_CFG_ID_STATS_TEST,
        .updateRxFlowPrms = NULL,
        .rxFlowMtu        = CPSW_TEST_COMMON_PKT_LEN,
        .processRxPkt     = NULL,
        .pktRecvCount     = CPSW_TEST_STATS_PKT_COUNT,
        .useDefaultFlow   = true,
        .postRxProcess    = &CpswTestStats_PostProcessRxPkt,
    },
    {
        .rxFlowCfgId      = RXFLOW_CFG_ID_SECURITY_TEST,
        .updateRxFlowPrms = NULL,
        .rxFlowMtu        = CPSW_TEST_COMMON_PKT_LEN,
        .processRxPkt     = &CpswTestSecurity_processRxPkt,
        .pktRecvCount     = CPSW_TEST_COMMON_PKT_RECVCOUNT,
        .useDefaultFlow   = true,
    },
    {
        .rxFlowCfgId      = RXFLOW_CFG_ID_RXFILTER_TEST,
        .updateRxFlowPrms = NULL,
        .rxFlowMtu        = CPSW_TEST_COMMON_PKT_LEN,
        .processRxPkt     = &CpswTestRxFilter_processRxPkt,
        .pktRecvCount     = CPSW_TEST_RXFILTER_PKT_RECVCOUNT,
        .useDefaultFlow   = true,
    },
    {
        .rxFlowCfgId      = RXFLOW_CFG_ID_POLICER1_TEST,
        .updateRxFlowPrms = &CpswTestPolicer_setRxOpenParams,
        .rxFlowMtu        = CPSW_TEST_COMMON_PKT_LEN,
        .processRxPkt     = NULL,
        .pktRecvCount     = CPSW_TEST_POLICER_PKT_RECVCOUNT,
        .useDefaultFlow   = true,
    },
    {
        .rxFlowCfgId      = RXFLOW_CFG_ID_POLICER2_TEST,
        .updateRxFlowPrms = NULL,
        .rxFlowMtu        = CPSW_TEST_COMMON_PKT_LEN,
        .processRxPkt     = NULL,
        .pktRecvCount     = CPSW_TEST_POLICER_PKT_RECVCOUNT,
        .useDefaultFlow   = false,
    },
    {
        .rxFlowCfgId      = RXFLOW_CFG_ID_POLICER_NOMATCH_TEST,
        .updateRxFlowPrms = NULL,
        .rxFlowMtu        = CPSW_TEST_COMMON_PKT_LEN,
        .processRxPkt     = &CpswTestPolicerNomatch_processRxPkt,
        .pktRecvCount     = CPSW_TEST_POLICER_NOMATCH_PKT_RECVCOUNT,
        .useDefaultFlow   = true,
    },
    {
        .rxFlowCfgId      = RXFLOW_CFG_ID_NONDEFAULT_POLICER_NOMATCH_TEST,
        .updateRxFlowPrms = NULL,
        .rxFlowMtu        = CPSW_TEST_COMMON_PKT_LEN,
        .processRxPkt     = &CpswTestPolicerNomatch_processRxPkt,
        .pktRecvCount     = CPSW_TEST_POLICER_NOMATCH_PKT_RECVCOUNT,
        .useDefaultFlow   = false,
    },
    {
        .rxFlowCfgId      = RXFLOW_CFG_ID_INTERVLAN_TEST,
        .updateRxFlowPrms = NULL,
        .rxFlowMtu        = CPSW_TEST_COMMON_PKT_LEN,
        .processRxPkt     = &CpswTestInterVlan_processRxPkt,
        .pktRecvCount     = CPSW_TEST_INTERVLAN_PKT_RECVCOUNT,
        .useDefaultFlow   = true,
    },
    {
        .rxFlowCfgId      = RXFLOW_CFG_ID_PVID_TEST,
        .updateRxFlowPrms = NULL,
        .rxFlowMtu        = CPSW_TEST_COMMON_PKT_LEN,
        .processRxPkt     = &CpswTestPvid_processRxPkt,
        .pktRecvCount     = CPSW_TEST_COMMON_PKT_RECVCOUNT,
        .useDefaultFlow   = true,
    },
    {
        .rxFlowCfgId      = RXFLOW_CFG_ID_OUTERVLAN_TEST,
        .updateRxFlowPrms = NULL,
        .rxFlowMtu        = CPSW_TEST_COMMON_PKT_LEN,
        .processRxPkt     = &CpswTestOuterVlan_processRxPkt,
        .pktRecvCount     = CPSW_TEST_OUTERVLAN_PKT_RECVCOUNT,
        .useDefaultFlow   = true,
    },
    {
        .rxFlowCfgId      = RXFLOW_CFG_ID_RXFLOW_MTU_TEST,
        .updateRxFlowPrms = NULL,
        .rxFlowMtu        = CPSW_TEST_RXFLOW_MTU_PKT_LEN,
        .processRxPkt     = NULL,
        /* We drop half of transmitted packets */
        .pktRecvCount   = CPSW_TEST_RXFLOW_MTU_PKT_RECVCOUNT,
        .useDefaultFlow = true,
    },
#if defined(SOC_J721E)
    {
        .rxFlowCfgId      = RXFLOW_CFG_ID_DEFAULT_PRIORITY_TEST,
        .updateRxFlowPrms = NULL,
        .rxFlowMtu        = CPSW_TEST_COMMON_PKT_LEN,
        .processRxPkt     = &CpswTestDefaultPriority_processRxPkt,
        .pktRecvCount     = CPSW_TEST_COMMON_PKT_RECVCOUNT,
        .useDefaultFlow   = false,
    },
    {
        .rxFlowCfgId      = RXFLOW_CFG_ID_CPTS_EVENT_TEST,
        .updateRxFlowPrms = NULL,
        .rxFlowMtu        = CPSW_TEST_COMMON_PKT_LEN,
        .processRxPkt     = NULL,
        .pktRecvCount     = CPSW_TEST_CPTS_EVENT_RECVCOUNT,
        .useDefaultFlow   = true,
    },
    {
        .rxFlowCfgId      = RXFLOW_CFG_ID_DIRECTED_PKT_TEST,
        .updateRxFlowPrms = NULL,
        .rxFlowMtu        = CPSW_TEST_COMMON_PKT_LEN,
        .processRxPkt     = &CpswTestDirectedPkt_processRxPkt,
        .pktRecvCount     = CPSW_TEST_DIRECTED_PKT_RECVCOUNT,
        .useDefaultFlow   = true,
    },
#endif
};

/* ========================================================================== */
/*                          Function Definitions                              */
/* ========================================================================== */
static inline CpswTestTaskCfg *Test_getTaskCfgParams(uint32_t taskCfgId)
{
    uint32_t loopCnt;

    for (loopCnt = 0U; loopCnt < CPSW_TEST_NUM_TASK_CFG_PARAMS; loopCnt++)
    {
        if (gTestTaskCfg[loopCnt].taskCfgId == taskCfgId)
        {
            break;
        }
    }

    if (loopCnt < CPSW_TEST_NUM_TASK_CFG_PARAMS)
    {
        return(&gTestTaskCfg[loopCnt]);
    }
    else
    {
        return(NULL);
    }
}

static inline CpswTestTxChCfgInfo *Test_getTxChCfgParams(uint32_t txChCfgId)
{
    uint32_t loopCnt;

    for (loopCnt = 0U; loopCnt < CPSW_TEST_NUM_TX_CH_CFG_PARAMS; loopCnt++)
    {
        if (gTestTxChCfg[loopCnt].txChCfgId == txChCfgId)
        {
            break;
        }
    }

    if (loopCnt < CPSW_TEST_NUM_TX_CH_CFG_PARAMS)
    {
        return(&gTestTxChCfg[loopCnt]);
    }
    else
    {
        return(NULL);
    }
}

static inline CpswTestRxFlowCfgInfo *Test_getRxFlowCfgParams(uint32_t rxFlowCfgId)
{
    uint32_t loopCnt;

    for (loopCnt = 0U; loopCnt < CPSW_TEST_NUM_RX_FLOW_CFG_PARAMS; loopCnt++)
    {
        if (gTestRxFlowCfg[loopCnt].rxFlowCfgId == rxFlowCfgId)
        {
            break;
        }
    }

    if (loopCnt < CPSW_TEST_NUM_RX_FLOW_CFG_PARAMS)
    {
        return(&gTestRxFlowCfg[loopCnt]);
    }
    else
    {
        return(NULL);
    }
}

#ifdef __cplusplus
}
#endif

#endif /* #ifndef CPSW_TEST_CONFIG_H_ */
