# File: enet_component.mk
#       This file is component include make file of XYZ driver library.
# List of variables set in this file and their purpose:
# <mod>_RELPATH        - This is the relative path of the module, typically from
#                        top-level directory of the package
# <mod>_PATH           - This is the absolute path of the module. It derives from
#                        absolute path of the top-level directory (set in env.mk)
#                        and relative path set above
# <mod>_INCLUDE        - This is the path that has interface header files of the
#                        module. This can be multiple directories (space separated)
# <mod>_PKG_LIST       - Names of the modules (and sub-modules) that are a part
#                        part of this module, including itself.
# <mod>_BOARD_DEPENDENCY - "yes": means the code for this module depends on
#                             platform and the compiled obj/lib has to be kept
#                             under <platform> directory
#                             "no" or "" or if this variable is not defined: means
#                             this module has no platform dependent code and hence
#                             the obj/libs are not kept under <platform> dir.
# <mod>_CORE_DEPENDENCY     - "yes": means the code for this module depends on
#                             core and the compiled obj/lib has to be kept
#                             under <core> directory
#                             "no" or "" or if this variable is not defined: means
#                             this module has no core dependent code and hence
#                             the obj/libs are not kept under <core> dir.
# <mod>_APP_STAGE_FILES     - List of source files that belongs to the module
#                             <mod>, but that needs to be compiled at application
#                             build stage (in the context of the app). This is
#                             primarily for link time configurations or if the
#                             source file is dependent on options/defines that are
#                             application dependent. This can be left blank or
#                             not defined at all, in which case, it means there
#                             no source files in the module <mod> that are required
#                             to be compiled in the application build stage.
#
ifeq ($(enet_component_make_include), )

drvenet_SOCLIST         = am65xx j721e j7200
drvenet_BOARDLIST       = am65xx_evm am65xx_idk j721e_evm j7200_evm
drvenet_am65xx_CORELIST = mpu1_0 mcu1_0
drvenet_j721e_CORELIST  = mpu1_0 mcu1_0 mcu2_0 mcu2_1
drvenet_j7200_CORELIST  = mpu1_0 mcu1_0 mcu2_0 mcu2_1
ifeq ($(BUILD_OS_TYPE), qnx)
drvenet_j721e_CORELIST += qnx_mpu1_0
endif

############################
# enet package
# List of components included under enet lib
# The components included here are built and will be part of enet lib
############################
enet_LIB_LIST =

############################
# enet app lib package
# List of components included under enet app lib
# The components included here are built and will be part of enet app lib
############################
enet_APP_LIB_LIST =

############################
# enet examples
# List of examples under enet (+= is used at each example definition)
# All the tests mentioned in list are built when test target is called
# List below all examples for allowed values
############################
enet_EXAMPLE_LIST =

#
# Enet Modules
#

#
# Enet library
#
export enet_COMP_LIST = enet
enet_RELPATH = ti/drv/enet
enet_PATH = $(PDK_ENET_COMP_PATH)
export enet_LIBNAME = enet
export enet_LIBPATH = $(PDK_ENET_COMP_PATH)/lib
export enet_MAKEFILE = -fsrc/makefile
export enet_BOARD_DEPENDENCY = no
export enet_CORE_DEPENDENCY = no
enet_PKG_LIST = enet
enet_INCLUDE = $(enet_PATH)
export enet_SOCLIST = $(drvenet_SOCLIST)
export enet_$(SOC)_CORELIST = $(drvenet_$(SOC)_CORELIST)
enet_LIB_LIST += enet

#
# Enet SoC library
#
export enetsoc_COMP_LIST = enetsoc
enetsoc_RELPATH = ti/drv/enet/soc
enetsoc_PATH = $(PDK_ENET_COMP_PATH)/soc
export enetsoc_LIBNAME = enetsoc
export enetsoc_LIBPATH = $(PDK_ENET_COMP_PATH)/lib
export enetsoc_MAKEFILE = -fmakefile
export enetsoc_BOARD_DEPENDENCY = no
export enetsoc_CORE_DEPENDENCY = yes
enetsoc_PKG_LIST = enetsoc
enetsoc_INCLUDE = $(enetsoc_PATH)
export enetsoc_SOCLIST = $(drvenet_SOCLIST)
export enetsoc_$(SOC)_CORELIST = $(drvenet_$(SOC)_CORELIST)
enet_LIB_LIST += enetsoc

#
# Enet PHY library
#
export enetphy_COMP_LIST = enetphy
enetphy_RELPATH = ti/drv/enet/src/phy
enetphy_PATH = $(PDK_ENET_COMP_PATH)/src/phy
export enetphy_LIBNAME = enetphy
export enetphy_LIBPATH = $(PDK_ENET_COMP_PATH)/lib
export enetphy_MAKEFILE = -fmakefile
export enetphy_BOARD_DEPENDENCY = no
export enetphy_CORE_DEPENDENCY = yes
enetphy_PKG_LIST = enetphy
enetphy_INCLUDE = $(enetphy_PATH)
export enetphy_SOCLIST = $(drvenet_SOCLIST)
export enetphy_$(SOC)_CORELIST = $(drvenet_$(SOC)_CORELIST)
enet_LIB_LIST += enetphy

#
# NIMUENET library
#
export nimuenet_COMP_LIST = nimuenet
nimuenet_RELPATH = ti/drv/enet/nimuenet
nimuenet_PATH = $(PDK_ENET_COMP_PATH)/nimuenet
export nimuenet_LIBNAME = nimuenet
export nimuenet_LIBPATH = $(PDK_ENET_COMP_PATH)/lib
export nimuenet_MAKEFILE = -fmakefile
export nimuenet_BOARD_DEPENDENCY = no
export nimuenet_CORE_DEPENDENCY = no
nimuenet_PKG_LIST = nimuenet
nimuenet_INCLUDE = $(nimuenet_PATH)
export nimuenet_SOCLIST = $(drvenet_SOCLIST)
export nimuenet_$(SOC)_CORELIST = $(drvenet_$(SOC)_CORELIST)
enet_LIB_LIST += nimuenet

#
# Enet config server library
#
enet_cfgserver_COMP_LIST = enet_cfgserver
enet_cfgserver_RELPATH = ti/drv/enet/enet_cfgserver
enet_cfgserver_PATH = $(PDK_ENET_COMP_PATH)/enet_cfgserver
enet_cfgserver_LIBNAME = enet_cfgserver
enet_cfgserver_LIBPATH = $(PDK_ENET_COMP_PATH)/lib
enet_cfgserver_MAKEFILE = -fmakefile
export enet_cfgserver_MAKEFILE
export enet_cfgserver_LIBNAME
export enet_cfgserver_LIBPATH
enet_cfgserver_BOARD_DEPENDENCY = no
enet_cfgserver_CORE_DEPENDENCY = no
export enet_cfgserver_COMP_LIST
export enet_cfgserver_BOARD_DEPENDENCY
export enet_cfgserver_CORE_DEPENDENCY
enet_cfgserver_PKG_LIST = enet_cfgserver
enet_cfgserver_INCLUDE = $(enet_cfgserver_PATH)
enet_cfgserver_SOCLIST = $(drvenet_SOCLIST)
export enet_cfgserver_SOCLIST
enet_cfgserver_$(SOC)_CORELIST = $(drvenet_$(SOC)_CORELIST)
export enet_cfgserver_$(SOC)_CORELIST
enet_LIB_LIST += enet_cfgserver


#
# Enet example utils (TI-RTOS) library
#
export enet_example_utils_COMP_LIST = enet_example_utils
enet_example_utils_RELPATH = ti/drv/enet/examples/utils
enet_example_utils_PATH = $(PDK_ENET_COMP_PATH)/examples/utils
export enet_example_utils_LIBNAME = enet_example_utils
export enet_example_utils_LIBPATH = $(PDK_ENET_COMP_PATH)/lib
export enet_example_utils_OBJPATH = $(enet_example_utils_RELPATH)/tirtos
export enet_example_utils_MAKEFILE = -fmakefile IS_BAREMETAL=no
export enet_example_utils_BOARD_DEPENDENCY = yes
export enet_example_utils_CORE_DEPENDENCY = yes
enet_example_utils_PKG_LIST = enet_example_utils
enet_example_utils_INCLUDE = $(enet_example_utils_PATH)
export enet_example_utils_SOCLIST = $(drvenet_SOCLIST)
export enet_example_utils_$(SOC)_CORELIST = $(drvenet_$(SOC)_CORELIST)
enet_LIB_LIST += enet_example_utils

#
# Enet example utils (TI-RTOS) library (full version)
#
export enet_example_utils_full_COMP_LIST = enet_example_utils_full
enet_example_utils_full_RELPATH = ti/drv/enet/examples/utils
enet_example_utils_full_PATH = $(PDK_ENET_COMP_PATH)/examples/utils
export enet_example_utils_full_LIBNAME = enet_example_utils_full
export enet_example_utils_full_LIBPATH = $(PDK_ENET_COMP_PATH)/lib
export enet_example_utils_full_OBJPATH = $(enet_example_utils_full_RELPATH)/tirtos_full
export enet_example_utils_full_MAKEFILE = -fmakefile IS_BAREMETAL=no APPUTILS_TYPE=full
export enet_example_utils_full_BOARD_DEPENDENCY = yes
export enet_example_utils_full_CORE_DEPENDENCY = yes
enet_example_utils_full_PKG_LIST = enet_example_utils_full
enet_example_utils_full_INCLUDE = $(enet_example_utils_full_PATH)
export enet_example_utils_full_SOCLIST = $(drvenet_SOCLIST)
export enet_example_utils_full_$(SOC)_CORELIST = $(drvenet_$(SOC)_CORELIST)
enet_LIB_LIST += enet_example_utils_full

#
# Enet example utils (baremetal) library
#
export enet_example_utils_baremetal_COMP_LIST = enet_example_utils_baremetal
enet_example_utils_baremetal_RELPATH = ti/drv/enet/examples/utils
enet_example_utils_baremetal_PATH = $(PDK_ENET_COMP_PATH)/examples/utils
export enet_example_utils_baremetal_LIBNAME = enet_example_utils_baremetal
export enet_example_utils_baremetal_LIBPATH = $(PDK_ENET_COMP_PATH)/lib
export enet_example_utils_baremetal_OBJPATH = $(enet_example_utils_baremetal_RELPATH)/baremetal
export enet_example_utils_baremetal_MAKEFILE = -fmakefile IS_BAREMETAL=yes
export enet_example_utils_baremetal_BOARD_DEPENDENCY = yes
export enet_example_utils_baremetal_CORE_DEPENDENCY = yes
enet_example_utils_baremetal_PKG_LIST = enet_example_utils_baremetal
enet_example_utils_baremetal_INCLUDE = $(enet_example_utils_baremetal_PATH)
export enet_example_utils_baremetal_SOCLIST = $(drvenet_SOCLIST)
export enet_example_utils_baremetal_$(SOC)_CORELIST = $(drvenet_$(SOC)_CORELIST)
enet_LIB_LIST += enet_example_utils_baremetal

#
# ENET NIMU example (TI-RTOS)
#
export enet_nimu_example_COMP_LIST = enet_nimu_example
enet_nimu_example_RELPATH = ti/drv/enet/examples/enet_nimu_example
enet_nimu_example_PATH = $(PDK_ENET_COMP_PATH)/examples/enet_nimu_example
export enet_nimu_example_BOARD_DEPENDENCY = yes
export enet_nimu_example_CORE_DEPENDENCY = yes
export enet_nimu_example_XDC_CONFIGURO = yes
enet_nimu_example_PKG_LIST = enet_nimu_example
enet_nimu_example_INCLUDE = $(enet_nimu_example_PATH)
export enet_nimu_example_BOARDLIST = $(drvenet_BOARDLIST)
export enet_nimu_example_$(SOC)_CORELIST = $(drvenet_$(SOC)_CORELIST)
export enet_nimu_example_SBL_APPIMAGEGEN = yes
enet_EXAMPLE_LIST += enet_nimu_example

#
# Loopback test (TI-RTOS)
#
export enet_loopback_test_COMP_LIST = enet_loopback_test
enet_loopback_test_RELPATH = ti/drv/enet/examples/enet_loopback_test
enet_loopback_test_PATH = $(PDK_ENET_COMP_PATH)/examples/enet_loopback_test
export enet_loopback_test_BOARD_DEPENDENCY = yes
export enet_loopback_test_CORE_DEPENDENCY = yes
export enet_loopback_test_XDC_CONFIGURO = yes
enet_loopback_test_PKG_LIST = enet_loopback_test
enet_loopback_test_INCLUDE = $(enet_loopback_test_PATH)
export enet_loopback_test_BOARDLIST = $(drvenet_BOARDLIST)
export enet_loopback_test_$(SOC)_CORELIST = $(drvenet_$(SOC)_CORELIST)
export enet_loopback_test_SBL_APPIMAGEGEN = yes
enet_EXAMPLE_LIST += enet_loopback_test

#
# Unit test (TI-RTOS)
#
-include $(PDK_ENET_COMP_PATH)/unit_test/enet_ut_component.mk
ifneq ($(enet_ut_LIB_LIST),)
  enet_LIB_LIST += $(enet_ut_LIB_LIST)
endif
ifneq ($(enet_ut_EXAMPLE_LIST),)
  enet_EXAMPLE_LIST += $(enet_ut_EXAMPLE_LIST)
endif

ifeq ($(BUILD_OS_TYPE), qnx)
enet_LIB_LIST = enet enetsoc enetphy
endif

export enet_LIB_LIST
export enet_EXAMPLE_LIST
export drvenet_LIB_LIST = $(enet_LIB_LIST)
export drvenet_EXAMPLE_LIST = $(enet_EXAMPLE_LIST)

# Enable asserts and prints
ENET_CFLAGS =
ENET_CFLAGS += -DENET_CFG_ASSERT=1
#ENET_CFLAGS += -DENET_CFG_USE_STD_ASSERT
ENET_CFLAGS += -DENET_CFG_PRINT_ENABLE
ENET_CFLAGS += -DUART_ENABLED

# Trace level per build profile:
# 0 - None
# 1 - Error
# 2 - Warning
# 3 - Info
# 4 - Debug
# 5 - Verbose
ifeq ($(BUILD_PROFILE),debug)
    ENET_CFLAGS += -DENET_CFG_TRACE_LEVEL=4
    ENET_CFLAGS += -DENET_CFG_DEV_ERROR=1
    ENET_CFLAGS += -DENETDMA_INSTRUMENTATION_ENABLED
    ENET_CFLAGS += -DENETCPTS_INSTRUMENTATION_ENABLED
else
    ENET_CFLAGS += -DENET_CFG_TRACE_LEVEL=3
endif

ifeq ($(BUILD_OS_TYPE), qnx)
ENET_CFLAGS =
# Enable asserts and prints
ENET_CFLAGS += -DENET_CFG_ASSERT_ENABLE
ENET_CFLAGS += -DENET_CFG_PRINT_ENABLE
ENET_CFLAGS += -DENET_CFG_TRACE_LEVEL=5
//ENET_CFLAGS += -DENET_CFG_DEV_ERROR=1
endif

ifeq ($(SOC),$(filter $(SOC), j721e am65xx))
ifeq ($(CORE),$(filter $(CORE), mcu1_0 mpu1_0))
    ENET_CFLAGS += -DENET_ENABLE_ICSSG
    export ENET_ENABLE_ICSSG=yes
endif
endif

export ENET_CFLAGS

enet_component_make_include := 1
endif
